<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : GetDataProcessLib.xsl

Notes       :

Open Issues :

$revision_history$
11-aug-2003   Steven Davelaar
  2.10        Added support for maxNumberQueryHits in FindDataObjectSet
28-jul-2003   Steven Davelaar
  2.9         Added support for queryType in non-lookup GetDataObjectSet
30-jun-2003   STEVEN DAVELAAR
  2.8         Added support for queryType in FindDataObjectSet
23-may-2003   STEVEN DAVELAAR
  2.7         Added childAttribute argument to GetChildDataObjectSet
17-apr-2003   STEVEN DAVELAAR
  2.6         cleaned up revision history
16-apr-2003   STEVEN DAVELAAR
  2.4         Added support for queryType in lookupGetDataObjectSet
17-mrt-2003   STEVEN DAVELAAR
  2.3         .
10-jun-2002   Peter Ebell
  2.2         Added constructor argument 'updateable' to BrowseDataSourceSet
14-mei-2002   Peter Ebell
  2.1         Pre-Prod release
14-jan-2002   STEVEN DAVELAAR
  1.3         Changed references to clex.trial classes to clex
14-jan-2002   Peter Ebell
  1.2         Added explicit parameter passing between templates for compatibility with 
              JDeveloper 9i
02-nov-2001   STEVEN DAVELAAR
  1.1         Removed hardcoded prefix 'oracle.' from package and resource name
25-sep-2001   STEVEN DAVELAAR
  1.0         initial creation

17-mar-2003   Steven Davelaar
  2.3         
10-jun-2002   Peter Ebell
  2.2         Added constructor argument 'updateable' to BrowseDataObjectSet
14-may-2002   Peter Ebell
  2.1         Pre-Prod release
28-mrt-2002   Peter Ebell
  2.0         Rewritten for JHeadstart 9i. Deleted obsolete revision history.

*******************************************************************************
$add_to_revision_history
$
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="xml" indent="yes" />


  <!-- get dataObject set process -->
  <xsl:template name="getDataObjectSet">
      <xsl:param name="processName"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="userErrorProcess"/>
      <xsl:param name="dataObjectNotFoundProcess"/>
      <xsl:param name="resultKey"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

  <!-- First set variables that determine number and sequence of
       constructor arguments
       Note: only set rangeSize if useTableRange is set to true -->
    <xsl:variable name="rangeSize">
      <xsl:choose>
        <xsl:when test="@useTableRange='true'">
          <xsl:value-of select="@tableRangeSize"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="newRows">
      <xsl:value-of select="@newRows"/>
    </xsl:variable>
    <!-- should dataObject set be updateable? -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>  

    <xsl:choose>
      <xsl:when test="@queryType!=''">
        <xsl:call-template name="getDataObjectSetWithQueryType">
            <xsl:with-param name="processName" select="$processName"/>
            <xsl:with-param name="resultKey" select="$resultKey"/>
        </xsl:call-template>    
      </xsl:when>
      <xsl:otherwise>
        <process>
          <xsl:attribute name="name">
            <xsl:value-of select="$processName"/>
          </xsl:attribute>
          <!-- name of get dataObject set process -->
          <xsl:attribute name="class">
            <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.GetDataObjectSet'"/>
          </xsl:attribute>
          <xsl:attribute name="end">false</xsl:attribute>
          <!-- only add updateable argument at this position if
               ranges are used. If no ranges are used this argument is
               the third in the constructor -->
          <xsl:choose>
            <xsl:when test="$rangeSize!=''">
              <!-- is DataObjectSet updateable? -->
              <xsl:comment> is DataObjectSet updateable? </xsl:comment>
              <argument type="Boolean">
                <xsl:attribute name="value">
                  <xsl:value-of select="$updateable"/>
                </xsl:attribute>
              </argument>
            </xsl:when>
          </xsl:choose>
          <!-- fully qualified dataObject interface class name -->
          <xsl:comment> name of dataObject interface class </xsl:comment>
          <argument type="String">
            <xsl:attribute name="value">
              <xsl:value-of select="concat($modelBase,@dataObject)"/>
            </xsl:attribute>
          </argument>
          <!-- process result select key of DataObjectSet -->
          <xsl:comment> key used to store result DataObjectSet in infoTable </xsl:comment>
          <argument type="String">
            <xsl:attribute name="value">
              <xsl:value-of select="$resultKey"/>
            </xsl:attribute>
          </argument>
          <!-- only add updateable argument at this position if
               no ranges are used. With ranges this argument is
               the first in the constructor -->
          <xsl:choose>
            <xsl:when test="$rangeSize=''">
              <!-- is DataObjectSet updateable? -->
              <xsl:comment> is DataObjectSet updateable? </xsl:comment>
              <argument type="Boolean">
                <xsl:attribute name="value">
                  <xsl:value-of select="$updateable"/>
                </xsl:attribute>
              </argument>
            </xsl:when>
          </xsl:choose>
          <!-- Add rangesize or new rows when set
                Note: ranges and new rows are not supported in combination -->
          <xsl:choose>
            <xsl:when test="$rangeSize!=''">
              <xsl:comment> range size </xsl:comment>
              <argument type="Integer">
                <xsl:attribute name="value">
                  <xsl:value-of select="$rangeSize"/>
                </xsl:attribute>
              </argument>
            </xsl:when>
            <xsl:when test="$newRows!='' and not(@insert='false')">
              <xsl:comment> number of new rows (empty lines) displayed </xsl:comment>
              <argument type="Integer">
                <xsl:attribute name="value">
                  <xsl:value-of select="$newRows"/>
                </xsl:attribute>
              </argument>
            </xsl:when>
          </xsl:choose>

        </process>

      </xsl:otherwise>
    </xsl:choose>

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$dataObjectNotFoundProcess"/>
    </xsl:call-template>

  </xsl:template> <!-- end getDataSOurceSet template-->

  <!-- get dataObject set with query type process -->
  <xsl:template name="getDataObjectSetWithQueryType">
      <xsl:param name="processName"/>
      <xsl:param name="resultKey"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

  <!-- First set variables that determine number and sequence of
       constructor arguments
       Note: only set rangeSize if useTableRange is set to true -->
    <xsl:variable name="rangeSize">
      <xsl:choose>
        <xsl:when test="@useTableRange='true'">
          <xsl:value-of select="@tableRangeSize"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="newRows">
      <xsl:choose>
        <xsl:when test="@newRows!=''">
          <xsl:value-of select="@newRows"/>
        </xsl:when>
        <xsl:otherwise>0</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!-- should dataObject set be updateable? -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>  

    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!-- name of get dataObject set process -->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.GetDataObjectSet'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>


      <!-- only add updateable argument at this position if
           ranges are used. If no ranges are used this argument is
           the third in the constructor -->

      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> name of dataObject interface class </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($modelBase,@dataObject)"/>
        </xsl:attribute>
      </argument>
      <xsl:comment> InfoTable attributes that are used as query type arguments </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="@queryTypeArguments"/>
        </xsl:attribute>
      </argument>
      <xsl:comment> use attribute name as key? </xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">false</xsl:attribute>
      </argument>
      <xsl:comment> query Type </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="@queryType"/>
        </xsl:attribute>
      </argument>
      <!-- process result select key of DataObjectSet -->
      <xsl:comment> key used to store result DataObjectSet in infoTable </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </argument>
           
      <xsl:choose>
        <xsl:when test="$rangeSize!=''">
          <xsl:comment> range size </xsl:comment>
          <argument type="Integer">
            <xsl:attribute name="value">
              <xsl:value-of select="$rangeSize"/>
            </xsl:attribute>
          </argument>
          <!-- is DataObjectSet updateable? -->
          <xsl:comment> is DataObjectSet updateable? </xsl:comment>
          <argument type="Boolean">
            <xsl:attribute name="value">
              <xsl:value-of select="$updateable"/>
            </xsl:attribute>
          </argument>
        </xsl:when>
        <xsl:otherwise>
          <!-- is DataObjectSet updateable? -->
          <xsl:comment> is DataObjectSet updateable? </xsl:comment>
          <argument type="Boolean">
            <xsl:attribute name="value">
              <xsl:value-of select="$updateable"/>
            </xsl:attribute>
          </argument>
          <xsl:comment> number of new rows (empty lines) displayed </xsl:comment>
          <argument type="Integer">
            <xsl:attribute name="value">
              <xsl:value-of select="$newRows"/>
            </xsl:attribute>
          </argument>
          <xsl:comment> show new row (empty line) at top? </xsl:comment>
          <argument type="Boolean">
            <xsl:attribute name="value">false</xsl:attribute>
          </argument>
        
        </xsl:otherwise>
      </xsl:choose>


    </process>

    <!-- transitions are added in getDataObjectSet template!-->

  </xsl:template> <!-- end getDataObjectSetWithQueryType template-->


  <!-- get lookup dataObject set process -->
  <xsl:template name="getLookupDataObjectSet">
      <xsl:param name="processName"
             select="concat('Get',@name)"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="userErrorProcess"/>
      <xsl:param name="dataObjectNotFoundProcess"/>
      <xsl:param name="resultKey"
             select="@name"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!-- name of get dataObject set process -->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.GetDataObjectSet'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>

      <!-- fully qualified lookup dataObject interface class name -->
      <xsl:comment> name of dataObject interface class </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($modelBase,@dataObject)"/>
        </xsl:attribute>
      </argument>


      <!-- check for queryType and add additional arguments when set  -->
      <xsl:if test="@queryType!=''">
        <xsl:comment> InfoTable attributes that are used as query type arguments </xsl:comment>
        <argument type="String">
          <xsl:attribute name="value">
            <xsl:value-of select="@queryTypeArguments"/>
          </xsl:attribute>
        </argument>
        <xsl:comment> use attribute name as key? </xsl:comment>
        <argument type="Boolean">
          <xsl:attribute name="value">false</xsl:attribute>
        </argument>
        <xsl:comment> query Type </xsl:comment>
        <argument type="String">
          <xsl:attribute name="value">
            <xsl:value-of select="@queryType"/>
          </xsl:attribute>
        </argument>
      </xsl:if>

      <!-- process result select key of DataObjectSet
           Name of Lookup is used as select key -->
      <xsl:comment> key used to store result DataObjectSet in infoTable </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </argument>

      <xsl:if test="@queryType!=''">
        <xsl:comment> is set updatable? </xsl:comment>
        <argument type="Boolean">
          <xsl:attribute name="value">false</xsl:attribute>
        </argument>
      </xsl:if>

      <!-- Lookup dataObject sets always have one new row at top -->
      <xsl:comment> number of new rows (empty lines) </xsl:comment>
      <argument type="Integer">
        <xsl:attribute name="value">1</xsl:attribute>
      </argument>
      <xsl:comment> show new row (empty line) at top? </xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">true</xsl:attribute>
      </argument>
    </process>

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$dataObjectNotFoundProcess"/>
    </xsl:call-template>

  </xsl:template> <!-- end getLookupDataObjectSet template  -->

  <!-- get dataObject set process -->
  <xsl:template name="findDataObjectSet">
      <xsl:param name="processName"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="userErrorProcess"/>
      <xsl:param name="dataObjectNotFoundProcess"/>
      <xsl:param name="resultKey"/>
      <xsl:param name="findPage"/>
             
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

  <!-- First set variables that determine number and sequence of
       constructor arguments
       Note: only set rangeSize if useTableRange is set to true -->
    <xsl:variable name="rangeSize">
      <xsl:choose>
        <xsl:when test="@useTableRange='true'">
          <xsl:value-of select="@tableRangeSize"/>
        </xsl:when>
        <xsl:otherwise>0</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="newRows" select="@newRows"/>
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>  

    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!-- name of find dataObject set process -->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.FindDataObjectSet'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>
      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> name of dataObject interface class </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($modelBase,@dataObject)"/>
        </xsl:attribute>
      </argument>
      <!-- process result select key of DataObjectSet -->
      <xsl:comment> key used to store result DataObjectSet in infoTable </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </argument>
      <!-- file name of find page -->
      <xsl:comment> file name of find page </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$findPage"/>
        </xsl:attribute>
      </argument>
      <!-- Is the result set updateable? -->
      <xsl:comment> Is the result set updateable? </xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">
          <xsl:value-of select="$updateable"/>
        </xsl:attribute>  
      </argument>
      <!-- Only Add rangesize when set -->


      <xsl:choose>
      <!-- Note that constructors that combine new rows with queryType or max
           query hot rows do not exist!  This is not as bad as
          it seems. When you expect many rows to be returned and want to limit
          this number, ir makes a lot of sense to also use ranges -->


        <xsl:when test="@queryType!='' or @useTableRange='true' or @maxNumberQueryHits != ''">
          <xsl:comment> range size </xsl:comment>
          <argument type="Integer">
            <xsl:attribute name="value">
              <xsl:value-of select="$rangeSize"/>
            </xsl:attribute>
          </argument>

          <xsl:if test="@queryType!=''">
            <xsl:comment> queryType </xsl:comment>
            <argument type="String">
              <xsl:attribute name="value">
                <xsl:value-of select="@queryType"/>
              </xsl:attribute>
            </argument>
          </xsl:if>  
          
          <xsl:if test="@maxNumberQueryHits != ''">
            <xsl:comment> The maximum number of query hits </xsl:comment>
            <argument type="Integer">
              <xsl:attribute name="value">
                <xsl:value-of select="@maxNumberQueryHits"/>
              </xsl:attribute>
            </argument>
          </xsl:if>
        </xsl:when>
        
        <xsl:when test="$newRows!='' and not(@insert='false') and (@layoutStyle='table' or @layoutStyle='table-form')">
          <xsl:comment> number of new rows (empty lines) displayed </xsl:comment>
          <argument type="Integer">
            <xsl:attribute name="value">
              <xsl:value-of select="$newRows"/>
            </xsl:attribute>
          </argument>
          <xsl:comment> add new rows (empty lines) at top? </xsl:comment>
          <argument type="Boolean">
            <xsl:attribute name="value">false</xsl:attribute>
          </argument>
        </xsl:when>
      </xsl:choose>

      
    </process>

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$dataObjectNotFoundProcess"/>
    </xsl:call-template>

  </xsl:template> <!-- end findDataSOurceSet template-->


    <!-- get child dataObject set process -->
  <xsl:template name="getChildDataObjectSet">
      <xsl:param name="processName"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="userErrorProcess"/>
      <xsl:param name="dataObjectNotFoundProcess"/>
      <xsl:param name="masterKey"/>
      <xsl:param name="resultKey"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

    <xsl:variable name="rangeSize">
      <xsl:choose>
        <xsl:when test="@useTableRange='true'">
          <xsl:value-of select="@tableRangeSize"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <!-- Should dataObject set be updateable? -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>  
    
    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!-- name of get dataObject set process -->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.GetChildDataObjectSet'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>
      <!-- add constructor arguments:

                          (String masterDataObjectName
                          ,String processParameterName
                          ,String childDataObjectName
                          ,String childAttribute
                          ,String processResultName
                          ,Boolean keepConsistent
                          ,Boolean isUpdatable
                          ,Integer emptyRows
                          ,Boolean emptyRowsAtTop)
      -->
      <!-- fully qualified master dataObject interface class name -->
      <xsl:comment> name of master dataObject interface </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($modelBase,../@dataObject)"/>
        </xsl:attribute>
      </argument>
      <!-- name under which master dataObject is stored in infoTabl -->
      <xsl:comment> select key of master dataObject in infoTable</xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$masterKey"/>
        </xsl:attribute>
      </argument>
      <!-- name of detail view dataObject name -->
      <xsl:comment> name of detail dataObject interface </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($modelBase,@dataObject)"/>
        </xsl:attribute>                              
      </argument>
      <!-- name of accessor attribute in master dataObject -->
      <xsl:comment> name of accessorAttribute in master dataObject </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="@childAttribute"/>
        </xsl:attribute>                              
      </argument>
      
      <!-- process result select key of ChildDataObjectSet -->
      <xsl:comment> select key of result child DataObjectSet in infoTable </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </argument>
      <!-- keep child DataObjectSet consistent?
           S. Davelaar, 09-aug-2001: set to false to
           work around bc4j bug that shows inserted record twice -->
      <xsl:comment> keep child DataObjectSet consistent? </xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">false</xsl:attribute>
      </argument>
      <!-- is child DataObjectSet updateable? -->
      <xsl:comment> is child DataObjectSet updateable? </xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">
          <xsl:value-of select="$updateable"/>
        </xsl:attribute>
      </argument>
      <!-- if rangeSize is specified, we set the ranbgeSize as
           arguments. Otherwise we set the numbe rof new rows and
           rows at top as ast twpo arguments (ranges are not supported
           in combination with new rows (inserts) -->
      <xsl:choose>
        <xsl:when test="$rangeSize!=''">
          <!-- table range size argument -->
          <xsl:comment> table range size </xsl:comment>
          <argument type="Integer">
            <xsl:attribute name="value">
              <xsl:value-of select="$rangeSize"/>
            </xsl:attribute>
          </argument>
        </xsl:when>
        <xsl:otherwise>
          <!-- number of empty rows -->
          <xsl:comment> number of new rows (i.e. empty lines) </xsl:comment>
          <argument type="Integer">
            <xsl:attribute name="value">
              <xsl:choose>
                <xsl:when test="@newRows!='' and not(@insert='false') and contains(@layoutStyle,'table')">
                  <xsl:value-of select="@newRows"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="'0'"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:attribute>
          </argument>
          <!-- empty rows at top? -->
          <xsl:comment> add new rows (empty lines) at top? </xsl:comment>
          <argument type="Boolean">
            <xsl:attribute name="value">false</xsl:attribute>
          </argument>
        </xsl:otherwise>
      </xsl:choose>
    </process>

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$dataObjectNotFoundProcess"/>
    </xsl:call-template>

  </xsl:template> <!-- end getChildDataObjectSet template-->


  <!-- Create Default DataObject process -->
  <xsl:template name="getDefaultDataObject">
    <xsl:param name="processName"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="dataObjectNotFoundProcess"/>
    <xsl:param name="resultKey"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

    <process name="{$processName}" class="oracle.jheadstart.controller.mvc.process.CreateDefaultDataObject" end="false">

      <!-- dataObject name -->
      <xsl:comment> name of dataObject interface </xsl:comment>
      <argument type="String" value="{concat($modelBase,@dataObject)}"/>

      <!-- process result select -->
      <xsl:comment> select key of result DataObject in infoTable </xsl:comment>
      <argument type="String" value="{$resultKey}"/>

      <!-- remove entries -->
      <xsl:variable name="removelist">
        <xsl:for-each select="Group[@samePage='true']">
          <xsl:copy-of select="concat(@name,'Set,')"/>
        </xsl:for-each>
      </xsl:variable>

      <xsl:comment> entries to be removed from the infoTable </xsl:comment>
      <argument type="String" value="{translate(concat($removelist,','),',,','')}"/>

    </process>

    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess" select="$dataObjectNotFoundProcess"/>
    </xsl:call-template>

  </xsl:template> <!-- end GetDefaultDataObject template-->


  <xsl:template name="getDataObjectTransitions">

    <xsl:param name="sourceProcess"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="dataObjectNotFoundProcess"/>

    <xsl:call-template name="persistentProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$sourceProcess"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
    </xsl:call-template>

    <!-- dataObjectNotFound condition -->
    <transition>
      <xsl:attribute name="src">
        <xsl:value-of select="$sourceProcess"/>
      </xsl:attribute>
      <xsl:attribute name="condition">dataObjectNotFound</xsl:attribute>
      <xsl:attribute name="dest">
        <xsl:value-of select="$dataObjectNotFoundProcess"/>
      </xsl:attribute>
    </transition>
  </xsl:template> <!-- end getDataObjectTransitions template-->

  <!-- get dataObject process ->
  <xsl:template name="getDataObject">
    <xsl:param name="processName"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="dataObjectNotFoundProcess"/>
    <xsl:param name="selectKey"
             select="concat(@dataObject,@identifier)"/>
    <xsl:param name="resultKey"
             select="@dataObject"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!- name of get dataObject process ->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.GetDataObject'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>
      <!- fully qualified dataObject interface class name ->
      <xsl:comment> name of dataObject interface class</xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($package
                                      ,'.dataObject.'
                                      ,@dataObject)"/>
        </xsl:attribute>
      </argument>
      <!- the primary key column of the DataObject used to get the object ->
      <xsl:comment> select key of the identifier (primary key) of the DataObject in infoTable</xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$selectKey"/>
        </xsl:attribute>
      </argument>
      <!- process result select key of DataObject ->
      <xsl:comment> select key of result DataObject in infoTable </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </argument>
      <!- is usage dataObject updatable  ->
      <xsl:comment> is DataObject updatable? </xsl:comment>
      <argument type="Boolean" value="true"/>

     </process>

    <!- call template to add process transitions ->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$dataObjectNotFoundProcess"/>
    </xsl:call-template>

  </xsl:template> <!- end getDataSOurce for edit template->
-->

  <!-- get DataObject from Set process -->
  <xsl:template name="getSelectedDataObject">
    <xsl:param name="dataObjectSetKey"/>
    <xsl:param name="dataObjectName"/>
    <xsl:param name="processName"/>
<!--  <xsl:param name="buttonName"/> -->
    <xsl:param name="resultKey"/>
    <xsl:param name="selectKey"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="dataObjectNotFoundProcess"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

    <!-- should dataObject set be updateable? -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>  

    <process name="{$processName}" class="oracle.jheadstart.controller.mvc.process.GetSelectedDataObject" end="false">

      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> name of dataObject set to get dataObject from</xsl:comment>
      <argument type="String" value="{$dataObjectSetKey}"/>

      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> name of dataObject interface class</xsl:comment>
      <argument type="String" value="{concat($modelBase,$dataObjectName)}"/>

      <xsl:if test="$selectKey!=''">  
        <!-- the primary key column of the DataObject used to get the object -->
        <xsl:comment> select key of the identifier (primary key) of the DataObject in infoTable</xsl:comment>
        <argument type="String" value="{$selectKey}"/>
      </xsl:if>         

      <!-- name of the button 
      <xsl:comment> name of the button </xsl:comment>
      <argument type="String" value="{$buttonName}"/>
      -->
      
      <!-- process result select key of DataObject -->
      <xsl:comment> select key of result DataObject in infoTable </xsl:comment>
      <argument type="String" value="{$resultKey}"/>

      <!-- is the dataObject updateable? -->
      <xsl:comment> is the DataObject updateable? </xsl:comment>
      <argument type="Boolean" value="{$updateable}"/>

     </process>

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess" select="$dataObjectNotFoundProcess"/>
    </xsl:call-template>

  </xsl:template> <!-- end get DataObject From Set-->



  <!-- Browse DataObject from Set process -->
  <xsl:template name="browseDataObjectSet">
    <xsl:param name="processName"/>
    <xsl:param name="dataObjectSetKey"/>
    <xsl:param name="buttonName"/>
    <xsl:param name="resultKey"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="dataObjectNotFoundProcess"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

    <!-- should dataObject set be updateable? -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable> 

    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!-- name of get dataObject process -->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.BrowseDataObjectSet'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>

      <!-- name of dataObject set to get dataObject from -->
      <xsl:comment> name of dataObject set to get dataObject from</xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$dataObjectSetKey"/>
        </xsl:attribute>
      </argument>

      <!-- name of the button -->
      <xsl:comment> infoTable select key of browse action (name of the browse button </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$buttonName"/>
        </xsl:attribute>
      </argument>

      <!-- process result select key of DataObject -->
      <xsl:comment> select key of result DataObject in infoTable </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </argument>
      
      <!-- Is dataObject updateable -->
      <xsl:comment>Should the result dataObject be updateable?</xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">
          <xsl:value-of select="$updateable"/>
        </xsl:attribute>
      </argument>
    </process>

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess" select="$dataObjectNotFoundProcess"/>
    </xsl:call-template>


  </xsl:template> <!-- end Browse DataObject Set-->









  <!-- get selected table row for edit process -->
  <xsl:template name="getSelectedRowForEdit">
      <xsl:param name="processName"
        select="concat('Get',@dataObject,'ForEdit')"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="tableName"
             select="concat(@dataObject,'Set')"/>
      <xsl:param name="tableDataSetKey"
             select="concat(@dataObject,'Set')"/>
      <xsl:param name="resultKey"
             select="@dataObject"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!-- name of get dataObject process -->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.GetSelectedRow'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>
      <!-- table name -->
      <xsl:comment> table name </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$tableName"/>
        </xsl:attribute>
      </argument>
      <!-- dataObject set name used to populate table -->
      <xsl:comment> key of table dataObject set in infoTable</xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$tableDataSetKey"/>
        </xsl:attribute>
      </argument>
      <!-- process result select key of selected DataObject -->
      <xsl:comment> key of selected dataObject in infoTable </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </argument>
     </process>

    <!-- call template to add process transitions -->
    <xsl:call-template name="genericProcessTransitions">
      <xsl:with-param name="sourceProcess"
        select="$processName"/>
      <xsl:with-param name="successProcess"
        select="$successProcess"/>
    </xsl:call-template>

  </xsl:template> <!-- end selected row for edit template-->

</xsl:stylesheet>
