<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : JSPGlobalHeaders.xsl

Notes       :

$revision_history$
31-aug-2003   Aard-Jan van Kesteren
  1.10        - Only initialize "user" once
              - Generate the actual name of a tab in hasAccess( instead of an hardcoded @name
25-aug-2003   Peter Ebell
  1.9         Hide unauthorized groups if hideUnauthorizedGroupTabs property is set
14-aug-2003   Aard-Jan van Kesteren
  1.8         
07-aug-2003   Steven Davelaar
  1.7         Convert nls keys to uppercase
05-aug-2003   Aard-Jan van Kesteren
  1.6         Added nls
29-jul-2003   Aard-Jan van Kesteren
  1.5         - Use go2 instead of jhsGoto
28-jul-2003   Steven Davelaar
  1.4         Use tabName property first as label
23-jul-2003   Aard-Jan van Kesteren
  1.3         - Solved bug 3062844. Rename js function goto to jhsGoto
14-mrt-2003   Aard-Jan van Kesteren
  1.0         Initial creation

*******************************************************************************
$add_to_revision_history
$
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  xmlns:string="http://www.oracle.com/XSL/Transform/java/java.lang.String"      
  exclude-result-prefixes="utils string"
>
  <xsl:import href="JSPLibrary.xsl"/>
  
  <xsl:output method="xml" indent="yes" />
  
  <xsl:param name="groupName"></xsl:param>

  <xsl:variable name="hideUnauthorizedGroupTabs" select="Service/@hideUnauthorizedGroupTabs"/>

  <xsl:variable name="groupIncludeFile" select="Service/@groupIncludeFile"/>

  <xsl:variable name="globalHeaderName" select="concat($groupName,'GlobalHeader')"/>


  <xsl:template match="/">
  
<xsl:text disable-output-escaping="yes">
<![CDATA[<%@ taglib uri="/WEB-INF/struts-bean.tld" prefix="bean" %> 
<%@ page import="oracle.jheadstart.model.JhsUser" %>
<%@ page import="oracle.jheadstart.JhsConstants" %>

<bean:parameter id="current" name="current"/>]]>
</xsl:text>
<xsl:if test="$hideUnauthorizedGroupTabs='true'">
<xsl:text disable-output-escaping="yes">
<![CDATA[
<% JhsUser user = (JhsUser)session.getAttribute(JhsConstants.JHS_USER); %>
]]>  
</xsl:text>
</xsl:if>
  
  <table class="subtabBar" cellpadding="3" cellspacing="0">
    <tr>      
            <xsl:call-template name="masterGroupLink"/>
      <td class="empty" width="100%"/>            
    </tr>
  </table>
  </xsl:template>

  <xsl:template name="masterGroupLink">
   <xsl:for-each select="Service/Group[@name=$groupName]">

    <xsl:variable name="masterTargetProcess" select="'groupPage'"/>
<!--<xsl:text>edit</xsl:text>
      <xsl:value-of select="@name"/>
      <xsl:text>Page</xsl:text>
    </xsl:variable>
-->    
    <xsl:call-template name="groupLink">
      <xsl:with-param name="targetProcess" select="$masterTargetProcess"/>
    </xsl:call-template>
    
    <xsl:call-template name="detailGroupLinks"/>

   </xsl:for-each>
  </xsl:template>

  <xsl:template name="detailGroupLinks">
   <xsl:for-each select="Group[@samePage='false']">
    <xsl:call-template name="groupLink">
       <xsl:with-param name="targetProcess" select="''"/>
    </xsl:call-template>
   </xsl:for-each>
  </xsl:template>

  <xsl:template name="groupLink">
   <xsl:param name="targetProcess"/>

  <!-- Hide groups that the user is not authorized for -->
  <!-- Hide groups that the user is not authorized for -->
  <xsl:if test="$hideUnauthorizedGroupTabs='true'">
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>  </xsl:text>  
    <xsl:value-of select="concat('jspO if (user != null &amp;&amp; user.hasAccess(``', @name, '``, session.getId())) { jspC')"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>    </xsl:text>  
  </xsl:if>

   <td nowrap="nowrap" class="{concat('jspO= (current.equals(``', @name, '``)  ? ``subtabCurrent`` : `` ``) jspC')}">
    <xsl:text>&amp;nbsp;</xsl:text>
    <a>
      <xsl:attribute name="href">
        <xsl:text>javascript:go2(['targetGroup','targetCondition'],['</xsl:text>
        <xsl:value-of select="@name"/>
        <xsl:text>','</xsl:text>
        <xsl:if test="$targetProcess != ''">
          <xsl:value-of select="$targetProcess"/>
        </xsl:if>
        <xsl:text>']);</xsl:text>
      </xsl:attribute>

   <xsl:variable name="displayLabel">
      <xsl:choose>
          <xsl:when test="@tabName!=''">
            <xsl:value-of select="@tabName"/>
          </xsl:when>
        <xsl:when test="@displayTitle!=''">
          <xsl:value-of select="@displayTitle"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@dataObject"/>
        </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>

      <xsl:call-template name="nlsTextAttribute">
        <xsl:with-param name="key">
          <xsl:value-of select="concat(string:toUpperCase(@name),'_LEVEL_2_TAB')"/>
        </xsl:with-param>
        <xsl:with-param name="value">
          <xsl:value-of select="$displayLabel"/>
        </xsl:with-param>
        <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
      </xsl:call-template>
  
    </a>
    <xsl:text>&amp;nbsp;</xsl:text>      
    </td>

  <xsl:if test="$hideUnauthorizedGroupTabs='true'">
    <xsl:text>&#xa;</xsl:text>  
    <xsl:text>  </xsl:text>
    <xsl:value-of select="'jspO } jspC'"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>  </xsl:text>
  </xsl:if>

    
  </xsl:template>

</xsl:stylesheet>
