<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : JSPInput.xsl

Notes       :

$revision_history$
21-jul-2004   Lucas Jellema (AMIS)
  1.20AMIS1.8 - support for ON_CLICK, ON_DBLCLICK, ON_FOCUS, ON_CHANGE
19-jul-2004   Lucas Jellema (AMIS)
  1.20AMIS1.7 Support ON_BLUR, RADIO_MATRIX_SIZE
19-jul-2004   Lucas Jellema (AMIS)
  1.20AMIS1.6 Make HINT_TEXT and TITLE NLS-enabled (include in ResourceBundle) 
19-jul-2004   Lucas Jellema (AMIS)
  1.20AMIS1.5 Support detailsLink tableDisplayType value: requires detailsLink.jsp
19-jul-2004   Lucas Jellema (AMIS)
  1.20AMIS1.4 Support HTML TITLE attribute (based on HINT property) (see e.g. http://www.netmechanic.com/news/vol6/html_no1.htm)
19-jul-2004   Lucas Jellema (AMIS)
  1.20AMIS1.3 Support NULL_VALUE_LABEL, RADIO_STYLE
19-jul-2004   Lucas Jellema (AMIS)
  1.20AMIS1.2 Support PROMPT_STYLE_CLASS
18-jul-2004   Lucas Jellema (AMIS)
  1.20AMIS1.1 Support RADIO GROUP (requires JSTL to support Radio for DOMAIN)
17-jul-2004   Lucas Jellema (AMIS)
  1.20AMIS1.0 Support STYLE_CLASS, STYLE_ID
26-apr-2004   Steven Davelaar
  1.20        Do not make LOV field disabled if baseValueAttribute is equal to baseDisplayAttribute.
27-jan-2004   Peter Ebell
  1.19        Also apply length and maxlength on displayfields.
19-dec-2003   Jan Kettenis
  1.18        Renamed baseDescriptorItem to baseDescriptorItems to reflect that more than one item can be passed
28-nov-2003   Jan Kettenis
  1.17        Reordered, renamed LoV parameters for call to lovJsp.jsp.
24-nov-2003   Steven Davelaar
  1.16        Generate logic:present around image and fileDownload
19-nov-2003   Steven Davelaar
  1.15        Do not set disabled attribute when display type is
              image or fileDownload (causing error when running JAG)
14-nov-2003   Steven Davelaar
  1.14        Added fileUpload, fileDownload and image
05-nov-2003   Aard-Jan van Kesteren
  1.13        Rewrote IsInserting.booleanValue() occurences
26-oct-2003   Aard-Jan van Kesteren
  1.12        Rewrote Boolean.TRUE and Boolean.FALSE occurences
20-okt-2003   Steven Davelaar
  1.11        Prefix property name of find items with "Find"
15-aug-2003   Aard-Jan van Kesteren
  1.10        Added the format of a date or timedate as paremeter to the calender popup
              jspinclude
14-aug-2003   Aard-Jan van Kesteren
  1.7
14-aug-2003   Aard-Jan van Kesteren
  1.6         Dropped parameter formName in jsp include calls editorPopup and calendarPopup
14-aug-2003   Aard-Jan van Kesteren
  1.5         Display readOnly fields as disabled html instead of plain text
12-aug-2003   Aard-Jan van Kesteren
  1.4         Solved bug 3092189. Added suport for datetimeField
01-aug-2003   Aard-Jan van Kesteren
  1.3         In the lovPopup jsp include, removed the formName and isFindPage arguments
14-mrt-2003   Aard-Jan van Kesteren
  1.0         Initial creation

*******************************************************************************
$add_to_revision_history
$
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  xmlns:string="http://www.oracle.com/XSL/Transform/java/java.lang.String"
  exclude-result-prefixes="utils string"
>

  <xsl:output method="xml" indent="yes" />

  <!-- this template will write all required onBlur code; this may be specified
       in the ON_BLUR attribute or implicitly by setting the RESTRICT_CASE property
       to either lower or upper; should we include code for mandatory properties??
    -->
       
  <xsl:template name="onBlur">
     <xsl:param name="onBlur" />
     <xsl:param name="restrictCase" />
     <xsl:if test="$onBlur !='' or $restrictCase='lower' or $restrictCase='upper' " >
       <xsl:attribute name="onblur">
          <xsl:if test="$restrictCase='lower' " >this.value=this.value.toLowerCase();</xsl:if>
          <xsl:if test="$restrictCase='upper' " >this.value=this.value.toUpperCase();</xsl:if>
          <xsl:if test="$restrictCase='initCap' " >this.value=this.value.substring(0,1).toUpperCase()+this.value.substring(1).toLowerCase();</xsl:if>
          <xsl:value-of select="$onBlur" />
       </xsl:attribute>
    </xsl:if>
  </xsl:template> <!-- onBlur -->

  <xsl:template name="onFocus">
     <xsl:param name="onFocus" />
     <xsl:if test="$onFocus !=''"  >
       <xsl:attribute name="onfocus">
          <xsl:value-of select="$onFocus" />
       </xsl:attribute>
    </xsl:if>
  </xsl:template> <!-- onFocus -->


  <xsl:template name="onChange">
     <xsl:param name="onChange" />
     <xsl:if test="$onChange !=''"  >
       <xsl:attribute name="onchange">
          <xsl:value-of select="$onChange" />
       </xsl:attribute>
    </xsl:if>
  </xsl:template> <!-- onChange -->


  <xsl:template name="onClick">
     <xsl:param name="onClick" />
     <xsl:if test="$onClick !='' " >
       <xsl:attribute name="onclick">
          <xsl:value-of select="$onClick" />
       </xsl:attribute>
    </xsl:if>
  </xsl:template> <!-- onClick -->
  
  <xsl:template name="onDblClick">
     <xsl:param name="onDblClick" />
     <xsl:if test="$onDblClick !=''"  >
       <xsl:attribute name="ondblclick">
          <xsl:value-of select="$onDblClick" />
       </xsl:attribute>
    </xsl:if>
  </xsl:template> <!-- onDblClick -->


  <xsl:template name="htmlHidden">
    <xsl:param name="name"/>
    <xsl:param name="value">noValue</xsl:param>
    <input type="hidden">
      <xsl:attribute name="name">
        <xsl:value-of select="$name"/>
      </xsl:attribute>
      <xsl:if test="$value != 'noValue'">
        <xsl:attribute name="value">
          <xsl:value-of select="$value"/>
        </xsl:attribute>
      </xsl:if>
    </input>
  </xsl:template>


  <xsl:template name="image">
    <xsl:param name="name"/>
    <xsl:param name="propertyFinal"/>
    <xsl:param name="size"/>
    <xsl:param name="displayHeight"/>
    <xsl:param name="withIndex"/>
    <xsl:param name="groupName"/>
    <xsl:param name="property"/>

    <logic___present>
      <xsl:attribute name="name"><xsl:value-of select="$name"/></xsl:attribute>
      <xsl:attribute name="property">
         <xsl:value-of select="$propertyFinal"/>
      </xsl:attribute>

      <html___img>
        <xsl:if test="$size != ''">
          <xsl:attribute name="width"><xsl:value-of select="$size"/></xsl:attribute>
        </xsl:if>
        <xsl:if test="$displayHeight != 1">
          <xsl:attribute name="height"><xsl:value-of select="$displayHeight"/></xsl:attribute>
        </xsl:if>

        <xsl:attribute name="src">
          <xsl:choose>
            <xsl:when test="$withIndex = 'true'">
              <xsl:value-of select="concat('jspO= ``DownloadFile.do?dataObjectSet=', $groupName, 'Set&amp;attribute=',$property, '&amp;index=``+index jspC')"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="concat('DownloadFile.do?dataObject=',$groupName,'&amp;attribute=',$property)"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>

      </html___img>
    </logic___present>

  </xsl:template>

  <xsl:template name="fileDownload">
    <xsl:param name="name"/>
    <xsl:param name="propertyFinal"/>
    <xsl:param name="withIndex"/>
    <xsl:param name="groupName"/>
    <xsl:param name="property"/>
    <xsl:param name="prompt"/>

    <logic___present>
      <xsl:attribute name="name"><xsl:value-of select="$name"/></xsl:attribute>
      <xsl:attribute name="property">
         <xsl:value-of select="$propertyFinal"/>
      </xsl:attribute>

      <html___link>
        <xsl:attribute name="href">
          <xsl:choose>
            <xsl:when test="$withIndex = 'true'">
              <xsl:value-of select="concat('jspO= ``DownloadFile.do?dataObjectSet=', $groupName, 'Set&amp;attribute=',$property,'&amp;attachment=',$property, '&amp;index=``+index jspC')"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="concat('DownloadFile.do?dataObject=',$groupName,'&amp;attribute=',$property,'&amp;attachment=',$property)"/>
            </xsl:otherwise>
          </xsl:choose>

        </xsl:attribute>
        <xsl:value-of select="$prompt"/>
      </html___link>
    </logic___present>

  </xsl:template>

  <xsl:template name="htmlText">
    <xsl:param name="name"/>
    <xsl:param name="property"/>
    <xsl:param name="size"/>
    <xsl:param name="table">tr</xsl:param>
    <xsl:param name="prompt"><xsl:value-of select="$property"/></xsl:param>
    <xsl:param name="withIndex">false</xsl:param>
    <xsl:param name="required">false</xsl:param>
    <xsl:param name="hintText"/>
    <xsl:param name="group"/>
    <xsl:param name="formName"/>
    <xsl:param name="displayType"/>
    <xsl:param name="updateable">true</xsl:param>
    <xsl:param name="isReadOnly">false</xsl:param>
    <xsl:param name="showHyperlink">false</xsl:param>
    <xsl:param name="isFindElement">false</xsl:param>
    <xsl:param name="optionsName"/>
    <xsl:param name="optionsProperty"/>
    <xsl:param name="optionsLabelName"/>
    <xsl:param name="optionsLabelProperty"/>
    <xsl:param name="generatePropertyAttribute">false</xsl:param>
    <xsl:param name="domain"/>
    <xsl:param name="maximumLength"/>
    <xsl:param name="displayHeight"/>
    <xsl:param name="displayClass"/>
    <xsl:param name="lookupLovValues"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="listValidator"/>
    <!-- LJ -->
    <xsl:param name="styleClass" />
    <xsl:param name="styleId"/>
    <xsl:param name="promptStyleClass" />
    <xsl:param name="nullValueLabel" />
    <xsl:param name="radioStyle" />
    <xsl:param name="radioMatrixSize" />
    <xsl:param name="onBlur" />
    <xsl:param name="onFocus" />
    <xsl:param name="onChange" />
    <xsl:param name="onClick" />
    <xsl:param name="onDblClick" />
    <xsl:param name="restrictCase" />
    <!-- end LJ -->
    <xsl:variable name="propertyFinal">
      <xsl:choose>
        <xsl:when test="$withIndex = 'true'">
          <xsl:value-of select="concat('jspO= ``', $group/@name, $property, '[``+index+``]`` jspC')"/>
        </xsl:when>
        <xsl:when test="$isFindElement = 'true'">
          <xsl:value-of select="concat('Find',$group/@name, $property)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat($group/@name, $property)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="isInsertingProperty">
      <xsl:choose>
        <xsl:when test="$withIndex = 'true'">
          <xsl:value-of select="concat('jspO= ``', $group/@name, 'IsInserting', '[``+index+``]`` jspC')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat($group/@name, 'IsInserting')"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <xsl:choose>
      <xsl:when test="$table='no' or $displayType='hidden'">

        <xsl:variable name="elementName">

          <xsl:choose>
            <xsl:when test="$displayType = 'hidden'">
              <xsl:value-of select="'html___hidden'"/>
            </xsl:when>
            <xsl:when test="$displayType = 'secret'">
              <xsl:value-of select="'html___password'"/>
            </xsl:when>
            <xsl:when test="$displayType = 'choice'">
              <xsl:value-of select="'html___select'"/>
            </xsl:when>
            <!-- LJ -->
            <xsl:when test="$displayType = 'radio'">
              <xsl:value-of select="'html___radio'"/>
            </xsl:when>
            <!-- LJ -->
            <xsl:when test="$displayType = 'checkBox'">
              <xsl:value-of select="'html___checkbox'"/>
            </xsl:when>
            <xsl:when test="$displayType = 'fileUpload'">
              <xsl:value-of select="'html___file'"/>
            </xsl:when>
            <xsl:when test="$displayType = 'fileDownload'">
              <xsl:value-of select="'html___link'"/>
            </xsl:when>
            <xsl:when test="$displayType = 'image'">
              <xsl:value-of select="'html___img'"/>
            </xsl:when>
            <xsl:when test="($displayType = 'textInput' or $displayType = 'lov')
                            and $displayHeight != 1">
              <xsl:value-of select="'html___textarea'"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="'html___text'"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <xsl:if test="$generatePropertyAttribute='true'">
          <xsl:attribute name="property">
            <xsl:value-of select="$propertyFinal"/>
          </xsl:attribute>
        </xsl:if>

        <xsl:if test="$required = 'true' and $displayType != 'hidden' and $displayType != 'displayField'">
          <font class="required">*</font>
        </xsl:if>

        <!-- if display type is image or file download, we only render it when
             the attribute is not null. To do this we encapsultae the
             element with logic:present tags. -->

        <xsl:choose>
          <xsl:when test="$displayType = 'image'">

            <xsl:call-template name="image">
              <xsl:with-param name="name"  select="$name"/>
              <xsl:with-param name="propertyFinal" select = "$propertyFinal"/>
              <xsl:with-param name="size" select = "$size"/>
              <xsl:with-param name="displayHeight" select="$displayHeight"/>
              <xsl:with-param name="withIndex" select = "$withIndex"/>
              <xsl:with-param name="groupName" select="$group/@name"/>
              <xsl:with-param name="property" select = "$property"/>
            </xsl:call-template>

          </xsl:when>
          <xsl:when test="$displayType = 'fileDownload'">
            <xsl:call-template name="fileDownload">
              <xsl:with-param name="name"  select="$name"/>
              <xsl:with-param name="propertyFinal" select = "$propertyFinal"/>
              <xsl:with-param name="withIndex" select = "$withIndex"/>
              <xsl:with-param name="groupName" select="$group/@name"/>
              <xsl:with-param name="property" select = "$property"/>
              <xsl:with-param name="prompt" select = "$prompt"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:when test="$displayType = 'hyperlink'">
            <a href="#" >
              <xsl:attribute name="onClick">
                 popupAndGo('###LT###bean___write name=`<xsl:value-of select="$name"/>` property=`<xsl:value-of select="$propertyFinal"/>` /###GT###');
              </xsl:attribute>
              <bean___write>
                <xsl:attribute name="name"><xsl:value-of select="$name"/></xsl:attribute>
                <xsl:attribute name="property"><xsl:value-of select="$propertyFinal"/></xsl:attribute>
              </bean___write>
            </a>
         <!-- 
         <a href="#" onClick="popupAndGo('<bean:write name="PublisherBean" property="PublisherWebsite" />');" >
          <bean:write name="PublisherBean" property="PublisherWebsite" />
          </a>
         <a href="#" onClick="popupAndGo('<bean:write name="PublisherBean" property="PublisherWebsite" />');" >
          <img src="images\hyperlink.jpg"  alt="<bean:write name="PublisherBean" property="PublisherWebsite" />"/>
          
          </a>

          --> 
          </xsl:when>  <!-- hyperlink -->
          <xsl:when test="$displayType = 'detailsLink'">
              <xsl:call-template name="jspInclude">
                <xsl:with-param name="page" select="'detailsLink.jsp'"/>
                <xsl:with-param name="param1Name" select="'rownumField'"/>
                <xsl:with-param name="param1Value" select="concat('document.', $group/@name, 'Bean.', $group/@name, 'Rownum')"/>
                <xsl:with-param name="param2Name" select="'index'"/>
                <xsl:with-param name="param2Value" select="'jspO= index jspC'"/>
                <xsl:with-param name="param3Name" select="'formName'"/>
                <xsl:with-param name="param3Value" select="$formName"/>
              </xsl:call-template>

              <bean___write>
                <xsl:attribute name="name"><xsl:value-of select="$name"/></xsl:attribute>
                <xsl:attribute name="property"><xsl:value-of select="$propertyFinal"/></xsl:attribute>
              </bean___write>
                              ###LT###/a###GT###
          
         <!-- 
         <a href="#" onClick="popupAndGo('<bean:write name="PublisherBean" property="PublisherWebsite" />');" >
          <bean:write name="PublisherBean" property="PublisherWebsite" />
          </a>
         <a href="#" onClick="popupAndGo('<bean:write name="PublisherBean" property="PublisherWebsite" />');" >
          <img src="images\hyperlink.jpg"  alt="<bean:write name="PublisherBean" property="PublisherWebsite" />"/>
          
          </a>

          --> 
          </xsl:when>  <!-- detailsLink -->
          <xsl:when test="$displayType = 'radio'">
          <!--
            <logic:iterate id="row" name="newLookupPublishersBean"> 
               <html:radio property="BookGroupPbrId" value="Id" idName="row"/> 
               <bean:write name="row" property="Name"/> <br/>
            </logic:iterate>

            the poor domain solution; we need to get hold of the Meanings but how do we do that??  
                             <logic:iterate name="DomainsBean" id="row" property="LanguageValue">
                              <html:radio property="BookGroupLanguage"  value="row" />
                              <bean:write name="row" />
                           </logic:iterate>

            DOMAINS using JSTL:

                              <c:set var="counter" value="0" />
                              <c:forEach var="value" items="${sessionScope.DomainsBean.map['LanguageValue']}" >
                                 <html-el:radio property="BookGroupLanguage" value="${value}"/>
                                 <c:out value="${sessionScope.DomainsBean.map['LanguageMeaning'][counter]}"/>
                                 <c:set var="counter" value="${counter+1}" />
                                 <br/>
                              </c:forEach>
and in table pages:
                              <c:set var="counter" value="0"/>
                              <c:set var="ind"><%= index %></c:set>
                              <c:forEach var="value" items="${sessionScope.DomainsBean.map['LanguageValue']}">
                                 <html-el:radio property="BookGroupLanguage[${ind}]" value="${value}"/>
                                 <c:out value="${sessionScope.DomainsBean.map['LanguageMeaning'][counter]}"/>
                                 <c:set var="counter" value="${counter+1}"/>
                                 <br/>
                              </c:forEach>


                      -->
          <xsl:choose>
            <xsl:when test="$domain = ''">
<!-- if meaning is '' perhaps for radio write None or is it up to the user to only use Radio Group for Mandatory items or to provide the display value for a null value ?? -->
                 <xsl:if test="$radioStyle='matrix'">
                   <c___set var="inRowCounter" value="0" />
                 </xsl:if>
<!-- in matrix:
                  <table>
                  <tr>
-->
                   <xsl:if test="$radioStyle='matrix'">
                     <xsl:text>###LT###table###GT###</xsl:text>
                     <xsl:text>###LT###tr###GT###</xsl:text>
                   </xsl:if>
              <logic___iterate>
                 <!-- The name of the JSP bean containing the collection to be iterated -->
                 <xsl:attribute name="name">
                   <xsl:value-of select="$optionsName"/>
                 </xsl:attribute>
                 <!-- The name of a page scope JSP bean that will contain the current element of the collection on each iteration, if it is not null.-->   
                 <xsl:attribute name="id">row</xsl:attribute>
                   <xsl:if test="$radioStyle='matrix'">
                     <xsl:text>###LT###td###GT###</xsl:text>
                   </xsl:if>
                  <!-- perhaps we should create a Table to hold the radio group -->
                  <html___radio>
                    <xsl:attribute name="property">
                      <xsl:value-of select="$propertyFinal"/>
                    </xsl:attribute>
  	                <xsl:attribute name="styleClass"><xsl:value-of select="$styleClass"/></xsl:attribute>
	                  <xsl:attribute name="styleId"><xsl:value-of select="$styleId"/></xsl:attribute>
                    <xsl:attribute name="value">
                      <xsl:value-of select="$optionsProperty"/>
                    </xsl:attribute>
                    <xsl:attribute name="idName">row</xsl:attribute>
                  </html___radio>
                  <xsl:choose>
                    <xsl:when test="$nullValueLabel !=''"> <!-- nullValueProperty was specified --> 
                      <logic___empty>
                        <xsl:attribute name="name">row</xsl:attribute>
                        <xsl:attribute name="property">
                          <xsl:value-of select="$optionsLabelProperty"/>
                        </xsl:attribute>

                        <!-- add nullValueLabel to resourceBundel -->
                        <xsl:variable name="nullValueKey">
                          <xsl:value-of select="string:toUpperCase(concat($group/@name,'_',@Name,'NullValueLabel'))"/>
                        </xsl:variable>
                
                        <xsl:call-template name="nlsTextAttribute">
                          <xsl:with-param name="key">
                            <xsl:value-of select="$nullValueKey"/>
                          </xsl:with-param>
                          <xsl:with-param name="value">
                            <xsl:value-of select="$nullValueLabel"/>
                          </xsl:with-param>
                          <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
                        </xsl:call-template>

                           <!-- <xsl:value-of select="$nullValueLabel"/> -->
                      </logic___empty>
                      <logic___notEmpty>
                        <xsl:attribute name="name">row</xsl:attribute>
                        <xsl:attribute name="property">
                          <xsl:value-of select="$optionsLabelProperty"/>
                        </xsl:attribute>
                          <bean___write>
                            <xsl:attribute name="name">row</xsl:attribute>
                            <xsl:attribute name="property">
                              <xsl:value-of select="$optionsLabelProperty"/>
                            </xsl:attribute>
                          </bean___write>
                      </logic___notEmpty>
                    </xsl:when> 
                    <xsl:otherwise>
                      <bean___write>
                        <xsl:attribute name="name">row</xsl:attribute>
                        <xsl:attribute name="property">
                          <xsl:value-of select="$optionsLabelProperty"/>
                        </xsl:attribute>
                      </bean___write>
                    </xsl:otherwise>
                  </xsl:choose>
<!-- in matrix:
                  </td>
-->
                   <xsl:if test="$radioStyle='matrix'">
                     <xsl:text>###LT###/td###GT###</xsl:text>
                   <!--
                     <c:set var="icounter" value="${icounter+1}"/>
                     <c:if test="${icounter ==2 }">
                       </tr><tr>
                       <c:set var="icounter" value="0"/>
                     </c:if>
                   -->
                     <c___set var="inRowCounter"  >
                        <xsl:attribute name="value">${inRowCounter+1}</xsl:attribute>
                     </c___set>                      
                     <c___if>
                       <xsl:attribute name="test">${inRowCounter==<xsl:value-of select="$radioMatrixSize" />}</xsl:attribute>
                       ###LT###/tr###GT###
                       ###LT###tr###GT###
                       <c___set var="inRowCounter" value="0" />
                     </c___if>
                   </xsl:if>
                  <!-- this should depend on whether the user wants a Horizontal or Vertical Radiogroup -->
                  <xsl:if test="$radioStyle='vertical' or $radioStyle=''">
                    <br />
                  </xsl:if>
              </logic___iterate>
<!-- in matrix:
                  </tr>
                  </table>
-->
                   <xsl:if test="$radioStyle='matrix'">
                     <xsl:text>###LT###/tr###GT###</xsl:text>
                     <xsl:text>###LT###/table###GT###</xsl:text>
                   </xsl:if>
              </xsl:when> <!-- no domain -->

              <xsl:when test="$domain != ''"> <!-- DOMAIN BASED!! -->
                 <c___set var="counter" value="0" />
                 <xsl:if test="$radioStyle='matrix'">
                   <c___set var="inRowCounter" value="0" />
                 </xsl:if>
                 <xsl:if test="$withIndex = 'true'"> <!-- TABLE LAYOUT -->
                   <c___set var="ind" >jspO= index jspC</c___set>   <!-- jspO and jspC will be replaced by JHeadstart class utils.jspWriter with <%= and %> -->
                 </xsl:if>

<!-- in matrix:
                  <table>
                  <tr>
-->
                   <xsl:if test="$radioStyle='matrix'">
                     <xsl:text>###LT###table###GT###</xsl:text>
                     <xsl:text>###LT###tr###GT###</xsl:text>
                   </xsl:if>
                 <c___forEach var="value" >
                   <xsl:attribute name="items">${sessionScope.DomainsBean.map['<xsl:value-of select="concat($domain,'Value' )"/>']}</xsl:attribute>
<!-- in matrix:
                  <td>
-->
                   <xsl:if test="$radioStyle='matrix'">
                     <xsl:text>###LT###td###GT###</xsl:text>
                   </xsl:if>
                   <html-el___radio >
  	                 <xsl:attribute name="styleClass"><xsl:value-of select="$styleClass"/></xsl:attribute>
	                   <xsl:attribute name="styleId"><xsl:value-of select="$styleId"/></xsl:attribute>
                     <xsl:choose>
                       <xsl:when test="$withIndex = 'true'"> <!-- TABLE LAYOUT -->
                         <!-- change propertyFinal=<%= "BookGroupLanguage["+index+"]" %>
                              into BookGroupLanguage[${ind}] -->
                         <xsl:attribute name="property"><xsl:value-of select="concat($group/@name, $property)"/>[${ind}]</xsl:attribute>                       
                       </xsl:when>
                       <xsl:otherwise>
                         <xsl:attribute name="property"><xsl:value-of select="$propertyFinal"/></xsl:attribute>                       
                       </xsl:otherwise>
                     </xsl:choose>
                     <xsl:attribute name="value">${value}</xsl:attribute>
                   </html-el___radio>
                   <!--
                                        <c:choose>
                       <c:when test="${empty sessionScope.DomainsBean.map['LanguageValue'][counter]}">
                         <c:out value="Nada" />
                       </c:when>
                       <c:otherwise>
                        <c:out value="${sessionScope.DomainsBean.map['LanguageMeaning'][counter]}" default="Nada"/>
                       </c:otherwise>
                     </c:choose>                     
-->
                  <xsl:choose>
                    <xsl:when test="$nullValueLabel !=''"> <!-- nullValueProperty was specified --> 
                       <c___choose>
                         <c___when>
                           <xsl:attribute name="test">${empty sessionScope.DomainsBean.map['<xsl:value-of select="concat($domain,'Value' )"/>'][counter]}</xsl:attribute>
                            <!-- add nullValueLabel to resourceBundle -->
                            <xsl:variable name="nullValueKey">
                              <xsl:value-of select="string:toUpperCase(concat($group/@name,'_',@Name,'NullValueLabel'))"/>
                            </xsl:variable>
                    
                            <xsl:call-template name="nlsTextAttribute">
                              <xsl:with-param name="key">
                                <xsl:value-of select="$nullValueKey"/>
                              </xsl:with-param>
                              <xsl:with-param name="value">
                                <xsl:value-of select="$nullValueLabel"/>
                              </xsl:with-param>
                              <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
                            </xsl:call-template>

<!--
<c___out>
                             <xsl:attribute name="value"><xsl:value-of select="$nullValueLabel"/></xsl:attribute>
                           </c___out>
-->
                         </c___when>
                         <c___otherwise>
                           <c___out>
                             <xsl:attribute name="value">${sessionScope.DomainsBean.map['<xsl:value-of select="concat($domain,'Meaning' )"/>'][counter]}</xsl:attribute>
                           </c___out>
                         </c___otherwise>
                       </c___choose>
                     </xsl:when>
                     <xsl:otherwise>
                       <c___out>
                         <xsl:attribute name="value">${sessionScope.DomainsBean.map['<xsl:value-of select="concat($domain,'Meaning' )"/>'][counter]}</xsl:attribute>
                       </c___out>
                     </xsl:otherwise>
                   </xsl:choose>
                   <c___set var="counter"  >
                      <xsl:attribute name="value">${counter+1}</xsl:attribute>
                   </c___set>                      
<!-- in matrix:
                  </td>
-->
                   <xsl:if test="$radioStyle='matrix'">
                     <xsl:text>###LT###/td###GT###</xsl:text>
                   <!--
                     <c:set var="icounter" value="${icounter+1}"/>
                     <c:if test="${icounter ==2 }">
                       </tr><tr>
                       <c:set var="icounter" value="0"/>
                     </c:if>
                   -->
                     <c___set var="inRowCounter"  >
                        <xsl:attribute name="value">${inRowCounter+1}</xsl:attribute>
                     </c___set>                      
                     <c___if>
                       <xsl:attribute name="test">${inRowCounter==<xsl:value-of select="$radioMatrixSize" />}</xsl:attribute>
                       ###LT###/tr###GT###
                       ###LT###tr###GT###
                       <c___set var="inRowCounter" value="0" />
                     </c___if>
                   </xsl:if>
                   <!-- this should depend on whether the user wants a Horizontal or Vertical Radiogroup -->
                   <xsl:if test="$radioStyle='vertical' or $radioStyle=''">
                     <br />
                   </xsl:if>
                 </c___forEach>
<!-- in matrix:
                  </tr>
                  </table>
-->
                   <xsl:if test="$radioStyle='matrix'">
                     <xsl:text>###LT###/tr###GT###</xsl:text>
                     <xsl:text>###LT###/table###GT###</xsl:text>
                   </xsl:if>
              </xsl:when> <!-- DOMAIN based -->
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>

          <xsl:element name="{$elementName}">


            <xsl:attribute name="name"><xsl:value-of select="$name"/></xsl:attribute>
            <xsl:attribute name="property">
              <xsl:value-of select="$propertyFinal"/>
            </xsl:attribute>
          <xsl:if test="    $displayType != 'hidden'">
             <xsl:call-template name="onBlur" >
               <xsl:with-param name="onBlur" select="$onBlur"/>
               <xsl:with-param name="restrictCase" select="$restrictCase"/>
             </xsl:call-template>
             <xsl:call-template name="onChange" >
               <xsl:with-param name="onChange" select="$onChange"/>
             </xsl:call-template>
             <xsl:call-template name="onFocus" >
               <xsl:with-param name="onFocus" select="$onFocus"/>
             </xsl:call-template>
             <xsl:call-template name="onClick" >
               <xsl:with-param name="onClick" select="$onClick"/>
             </xsl:call-template>
             <xsl:call-template name="onDblClick" >
               <xsl:with-param name="onDblClick" select="$onDblClick"/>
             </xsl:call-template>
          </xsl:if>
          <xsl:if test="    $displayType != 'hidden'
                        and $displayType != 'checkBox'
                        and $elementName != 'html___textarea'
                        and $displayType != 'choice'">
            <xsl:attribute name="size"><xsl:value-of select="$size"/></xsl:attribute>
            <xsl:attribute name="maxlength"><xsl:value-of select="$maximumLength"/></xsl:attribute>
            <!-- LJ apply Style Class and Style Id -->
            <xsl:attribute name="styleClass"><xsl:value-of select="$styleClass"/></xsl:attribute>
            <xsl:attribute name="styleId"><xsl:value-of select="$styleId"/></xsl:attribute>
            <!-- LJ this sets the title attribute for most display types to the hintText
                 We may decide to use a new property for this item title
                 We also decide to only use it on Table Layout screens, have it
                 controlled by a Service (or Group?) level property
              -->
            <xsl:variable name="titleKey">
              <xsl:value-of select="string:toUpperCase(concat($group/@name,'_',@Name,'_TITLE'))"/>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test=" $nls ='true'">
                <xsl:attribute name="titleKey">            
                  <xsl:call-template name="nlsTextAttribute">
                    <xsl:with-param name="attributeName" select="'title'"/>
                     <xsl:with-param name="key">
                       <xsl:value-of select="$titleKey"/>
                     </xsl:with-param>
                     <xsl:with-param name="value">
                       <xsl:value-of select="$hintText"/>
                     </xsl:with-param>
                     <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
                    <xsl:with-param name="outputAsAttributeValue">true</xsl:with-param>
                  </xsl:call-template>
                </xsl:attribute>
              </xsl:when>
              <xsl:otherwise>
                <xsl:attribute name="title">            
                      <xsl:value-of select="$hintText"/>
                </xsl:attribute>
              </xsl:otherwise>
            </xsl:choose>

            <!-- end LJ -->
          </xsl:if>
          <xsl:if test="$displayType = 'choice' and $displayHeight != 1">
            <xsl:attribute name="size"><xsl:value-of select="$displayHeight"/></xsl:attribute>
          </xsl:if>



          <xsl:if test="$elementName = 'html___textarea'">
            <xsl:attribute name="cols"><xsl:value-of select="$size"/></xsl:attribute>
            <xsl:attribute name="rows"><xsl:value-of select="$displayClass"/></xsl:attribute>
          </xsl:if>
          <xsl:if test="$displayType = 'checkBox'">
            <xsl:attribute name="value">
              <xsl:choose>
                <xsl:when test="$domain!=''">
                  <!-- take first value of Domain defined in DomainDefinitions.xml -->
                  <xsl:value-of select="$domainDef/DomainSet/Domain[@name=$domain]/AllowableValue[1]/@value"/>
                </xsl:when>
                <xsl:otherwise>
                  <!-- take first value defined in ListValidator of attribute -->
                  <xsl:value-of select="$listValidator[1]/@Value"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:attribute>
          </xsl:if>

            <xsl:if test="($displayType != 'hidden') and (($displayType = 'lov' and $layoutStyle != 'find') or ($isReadOnly = 'true') or ($displayType = 'displayField'))">
              <xsl:attribute name="disabled">
                <xsl:choose>
                  <xsl:when test="($displayType = 'lov' and $layoutStyle != 'find' and $lookupLovValues/@baseValueAttribute != $lookupLovValues/@baseDisplayAttribute ) or ($displayType = 'displayField')">
                    <xsl:value-of select="'true'"/>
                  </xsl:when>
<!--
                    Fix for bug 3400587. 
-->
                  <xsl:when test="$displayType = 'lov' and $layoutStyle != 'find' and $lookupLovValues/@baseValueAttribute != $lookupLovValues/@baseDisplayAttribute and $updateable='while_insert' ">
                    <xsl:value-of select="concat('jspO= ', $group/@name, 'IsInserting == Boolean.FALSE jspC')"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="'false'"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:attribute>
            </xsl:if>

          <xsl:if test="$displayType = 'choice' ">
            <xsl:choose>
              <xsl:when test="$domain != '' or count($listValidator) = 0">
                <html___options>
                  <xsl:choose>
                    <xsl:when test="$domain = ''">
                      <xsl:attribute name="name"><xsl:value-of select="$optionsName"/></xsl:attribute>
                      <xsl:attribute name="property"><xsl:value-of select="$optionsProperty"/></xsl:attribute>
                      <xsl:attribute name="labelName"><xsl:value-of select="$optionsLabelName"/></xsl:attribute>
                      <xsl:attribute name="labelProperty"><xsl:value-of select="$optionsLabelProperty"/></xsl:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:attribute name="name">DomainsBean</xsl:attribute>
                      <xsl:attribute name="property"><xsl:value-of select="concat($domain, 'Value')"/></xsl:attribute>
                      <xsl:attribute name="labelName">DomainsBean</xsl:attribute>
                      <xsl:attribute name="labelProperty"><xsl:value-of select="concat($domain, 'Meaning')"/></xsl:attribute>
                    </xsl:otherwise>
                  </xsl:choose>
                </html___options>
              </xsl:when>
              <xsl:otherwise>
                <option/>
                <xsl:for-each select="$listValidator">
                  <html___option value="{@Value}">
                    <xsl:value-of select="@Value"/>
                  </html___option>
                </xsl:for-each>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:if>

        </xsl:element>
       </xsl:otherwise>
      </xsl:choose>
          <xsl:if test="$showHyperlink = 'true'">
            <a href="#" >
              <xsl:attribute name="onClick">
                 popupAndGo('###LT###bean___write name=`<xsl:value-of select="$name"/>` property=`<xsl:value-of select="$propertyFinal"/>` /###GT###');
              </xsl:attribute>
              <img src="images\hyperlink.jpg"  >
                <xsl:attribute name="alt">###LT###bean___write name=`<xsl:value-of select="$name"/>` property=`<xsl:value-of select="$propertyFinal"/>` /###GT###</xsl:attribute>
              </img>
            </a>
          </xsl:if>  <!-- hyperlink -->

        <xsl:if test="$displayType = 'editor'">
          <xsl:call-template name="jspInclude">
            <xsl:with-param name="page" select="'editorPopup.jsp'"/>
            <xsl:with-param name="param1Name" select="'editorField'"/>
            <xsl:with-param name="param1Value" select="$propertyFinal"/>
            <xsl:with-param name="param2Name" select="'maxlength'"/>
            <xsl:with-param name="param2Value" select="$maximumLength"/>
            <xsl:with-param name="param3Name" select="'display'"/>
            <xsl:with-param name="param3Value" select="concat('jspO= ', $group/@name, 'IsInserting == Boolean.FALSE jspC')"/>
            <xsl:with-param name="addParam3" select="($isReadOnly = 'true' and $layoutStyle != 'find')"/>
         </xsl:call-template>
        </xsl:if>


        <xsl:if test="$displayType = 'lov'">
          <xsl:call-template name="jspInclude">
            <xsl:with-param name="page" select="'lovPopup.jsp'"/>
            <xsl:with-param name="param1Name" select="'lovName'"/>
            <xsl:with-param name="param1Value" select="$lookupLovValues/@name"/>
            <xsl:with-param name="param2Name" select="'baseKeyItem'"/>
            <xsl:with-param name="param2Value">
              <xsl:choose>
                <xsl:when test="$withIndex = 'true'">
                  <xsl:value-of select="concat('jspO= ``', $group/@name, $lookupLovValues/@baseValueAttribute, '[``+index+``]`` jspC')"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="concat($group/@name, $lookupLovValues/@baseValueAttribute)"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:with-param>
            <xsl:with-param name="param3Name" select="'baseDescriptorItems'"/>
            <xsl:with-param name="param3Value" select="$propertyFinal"/>
            <xsl:with-param name="param4Name" select="'display'"/>
            <xsl:with-param name="param4Value" select="concat('jspO= ', $group/@name, 'IsInserting == Boolean.TRUE jspC')"/>
            <xsl:with-param name="addParam4" select="($isReadOnly = 'true' and $layoutStyle != 'find')"/>
         </xsl:call-template>
        </xsl:if>


        <xsl:if test="($displayType = 'dateField') or ($displayType = 'datetimeField')">
          <xsl:call-template name="jspInclude">
            <xsl:with-param name="page" select="'calendarPopup.jsp'"/>
            <xsl:with-param name="param1Name" select="'itemName'"/>
            <xsl:with-param name="param1Value" select="$propertyFinal"/>
            <xsl:with-param name="param2Name" select="'display'"/>
            <xsl:with-param name="param2Value" select="concat('jspO= ', $group/@name, 'IsInserting == Boolean.TRUE jspC')"/>
            <xsl:with-param name="addParam2" select="($isReadOnly = 'true' and $layoutStyle != 'find')"/>
            <xsl:with-param name="param3Name" select="'format'"/>
            <xsl:with-param name="param3Value">
              <xsl:choose>
                <xsl:when test="$displayType = 'dateField'">
                  <xsl:value-of select="$datePattern"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="$datetimePattern"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:with-param>
         </xsl:call-template>
        </xsl:if>

        <xsl:if test="    ($hintText != '')
                      and ($withIndex != 'true')
                      and $displayType != 'hidden'
                      and $displayType != 'displayField'
                      and $showHintText = 'true'">
          <br/><font class="hintText">
                  <xsl:variable name="hintKey">
                    <xsl:value-of select="string:toUpperCase(concat($group/@name,'_',@Name,'_HINT'))"/>
                  </xsl:variable>
              <xsl:call-template name="nlsTextAttribute">
                <xsl:with-param name="attributeName" select="'hint'"/>
                <xsl:with-param name="key">
                  <xsl:value-of select="$hintKey"/>
                </xsl:with-param>
                <xsl:with-param name="value">
                  <xsl:value-of select="$hintText"/>
                </xsl:with-param>
                <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
              </xsl:call-template>

           </font>
        </xsl:if>

      </xsl:when>

      <xsl:when test="$table='td'">

      <xsl:if test="$withIndex != 'true'">

        <td class="prompt" nowrap="nowrap">

         <xsl:if test="$displayType != 'fileDownload'">

          <xsl:if test="$required = 'true'">
            <font class="required">*</font>
          </xsl:if>
          <xsl:if test="$promptStyleClass != ''"> <!-- LJ a styleclass was specified for the prompt -->
            ###LT###span class="<xsl:value-of select="$promptStyleClass" />" ###GT###
          </xsl:if>


          <xsl:variable name="promptValue">
          <!-- test whether prompt is null, somehow the default value
          does not work with getCustomProperty used within xsl:param
          element -->
          <xsl:choose>
            <xsl:when test="$prompt=''">
              <xsl:variable name="customPrompt">
              <!-- get prompt from custom property when not specified -->
                <xsl:call-template name="getCustomProperty">
                  <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
                  <xsl:with-param name="propertyName" select="'PROMPT'"/>
                  <xsl:with-param name="defaultValue" select="@Name"/>
                </xsl:call-template>
              </xsl:variable>
              <xsl:value-of disable-output-escaping="yes" select="$customPrompt"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of disable-output-escaping="yes" select="$prompt"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <xsl:variable name="promptKey">
          <xsl:value-of select="string:toUpperCase(concat($group/@name,'_',@Name))"/>
        </xsl:variable>

        <xsl:call-template name="nlsTextAttribute">
          <xsl:with-param name="attributeName" select="'promptAndAccessKey'"/>
          <xsl:with-param name="key">
            <xsl:value-of select="$promptKey"/>
          </xsl:with-param>
          <xsl:with-param name="value">
            <xsl:value-of select="$promptValue"/>
          </xsl:with-param>
          <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
        </xsl:call-template>

        </xsl:if>
          <xsl:if test="$promptStyleClass != ''"> <!-- LJ a styleclass was specified for the prompt -->
            ###LT###/span###GT###  <!-- ###LT### gets replaced by util.jspWriter -->
          </xsl:if>
        </td>

        <xsl:variable name="dummy">
          <xsl:value-of select="utils:increaseCounter('columnsCounter')"/>
        </xsl:variable>
      </xsl:if>
        <td nowrap="nowrap">  <!-- LJ: perhaps apply styleClass here at the TD?? -->

          <xsl:call-template name="htmlText">
            <xsl:with-param name="name" select="$name"/>
            <xsl:with-param name="property" select="$property"/>
            <xsl:with-param name="size" select="$size"/>
            <xsl:with-param name="table" select="'no'"/>
            <xsl:with-param name="prompt" select="$prompt"/>
            <xsl:with-param name="withIndex" select="$withIndex"/>
            <xsl:with-param name="isFindElement" select="$isFindElement"/>
            <xsl:with-param name="required">
              <xsl:choose>
                <xsl:when test="$withIndex != 'true'">false</xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="$required"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:with-param>
            <xsl:with-param name="hintText" select="$hintText"/>
            <xsl:with-param name="group"    select="$group"/>
            <xsl:with-param name="formName" select="$formName"/>
            <xsl:with-param name="displayType" select="$displayType"/>
            <xsl:with-param name="optionsName" select="$optionsName"/>
            <xsl:with-param name="optionsProperty" select="$optionsProperty"/>
            <xsl:with-param name="optionsLabelName" select="$optionsLabelName"/>
            <xsl:with-param name="optionsLabelProperty" select="$optionsLabelProperty"/>
            <xsl:with-param name="domain" select="$domain"/>
            <xsl:with-param name="isReadOnly" select="$isReadOnly"/>
            <xsl:with-param name="showHyperlink" select="$showHyperlink"/>
            <xsl:with-param name="maximumLength" select="$maximumLength"/>
            <xsl:with-param name="displayHeight" select="$displayHeight"/>
            <xsl:with-param name="styleClass" select="$styleClass"/>
            <xsl:with-param name="styleId" select="$styleId"/>
            <xsl:with-param name="promptStyleClass" select="$promptStyleClass"/>
            <xsl:with-param name="nullValueLabel" select="$nullValueLabel"/>
            <xsl:with-param name="radioStyle" select="$radioStyle"/>
            <xsl:with-param name="radioMatrixSize" select="$radioMatrixSize"/>
            <xsl:with-param name="onBlur" select="$onBlur"/>
            <xsl:with-param name="onFocus" select="$onFocus"/>
            <xsl:with-param name="onChange" select="$onChange"/>
            <xsl:with-param name="onClick" select="$onClick"/>
            <xsl:with-param name="onDblClick" select="$onDblClick"/>
            <xsl:with-param name="restrictCase" select="$restrictCase"/>
            <xsl:with-param name="lookupLovValues" select="$lookupLovValues"/>
            <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
            <xsl:with-param name="listValidator" select="$listValidator"/>
          </xsl:call-template>
        </td>
      </xsl:when>
      <!-- items in a Table -->
      <xsl:when test="$table = 'tr'">
        <tr>
          <xsl:call-template name="htmlText">
            <xsl:with-param name="name" select="$name"/>
            <xsl:with-param name="property" select="$property"/>
            <xsl:with-param name="size" select="$size"/>
            <xsl:with-param name="table" select="'td'"/>
            <xsl:with-param name="prompt" select="$prompt"/>
            <xsl:with-param name="withIndex" select="$withIndex"/>
            <xsl:with-param name="isFindElement" select="$isFindElement"/>
            <xsl:with-param name="required" select="$required"/>
            <xsl:with-param name="hintText" select="$hintText"/>
            <xsl:with-param name="group"    select="$group"/>
            <xsl:with-param name="formName" select="$formName"/>
            <xsl:with-param name="displayType" select="$displayType"/>
            <xsl:with-param name="isReadOnly" select="$isReadOnly"/>
            <xsl:with-param name="showHyperlink" select="$showHyperlink"/>
            <xsl:with-param name="optionsName" select="$optionsName"/>
            <xsl:with-param name="optionsProperty" select="$optionsProperty"/>
            <xsl:with-param name="optionsLabelName" select="$optionsLabelName"/>
            <xsl:with-param name="optionsLabelProperty" select="$optionsLabelProperty"/>
            <xsl:with-param name="domain" select="$domain"/>
            <xsl:with-param name="maximumLength" select="$maximumLength"/>
            <xsl:with-param name="displayHeight" select="$displayHeight"/>
            <xsl:with-param name="styleClass" select="$styleClass"/>
            <xsl:with-param name="styleId" select="$styleId"/>
            <xsl:with-param name="promptStyleClass" select="$promptStyleClass"/>
            <xsl:with-param name="nullValueLabel" select="$nullValueLabel"/>
            <xsl:with-param name="radioStyle" select="$radioStyle"/>
            <xsl:with-param name="radioMatrixSize" select="$radioMatrixSize"/>
            <xsl:with-param name="onBlur" select="$onBlur"/>
            <xsl:with-param name="onChange" select="$onChange"/>
            <xsl:with-param name="onFocus" select="$onFocus"/>
            <xsl:with-param name="onClick" select="$onClick"/>
            <xsl:with-param name="onDblClick" select="$onDblClick"/>
            <xsl:with-param name="restrictCase" select="$restrictCase"/>
            <xsl:with-param name="lookupLovValues" select="$lookupLovValues"/>
            <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
            <xsl:with-param name="listValidator" select="$listValidator"/>
          </xsl:call-template>
        </tr>
      </xsl:when>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="htmlRadio">
    <xsl:param name="property"/>
    <xsl:param name="value"/>
    <xsl:param name="table">tr</xsl:param>
      <html___radio property="{$property}" value="{$value}"/>
  </xsl:template>

</xsl:stylesheet>