<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : JSPSelectFormLib.xsl

Notes       :

$revision_history$
14-aug-2003   Aard-Jan van Kesteren
  1.10        
12-aug-2003   Aard-Jan van Kesteren
  1.9         Solved bug 3092283: Show view label if not updatable and not deletable
07-aug-2003   Aard-Jan van Kesteren
  1.8         Only call getNlsKey for new button if the new button is actually displayed
07-aug-2003   Steven Davelaar
  1.7         typo in getNlsKey
14-mrt-2003  Aard-Jan van Kesteren
  1.0         Initial creation (based on UIXSelectFormLib.xsl)

*******************************************************************************
$add_to_revision_history
$
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  xmlns:string="http://www.oracle.com/XSL/Transform/java/java.lang.String"      
  exclude-result-prefixes="utils string"
>
  <xsl:import href="JSPLibrary.xsl"/>
  <xsl:import href="JSPInput.xsl"/>

  <xsl:output method="xml" indent="yes" />

  <xsl:template name="selectLayout">
    <xsl:param name="columns"/>
    <xsl:param name="voDoc"/>

    <xsl:variable name="descriptorAttribute" select="@descriptorAttribute"/>
    <xsl:variable name="dataObjectKey" select="@name"/>
    <xsl:variable name="dataObjectSetKey" select="concat(@name,'Set')"/>

    <xsl:call-template name="jspInclude">
      <xsl:with-param name="page" select="'jhsInfo.jsp'"/>
    </xsl:call-template>

    <html___form>

      <xsl:attribute name="name">
        <xsl:value-of select="concat('Select', @name)"/>
      </xsl:attribute>
      <xsl:attribute name="action">
        <xsl:value-of select="concat('/Start', /Service/@name)"/>
      </xsl:attribute>
      <xsl:attribute name="type">
        <xsl:text>oracle.jheadstart.view.struts.DynaBeanReadOnly</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="scope">
        <xsl:text>session</xsl:text>
      </xsl:attribute>

      <script>ignoreChanges=true;</script>

      <input type="hidden" name="targetCondition"/>
      <input type="hidden" name="targetConditionValue"/>    
 
      <xsl:call-template name="targetGroupHiddenFields"/>

      <xsl:variable name="displayTitle">
        <xsl:call-template name="getDisplayTitle"/>
      </xsl:variable>

      <xsl:call-template name="jspInclude">
        <xsl:with-param name="page" select="'heading1.jsp'"/>
        <xsl:with-param name="param1Name" select="'key'"/>      
        <xsl:with-param name="param1Value">
          <xsl:value-of select="utils:getNlsKey('SELECT_TITLE',$displayTitle,concat('SELECT_TITLE_',@name))"/>
        </xsl:with-param>                
      </xsl:call-template>

    <xsl:call-template name="htmlText">
      <xsl:with-param name="name" select="concat('Select', @name)"/>
      <xsl:with-param name="property" select="concat(@name, 'Rownum')"/>
      <xsl:with-param name="table" select="'no'"/>
      <xsl:with-param name="displayHeight" select="'10'"/>    
      <xsl:with-param name="group" select="@name"/>
      <xsl:with-param name="displayType" select="'choice'"/>
      <xsl:with-param name="optionsName" select="concat(@name, 'SetBean')"/>    
      <xsl:with-param name="optionsProperty" select="'rownum'"/>    
      <xsl:with-param name="optionsLabelName" select="concat(@name, 'SetBean')"/>    
      <xsl:with-param name="optionsLabelProperty" select="$descriptorAttribute"/>
      <!-- We need to give htmlText a listValidator parameter with an empty nodeset -->
      <xsl:with-param name="listValidator" select="//Nonsense/NodesetOf0"/>
    </xsl:call-template>

      <xsl:variable name="newKey">
        <xsl:choose>
          <xsl:when test="@insert">
            <xsl:value-of select="utils:getNlsKey('NEW_BUTTON_LABEL',$displayTitle,concat('NEW_BUTTON_LABEL_',@name))"/>
          </xsl:when>
          <xsl:otherwise>
            no new button
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:variable name="editKey">
        <xsl:choose>
          <xsl:when test="(@update = 'true') or (@delete = 'true')">
            <xsl:value-of select="utils:getNlsKey('EDIT_BUTTON_LABEL',$displayTitle,concat('EDIT_BUTTON_LABEL_',@name))"/>
          </xsl:when>
          <xsl:otherwise>
            no view button
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:variable name="viewKey">
        <xsl:choose>
          <xsl:when test="(@update = 'false') and (@delete = 'false')">
            <xsl:value-of select="utils:getNlsKey('VIEW_BUTTON_LABEL',$displayTitle,concat('VIEW_BUTTON_LABEL_',@name))"/>
          </xsl:when>
          <xsl:otherwise>
            no view button
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:call-template name="jspInclude">
        <xsl:with-param name="page" select="'buttonsSelectForm.jsp'"/>
        <xsl:with-param name="param1Name" select="'newKey'"/>        
        <xsl:with-param name="param1Value" select="$newKey"/>
        <xsl:with-param name="addParam1" select="@insert"/>        
        <xsl:with-param name="param2Name" select="'editKey'"/>        
        <xsl:with-param name="param2Value" select="$editKey"/>
        <xsl:with-param name="addParam2" select="(@update = 'true') or (@delete = 'true')"/>
        <xsl:with-param name="param3Name" select="'viewKey'"/>
        <xsl:with-param name="param3Value" select="$viewKey"/>
        <xsl:with-param name="addParam3" select="(@update= 'false') and (@delete = 'false')"/>
        <xsl:with-param name="param4Name" select="'formName'"/>
        <xsl:with-param name="param4Value" select="concat('Select', @name)"/>
      </xsl:call-template>

    </html___form>
  </xsl:template>

</xsl:stylesheet>
