<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : JSPTableLib.xsl

Notes       :

$revision_history$
20-jul-2004   Lucas Jellema (AMIS)
  1.25AMIS1.0 support for detailsLinkStyle, values: buttonBegin, buttonEnd, buttonBeginEnd and recordDoubleClick
27-jan-2004   Peter Ebell
  1.25        On sortable table headers, use the persistentAttribute instead of the viewattribute name.
14-nov-2003   Steven Davelaar
  1.24        Set form encType to multipart when appropriate
26-oct-2003   Aard-Jan van Kesteren
  1.23        Rewrote Boolean.TRUE and Boolean.FALSE occurences
14-aug-2003   Aard-Jan van Kesteren
  1.22        
14-aug-2003   Aard-Jan van Kesteren
  1.21        Added bean:define isInserting
12-aug-2003   Aard-Jan van Kesteren
  1.20        Solved bug 3089847. The display title of a child button should come from
              child group and not from the master group
12-aug-2003   Aard-Jan van Kesteren
  1.19        Solved Bug 3092285. Only show new button if table-form and insert = true
12-aug-2003   Aard-Jan van Kesteren
  1.18        Only show save button if it's not a view-only table
11-aug-2003   Steven Davelaar
  1.17        also enabled delete checkbox for  new rows: bug 2683576
08-aug-2003   Steven Davelaar
  1.16        Prefix deleteRow checkbox with group name
07-aug-2003   Aard-Jan van Kesteren
  1.15        use new getNlsKey for child_button_label
07-aug-2003   Aard-Jan van Kesteren
  1.14        Generate key SAVE_BUTTON_LABEL_(group name) instead of just SAVE_BUTTON_LABEL
07-aug-2003   Aard-Jan van Kesteren
  1.13        Only call getNlsKey for new button if this button is actually displayed
07-aug-2003   Steven Davelaar
  1.12        Convert nls keys to uppercase
06-aug-2003   Steven Davelaar
  1.11        
04-aug-2003   Steven Davelaar
  1.10        DELETE_COLUMN_HEADER always nls enabled (displayed with bean:message)
              master Context always nls enabled (displayed with bean:message)
01-aug-2003   Aard-Jan van Kesteren
  1.9         - Refactored attributes of jspInclude detailsButtonTableForm
              - Refactored attributes of jspInclude buttonsTableForm
              - Refactored attributes of jspInclude heading1
29-jul-2003   Aard-Jan van Kesteren
  1.8         - Dropped hidden fields value and source. These are now part of the routerForm.
              - removed js call addIgnoreChangeField() for rownum
              - removed js call setCheckForChangedItems()
              - dropped hidden input event and added hidden inputs targetCondition
                and targetConditionValue
              - Made changes necesarry for using form.js instead of jspForm.js
28-jul-2003   Aard-Jan van Kesteren
  1.7         - Solved bug 3071412. Show new button on table screen in case of table-form
27-jul-2003   Aard-Jan van Kesteren
  1.6         - Worked on bug 3054427. Support master-details
23-jul-2003   Aard-Jan van Kesteren
  1.5         - Solved bug 3062844. Rename js function goto to jhsGoto
22-jul-2003   Aard-Jan van Kesteren
  1.4         - Handled bug 3054575. Show a detail button instead of a link.
                The link code of Steven Davelaar is commented out and can be put back 
                quickly.
14-mrt-2003   Aard-Jan van Kesteren
  1.0         Initial creation (based on UIXTableLib.xsl)

*******************************************************************************
$add_to_revision_history
$
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  xmlns:string="http://www.oracle.com/XSL/Transform/java/java.lang.String"      
  exclude-result-prefixes="utils string"
>

  <xsl:output method="xml" indent="yes" />

  <xsl:template name="tableLayout">
    <xsl:param name="voDoc"/>
    <xsl:param name="masterDataObject"/>
    <xsl:param name="masterTabName"/>
    <xsl:param name="masterContextAttribute"/>
    <xsl:param name="display"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="groupLayoutStyle"/>

    <xsl:variable name="dataObjectSetKey" select="concat(@name,'Set')"/>
    <xsl:variable name="useTableRange" select="@useTableRange"/>

    <xsl:variable name="width">
      <xsl:choose>
        <xsl:when test="@tableWidth!=''">
          <xsl:value-of select="@tableWidth"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'100%'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- "Group" tests whether this group has detail childs -->        
    <xsl:variable name="hasDetailButton" select="$groupLayoutStyle='table-form' or ($groupLayoutStyle='table' and Group)"/>

    <xsl:choose>
      <xsl:when test="@samePage='true'">

            <xsl:call-template name="table">
              <xsl:with-param name="voDoc" select="$voDoc"/>
              <xsl:with-param name="width" select="$width"/>
              <xsl:with-param name="tableDataSet" select="$dataObjectSetKey"/>
              <xsl:with-param name="useTableRange" select="$useTableRange"/>
              <xsl:with-param name="deleteAllowed" select="@delete"/>
              <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
              <xsl:with-param name="groupLayoutStyle" select="$groupLayoutStyle"/>
            </xsl:call-template>

      </xsl:when>
      <xsl:otherwise>

      <xsl:call-template name="jspInclude">
        <xsl:with-param name="page" select="'jhsInfo.jsp'"/>
      </xsl:call-template>

    <xsl:variable  name="multipartAttrs">
      <xsl:call-template name="checkMultipartAttributes"/>
    </xsl:variable>

    <html___form>

      <xsl:attribute name="name">
        <xsl:value-of select="concat(@name, 'Bean')"/>
      </xsl:attribute>
      <xsl:attribute name="action">
        <xsl:value-of select="concat('/Start', /Service/@name)"/>
      </xsl:attribute>
        <xsl:if test="$multipartAttrs != ''">
          <xsl:attribute name="enctype">
            <xsl:value-of select="'multipart/form-data'"/>
          </xsl:attribute>
        </xsl:if>
      <xsl:attribute name="type">
        <xsl:text>oracle.jheadstart.view.struts.JhsDynaActionForm</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="scope">
        <xsl:text>session</xsl:text>
      </xsl:attribute>

      <input type="hidden" name="HasErrors"/>

      <xsl:call-template name="jspInclude">
        <xsl:with-param name="page" select="'timestamp.jsp'"/>
      </xsl:call-template>

      <xsl:call-template name="jspInclude">
        <xsl:with-param name="page" select="'heading1.jsp'"/>
        <xsl:with-param name="param1Name" select="'key'"/>
        <xsl:with-param name="param1Value">

        <xsl:variable name="displayTitle">
          <xsl:call-template name="getDisplayTitle"/>
        </xsl:variable>
        <xsl:variable name="dummy">
          <xsl:value-of select="utils:addNlsText(concat('TABLE_TITLE_',string:toUpperCase(@name)),$displayTitle)"/>
        </xsl:variable>

          <xsl:value-of select="concat('TABLE_TITLE_',string:toUpperCase(@name))"/>  
        </xsl:with-param>
 
      </xsl:call-template>

      <xsl:if test="@samePage!='true' or not(@samePage)">
        <xsl:call-template name="masterContext">
           <xsl:with-param name="masterDataObject" select="$masterDataObject"/>
           <xsl:with-param name="masterTabName" select="$masterTabName"/>
           <xsl:with-param name="masterContextAttribute" select="$masterContextAttribute"/>
        </xsl:call-template>
      </xsl:if>

      <input type="hidden" name="targetCondition"/>
      <input type="hidden" name="targetConditionValue"/>      

      <xsl:call-template name="targetGroupHiddenFields"/>
      
        <xsl:call-template name="table">
          <xsl:with-param name="voDoc" select="$voDoc"/>
          <xsl:with-param name="width" select="$width"/>
          <xsl:with-param name="tableDataSet" select="$dataObjectSetKey"/>
          <xsl:with-param name="useTableRange" select="$useTableRange"/>
          <xsl:with-param name="deleteAllowed" select="@delete"/>
          <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
          <xsl:with-param name="groupLayoutStyle" select="$groupLayoutStyle"/>
        </xsl:call-template>

           <xsl:variable name="displayTitle">
             <xsl:call-template name="getDisplayTitle"/>
           </xsl:variable>


      <xsl:variable name="newKey">
        <xsl:choose>
          <xsl:when test="$groupLayoutStyle='table-form'">
            <xsl:value-of select="utils:getNlsKey('NEW_BUTTON_LABEL', $displayTitle,concat('NEW_BUTTON_LABEL_',@name))"/>
          </xsl:when>
          <xsl:otherwise>
            no new button
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:variable name="saveTableKey">
        <xsl:choose>
          <xsl:when test="   (@delete = 'true') 
                          or (@update = 'true') 
                          or (@insert = 'true' and not($groupLayoutStyle='table-form'))">
            <xsl:value-of select="utils:getNlsKey('SAVE_BUTTON_LABEL', $displayTitle,concat('SAVE_BUTTON_LABEL_',@name))"/>
          </xsl:when>
          <xsl:otherwise>
            no save table button
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>      
      
      <xsl:call-template name="jspInclude">
        <xsl:with-param name="page" select="'buttonsTableForm.jsp'"/>
        <xsl:with-param name="param1Name" select="'saveTableKey'"/>              
        <xsl:with-param name="param1Value" select="$saveTableKey"/>        
        <xsl:with-param name="addParam1" 
                        select="   (@delete = 'true') 
                                or (@update = 'true') 
                                or (@insert = 'true' and not($groupLayoutStyle='table-form'))
                                "/>
        <xsl:with-param name="param2Name" select="'newKey'"/>              
        <xsl:with-param name="param2Value" select="$newKey"/>
        <xsl:with-param name="addParam2" select="$groupLayoutStyle='table-form' and @insert='true'"/>
        <xsl:with-param name="param3Name" select="'formName'"/>
        <xsl:with-param name="param3Value" select="concat(@name, 'Bean')"/>        
      </xsl:call-template>    
    
    </html___form>
        
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  <!-- template for adding master context -->
  <xsl:template name="masterContext">
    <xsl:param name="masterDataObject"/>
    <xsl:param name="masterTabName"/>
    <xsl:param name="masterContextAttribute"/>
    <xsl:choose>
      <xsl:when test="$masterDataObject!=''">
        <xsl:call-template name="jspInclude">
          <xsl:with-param name="page" select="'masterContext.jsp'"/>
          <xsl:with-param name="param1Name" select="'name'"/>
          <xsl:with-param name="param1Value">
            <!-- do not call nlsTextAttribute. masterContext must
                 always be nls-enabled because it is displayed through
                 include -->
            <xsl:variable name="dummy">
              <xsl:value-of select="utils:addNlsText(concat('MASTER_CONTEXT_',string:toUpperCase(../@name)),$masterTabName)"/>            
            </xsl:variable>

            <xsl:value-of select="concat('MASTER_CONTEXT_',string:toUpperCase(../@name))"/>

<!--                 
            <xsl:call-template name="nlsTextAttribute">
              <xsl:with-param name="attributeName" select="'prompt'"/>
              <xsl:with-param name="key">
                <xsl:value-of select="string:toUpperCase($masterTabName)"/>
              </xsl:with-param>
              <xsl:with-param name="value">
                <xsl:value-of select="$masterTabName"/>
              </xsl:with-param>
              <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
              <xsl:with-param name="outputAsAttributeValue" select="'true'"/>
            </xsl:call-template>
-->
          </xsl:with-param>
          <xsl:with-param name="param2Name" select="'bean'"/>
          <xsl:with-param name="param2Value" select="concat($masterDataObject, 'Bean')"/>
          <xsl:with-param name="param3Name" select="'property'"/>
          <xsl:with-param name="param3Value" select="concat($masterDataObject, $masterContextAttribute)"/>          
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="setDisplayVariable">
    <xsl:param name="entityAttributeNode"/>
    <xsl:param name="groupLayoutStyle"/>

    <xsl:variable name="displayAll">
      <xsl:call-template name="getCustomProperty">
        <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
        <xsl:with-param name="propertyName" select="'DISPLAY'"/>
        <xsl:with-param name="defaultValue" select="'true'"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="displayTable">
      <xsl:call-template name="getCustomProperty">
        <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
        <xsl:with-param name="propertyName" select="'DISPLAY_TABLE'"/>
        <xsl:with-param name="defaultValue" select="$displayAll"/>
      </xsl:call-template>
    </xsl:variable>
    <!-- LJ only when display/displayAll=true should the value returned be DISPLAY_TABLE -->
    <xsl:choose>
      <xsl:when test="$displayAll='true'">
        <xsl:choose>
          <xsl:when test="$groupLayoutStyle='table'">
            <xsl:value-of select="$displayAll"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$displayTable"/>
          </xsl:otherwise>
        </xsl:choose>  
      </xsl:when>
      <xsl:when test="$displayAll='hidden'">
        <xsl:value-of select="'hidden'"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="'false'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- main template for adding the table element -->
  <xsl:template name="table">
    <xsl:param name="voDoc"/>
    <xsl:param name="width"/>
    <xsl:param name="tableDataSet"/>
    <xsl:param name="useTableRange"/>
    <xsl:param name="deleteAllowed"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="groupLayoutStyle"/>

    <xsl:variable name="displayTitle">
      <xsl:call-template name="getDisplayTitle"/>
    </xsl:variable>

    <!-- we need to store the group node in a variable because we will
         loose the "Group" context from here on
    -->
    
    <xsl:variable name="group" select="current()"/>
    <xsl:variable name="resourceName" select="@name"/>
    <xsl:variable name="formName">
    <!--
      <xsl:choose>
        <xsl:when test="//Group/Group[@name=$resourceName]">
          <xsl:value-of select="concat($group/../@name, 'Bean')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat(@name, 'Bean')"/>
        </xsl:otherwise>
      </xsl:choose>
    -->
        <xsl:value-of select="concat(@name, 'Bean')"/>    
    </xsl:variable>

    <xsl:variable name="hasDetailButton" select="$groupLayoutStyle='table-form' or Group"/>
    
    <xsl:variable name="sortable">
      <xsl:value-of select="@sortable"/>
    </xsl:variable>

    <xsl:variable name="groupName">
      <xsl:value-of select="@name"/>
    </xsl:variable>
    <!-- LJ -->
    <xsl:variable name="detailsLinkStyle">
      <xsl:value-of select="@detailsLinkStyle"/>
    </xsl:variable>
    <!-- END LJ -->
    <bean___define>
      <xsl:attribute name="id">
        <xsl:value-of select="concat(@name, 'Set')"/>
      </xsl:attribute>
      <xsl:attribute name="name">
        <xsl:value-of select="concat(@name, 'Set')"/>
      </xsl:attribute>   
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.model.DataObjectSet'"/>
      </xsl:attribute>      
      <xsl:attribute name="scope">
        <xsl:value-of select="'session'"/>
      </xsl:attribute>            
    </bean___define>

    <xsl:if test="$hasDetailButton">
      <input type="hidden" name="{concat(@name, 'Rownum')}"/>
    </xsl:if>

    <table width="{$width}">

    <xsl:if test="@useTableRange='true'">
      <xsl:call-template name="jspInclude">
        <xsl:with-param name="page" select="'tableScrollButtons.jsp'"/>
        <xsl:with-param name="param1Name" select="'formName'"/>
        <xsl:with-param name="param1Value" select="$formName"/>
        <xsl:with-param name="param2Name" select="'setName'"/>
        <xsl:with-param name="param2Value" select="concat(@name, 'Set')"/>        
      </xsl:call-template>
    </xsl:if>
  <tr>
  <td>
  <table cellpadding="1" cellspacing="0" class="tableForm" width="100%">
      <xsl:attribute name="name">
        <xsl:value-of select="concat(@name, 'Set')"/>
      </xsl:attribute>

      <tr class="header">
        <xsl:if test="$hasDetailButton">      
          <xsl:if test="$detailsLinkStyle='buttonBegin' or $detailsLinkStyle='buttonBeginEnd'">
            <th class="fixed"></th>      
          </xsl:if>
        </xsl:if>        

        <xsl:for-each select="$voDoc/ViewObject/ViewAttribute">
 <!--          <xsl:message>Processing columnformat <xsl:value-of select="@Name"/></xsl:message> -->
          <!-- get corresponding entity attribute node (if any) -->  
          <xsl:variable name="eoFile">
            <xsl:call-template name="getEoFileName">
              <xsl:with-param name="voDoc" select="$voDoc"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="entityUsage" select="@EntityUsage"/>
          <xsl:variable name="eoAttributeName" select="@EntityAttrName"/>
          <xsl:variable name="eoDoc" select="document($eoFile)"/>
          <xsl:variable name="entityAttributeNode"  select="$eoDoc/Entity/Attribute[$entityUsage!='' and @Name=$eoAttributeName]|current()[$entityUsage='']"/>

          <!-- exclude attribute with custom property DISPLAY set to false -->
          <xsl:variable name="display">
            <xsl:call-template name="setDisplayVariable">
              <xsl:with-param name="entityAttributeNode" select="$entityAttributeNode"/>
              <xsl:with-param name="groupLayoutStyle" select="$groupLayoutStyle"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:if test="$display = 'true'">

               <xsl:call-template name="columnHeader">
                  <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
                  <xsl:with-param name="sortable" select="$sortable"/>
                  <xsl:with-param name="groupName" select="$groupName"/>
                  <xsl:with-param name="formName" select="$formName"/>
               </xsl:call-template>
          </xsl:if>
      </xsl:for-each>
        <xsl:if test="$deleteAllowed='true'">      
          <th class="fixed">
            <bean___message key="{utils:getNlsKey('DELETE_COLUMN_HEADER')}"/>
          </th>      
        </xsl:if>  

        <xsl:if test="$hasDetailButton">      
          <th class="fixed"></th>      
        </xsl:if>        
    
      </tr>


      <logic___iterate id="element">
        <xsl:attribute name="name">
          <xsl:value-of select="concat(@name, 'Set')"/>
        </xsl:attribute>
        <xsl:attribute name="indexId">
          <xsl:value-of select="'index'"/>
        </xsl:attribute>   

        <bean___define id="{concat(@name, 'IsInserting')}" name="{concat(@name, 'Bean')}" property="{concat('jspO= ``', @name, 'IsInserting[``+index+``]`` jspC')}" type="Boolean"/>

          <xsl:if test="$hasDetailButton">
              <xsl:call-template name="jspInclude">
                <xsl:with-param name="page" select="'detailsLinkFunction.jsp'"/>
                <xsl:with-param name="param1Name" select="'rownumField'"/>
                <xsl:with-param name="param1Value" select="concat('document.', @name, 'Bean.', @name, 'Rownum')"/>
                <xsl:with-param name="param2Name" select="'formName'"/>
                <xsl:with-param name="param2Value" select="$formName"/>
              </xsl:call-template>
          </xsl:if>

        <tr class="altRowjspO= (index.intValue() % 2) + 1 jspC ">
        <!-- if detailsLinkStyle = recordDoubleClick then xsl:attribute onDoubleClick -->
          <xsl:if test="$hasDetailButton">
            <xsl:if test="$detailsLinkStyle='recordDoubleClick'">
              <xsl:attribute name="onDblClick">
                 <xsl:text>detailsLink(`jspO= index jspC`);</xsl:text>
              </xsl:attribute>
            </xsl:if>
          </xsl:if>

        <!-- if detailsLinkStyle = buttonBegin or buttonBeginEnd then include jsp detailsbutton -->
          <xsl:if test="$hasDetailButton">
            <xsl:if test="$detailsLinkStyle='buttonBegin' or $detailsLinkStyle='buttonBeginEnd'">
              <td class="fixed" nowrap="true">
    
                <logic___notEqual name="{concat($resourceName, 'Bean')}" 
                                  value="true" 
                                  property="{concat('jspO= ``', $resourceName, 'IsInserting', '[``+index+``]`` jspC')}">          
    
                  <!-- if detailsLinkStyle = buttonEnd or buttonBeginEnd then include jsp detailsbutton -->
                    <xsl:if test="$groupLayoutStyle='table-form'">
                      <xsl:call-template name="jspInclude">
                        <xsl:with-param name="page" select="'detailsLinkButtonTableForm.jsp'"/>
                        <xsl:with-param name="param1Name" select="'index'"/>
                        <xsl:with-param name="param1Value" select="'jspO= index jspC'"/>
                        <xsl:with-param name="param2Name" select="'detailsKey'"/>
                        <xsl:with-param name="param2Value">
                          <xsl:value-of select="utils:getNlsKey('DETAIL_VIEW_BUTTON_LABEL')"/>
                        </xsl:with-param>
                      </xsl:call-template>
                    </xsl:if>          
                    <!-- generate buttons to navigate to each of the detail groups -->  
                    <xsl:if test="$groupLayoutStyle='table'">
                      <xsl:for-each select="Group">
          
                        <xsl:variable name="childDisplayTitle">
                          <xsl:call-template name="getDisplayTitle"/>
                        </xsl:variable>
                      
                        <xsl:call-template name="jspInclude">
                          <xsl:with-param name="page" select="'masterToDetailButton.jsp'"/>
                          <xsl:with-param name="param1Name" select="'rownumField'"/>
                          <xsl:with-param name="param1Value" select="concat('document.', $group/@name, 'Bean.', $group/@name, 'Rownum')"/>
                          <xsl:with-param name="param2Name" select="'targetGroupField'"/>
                          <xsl:with-param name="param2Value" select="concat('document.', $group/@name, 'Bean.', 'targetGroup')"/>
                          <xsl:with-param name="param3Name" select="'targetGroupValue'"/>
                          <xsl:with-param name="param3Value" select="@name"/>                
                          <xsl:with-param name="param4Name" select="'index'"/>
                          <xsl:with-param name="param4Value" select="'jspO= index jspC'"/>
                          <xsl:with-param name="param5Name" select="'key'"/>
                          <xsl:with-param name="param5Value">
          
                            <xsl:value-of select="utils:getNlsKey('CHILD_BUTTON_LABEL', $childDisplayTitle, string:toUpperCase(concat($group/@name, '_TO_', @name, '_BUTTON')))"/>
          
                          </xsl:with-param>
                          <xsl:with-param name="param6Name" select="'formName'"/>
                          <xsl:with-param name="param6Value" select="$formName"/>                
                        </xsl:call-template>
                      </xsl:for-each>
                    </xsl:if>
                 </logic___notEqual>        
               </td>
             </xsl:if> <!-- detailsLinkStyle -->
           </xsl:if> <!-- $hasDetailButton -->
        <xsl:for-each select="$voDoc/ViewObject/ViewAttribute">

          <!-- get corresponding entity attribute node (if any) -->  
          <xsl:variable name="eoFile">
            <xsl:call-template name="getEoFileName">
              <xsl:with-param name="voDoc" select="$voDoc"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="entityUsage" select="@EntityUsage"/>
          <xsl:variable name="eoAttributeName" select="@EntityAttrName"/>
          <xsl:variable name="eoDoc" select="document($eoFile)"/>
          <xsl:variable name="entityAttributeNode"  select="$eoDoc/Entity/Attribute[$entityUsage!='' and @Name=$eoAttributeName]|current()[$entityUsage='']"/>

          <!-- exclude attribute with custom property DISPLAY set to false -->
          <xsl:variable name="display">
            <xsl:call-template name="setDisplayVariable">
              <xsl:with-param name="entityAttributeNode" select="$entityAttributeNode"/>
              <xsl:with-param name="groupLayoutStyle" select="$groupLayoutStyle"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:if test = "$display != 'false'">

             <xsl:choose>
               <!-- You have to remove the false if you want to use the link code
                    of Steven Davelaar --> 
               <xsl:when test="false and $groupLayoutStyle='table-form' and position()=1">
          <td class="fixed">

<!--
Commented out A.J. van Kesteren. This is the code for a link instead of a details
button

added S. Davelaar
           <html___link>
             <xsl:attribute name="href">
               <xsl:text>jspO= ``javascript: document.</xsl:text>
               <xsl:value-of select="$groupName"/>
               <xsl:text>Bean.</xsl:text>
               <xsl:value-of select="$groupName"/>
               <xsl:text>Rownum.value=`` + index +``;submitForm(&quot;details&quot;, &quot;details&quot;, &quot;true&quot;);`` jspC</xsl:text>
             </xsl:attribute>
             <bean___write>
               <xsl:attribute name="name">
                 <xsl:value-of select="concat($groupName,'Bean')"/>
               </xsl:attribute>
               <xsl:attribute name="property">
                 <xsl:value-of select="concat('jspO= ``', $groupName, @Name, '[``+index+``]`` jspC')"/>
               </xsl:attribute>
             </bean___write>
           </html___link>
-->
          </td>                
               </xsl:when>
               <xsl:otherwise>
                  <xsl:call-template name="createFormElement">
                    <xsl:with-param name="attributeName" select="@Name"/>
                    <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
                    <xsl:with-param name="operator" select="''"/>
                    <xsl:with-param name="isFindElement">false</xsl:with-param>
                    <xsl:with-param name="display" select="$display"/>
                    <xsl:with-param name="tableDataSet" select="$tableDataSet"/>
                    <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
                    <xsl:with-param name="group" select="$group"/>
                    <xsl:with-param name="resourceName" select="$resourceName"/>
                    <xsl:with-param name="formName" select="$formName"/>
                    <xsl:with-param name="table" select="'td'"/>
                    <xsl:with-param name="withIndex" select="'true'"/>
                    <xsl:with-param name="elementName" select="concat($resourceName, 'Bean')"/>
                  </xsl:call-template>

               </xsl:otherwise>
             </xsl:choose>

          </xsl:if>
      </xsl:for-each>
        <xsl:if test="$deleteAllowed='true'">      
          <td class="fixed">
              <input type="checkbox" name="{concat('jspO= ``', $resourceName, 'DeleteRow', '[``+index+``]`` jspC')}" value="ok"/>
<!--
also enabled delete checkbox for  new rows: bug 2683576
            <logic___notEqual name="{concat($resourceName, 'Bean')}" 
                              value="true" 
                              property="{concat('jspO= ``', $resourceName, 'IsInserting', '[``+index+``]`` jspC')}">          
              <input type="checkbox" name="{concat('jspO= ``', $resourceName, 'DeleteRow', '[``+index+``]`` jspC')}" value="ok"/>
            </logic___notEqual>
-->
          </td>
        </xsl:if>   

          <xsl:if test="$hasDetailButton">
            <td class="fixed" nowrap="true">
  
              <logic___notEqual name="{concat($resourceName, 'Bean')}" 
                                value="true" 
                                property="{concat('jspO= ``', $resourceName, 'IsInserting', '[``+index+``]`` jspC')}">          
  
                <!-- if detailsLinkStyle = buttonEnd or buttonBeginEnd then include jsp detailsbutton -->
                <xsl:if test="$detailsLinkStyle='buttonEnd' or $detailsLinkStyle='buttonBeginEnd' or $detailsLinkStyle='' ">
                  <xsl:if test="$groupLayoutStyle='table-form'">
                      <xsl:call-template name="jspInclude">
                        <xsl:with-param name="page" select="'detailsLinkButtonTableForm.jsp'"/>
                        <xsl:with-param name="param1Name" select="'index'"/>
                        <xsl:with-param name="param1Value" select="'jspO= index jspC'"/>
                        <xsl:with-param name="param2Name" select="'detailsKey'"/>
                        <xsl:with-param name="param2Value">
                          <xsl:value-of select="utils:getNlsKey('DETAIL_VIEW_BUTTON_LABEL')"/>
                        </xsl:with-param>
                      </xsl:call-template>
                  </xsl:if>          
                </xsl:if>         
              <!-- generate buttons to navigate to each of the detail groups -->  
              <xsl:if test="$groupLayoutStyle='table'">
                <xsl:for-each select="Group">
    
                  <xsl:variable name="childDisplayTitle">
                    <xsl:call-template name="getDisplayTitle"/>
                  </xsl:variable>
                
                  <xsl:call-template name="jspInclude">
                    <xsl:with-param name="page" select="'masterToDetailButton.jsp'"/>
                    <xsl:with-param name="param1Name" select="'rownumField'"/>
                    <xsl:with-param name="param1Value" select="concat('document.', $group/@name, 'Bean.', $group/@name, 'Rownum')"/>
                    <xsl:with-param name="param2Name" select="'targetGroupField'"/>
                    <xsl:with-param name="param2Value" select="concat('document.', $group/@name, 'Bean.', 'targetGroup')"/>
                    <xsl:with-param name="param3Name" select="'targetGroupValue'"/>
                    <xsl:with-param name="param3Value" select="@name"/>                
                    <xsl:with-param name="param4Name" select="'index'"/>
                    <xsl:with-param name="param4Value" select="'jspO= index jspC'"/>
                    <xsl:with-param name="param5Name" select="'key'"/>
                    <xsl:with-param name="param5Value">
    
                      <xsl:value-of select="utils:getNlsKey('CHILD_BUTTON_LABEL', $childDisplayTitle, string:toUpperCase(concat($group/@name, '_TO_', @name, '_BUTTON')))"/>
    
                    </xsl:with-param>
                    <xsl:with-param name="param6Name" select="'formName'"/>
                    <xsl:with-param name="param6Value" select="$formName"/>                
                  </xsl:call-template>
                </xsl:for-each>
              </xsl:if>

            </logic___notEqual>

          </td>
        </xsl:if>            
      </tr>
      </logic___iterate>

    </table>
    </td>
    </tr>
    </table>

  </xsl:template>

  <!-- Set column header for each column to the prompt
   -->
  <xsl:template name="columnHeader">
    <xsl:param name="entityAttribute"/>
    <xsl:param name="sortable"/>
    <xsl:param name="groupName"/>
    <xsl:param name="formName"/>

    <xsl:variable name="jsFormName">
      <xsl:value-of select="concat('document.', $formName)"/>
    </xsl:variable>
  
    <xsl:variable name="prompt">
        <xsl:call-template name="getCustomProperty">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <!--LJ  support TABLE_PROMPT -->
          <xsl:with-param name="propertyName" select="'TABLE_PROMPT'"/>
          <xsl:with-param name="alternativePropertyName" select="'PROMPT'"/>
          <!-- end LJ -->
          <xsl:with-param name="defaultValue" select="@Name"/>
        </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="persistentAttribute">
      <xsl:call-template name="getCustomProperty">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="propertyName" select="'PERSISTENT_ATTRIBUTE'"/>
        <xsl:with-param name="defaultValue" select="@Name"/>
      </xsl:call-template>
    </xsl:variable>  

    <!-- Set column header 
          If the table is sortable, we check whether the attribute is
          queryable. It must be queriable at both EO and VO level!
          If it is, we make the column header sortable
    -->
    <th class="data">
       <xsl:choose>
         <xsl:when test="$sortable='true' and (not (@IsQueriable='false')) and (not ($entityAttribute/@IsQueriable='false'))">
           <a>
             <xsl:attribute name="href">
             <!--
               <xsl:text>javascript:</xsl:text>
               <xsl:value-of select="$jsFormName"/>
               <xsl:text>.value=</xsl:text>
               <xsl:value-of select="@name"/>
               <xsl:text>;</xsl:text>
               <xsl:value-of select="$jsFormName"/>
               <xsl:text>.source=</xsl:text>
               <xsl:value-of select="$groupName"/>               
               <xsl:text>Set;go2(['targetCondition','targetConditionValue'],['event', 'sort']);</xsl:text>
              -->
               <xsl:text>javascript:go2(['targetCondition','targetConditionValue', 'value', 'source'],['event', 'sort','</xsl:text>
               <xsl:value-of select="$persistentAttribute"/>
               <xsl:text>','</xsl:text>
               <xsl:value-of select="$groupName"/>
               <xsl:text>Set']);</xsl:text>
             </xsl:attribute>
           
             <!-- Use _TABLE_ as part of key so we can use different prompt in table layout
             This allows for shorter prompts, amd also prompts without & because
             promptAndAccessKey only applies to form layout -->
             <xsl:variable name="promptKey">
               <xsl:value-of select="string:toUpperCase(concat($groupName,'_TABLE_',@Name))"/>
             </xsl:variable>

             <xsl:call-template name="nlsTextAttribute">
               <xsl:with-param name="attributeName" select="'text'"/>
               <xsl:with-param name="key">
                 <xsl:value-of select="$promptKey"/>
               </xsl:with-param>
               <xsl:with-param name="value">
                 <xsl:value-of select="$prompt"/>
               </xsl:with-param>
               <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
             </xsl:call-template>
           </a>
         </xsl:when>
         <xsl:otherwise>
           <!-- just the column header label -->
             <!-- Use _TABLE_ as part of key so we can use different prompt in table layout
             This allows for shorter prompts, amd also prompts without & because
             promptAndAccessKey only applies to form layout -->
             <xsl:variable name="promptKey">
               <xsl:value-of select="string:toUpperCase(concat($groupName,'_TABLE_',@Name))"/>
             </xsl:variable>

             <xsl:call-template name="nlsTextAttribute">
               <xsl:with-param name="attributeName" select="'text'"/>
               <xsl:with-param name="key">
                 <xsl:value-of select="$promptKey"/>
               </xsl:with-param>
               <xsl:with-param name="value">
                 <xsl:value-of select="$prompt"/>
               </xsl:with-param>
               <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
             </xsl:call-template>
         </xsl:otherwise>
       </xsl:choose>  
    </th>

  </xsl:template> <!-- end columnHeader -->

</xsl:stylesheet>
