<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : PageProcessLib.xsl

Notes       :

Open Issues :

$revision_history$
17-mar-2003   Steven Davelaar
  2.3         
06-nov-2002   Peter Ebell
  2.2         Use oracle.jheadstart.controller.mvc.process.Forward instead of
              oracle.jheadstart.controller.mvc.process.Forward to display pages
10-jun-2002   Peter Ebell
  2.1         Use uixVirtualDir property instead of uixDir property in CreateSimpleUIXPage
14-may-2002   Peter Ebell
  2.0         Pre-prod release
28-mrt-2002   Peter Ebell
  1.1         Rewritten for JHeadstart 9i. Removed obsolete revision history.
25-sep-2001   Steven Davelaar
  1.0         initial creation

*******************************************************************************
$add_to_revision_history
$
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="xml" indent="yes" />


  <!-- find dataObject page process -->
  <xsl:template name="findPage">
    <xsl:param name="pageName"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Create find <xsl:value-of select="$pageName"/> page process </xsl:comment>

    <xsl:call-template name="createSimpleUixPage">
      <xsl:with-param name="fullPageName" select="concat('Find',$pageName,'Page')"/>
      <xsl:with-param name="uixFileName" select="concat('Find',$pageName,'Page',$uixExtension)"/>
    </xsl:call-template>
  </xsl:template> <!-- end find dataObject page process -->

  <!-- select dataObject page process -->
  <xsl:template name="selectPage">
    <xsl:param name="pageName"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Create select <xsl:value-of select="$pageName"/> page process </xsl:comment>

    <xsl:call-template name="createSimpleUixPage">
      <xsl:with-param name="fullPageName" select="concat('Select',$pageName,'Page')"/>
      <xsl:with-param name="uixFileName" select="concat('Select',$pageName,'Page',$uixExtension)"/>
    </xsl:call-template>
  </xsl:template> <!-- end select dataObject page process -->


  <!-- new form page process -->
  <xsl:template name="formPage">
    <xsl:param name="pageName"/>
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> <xsl:value-of select="$pageName"/> page process </xsl:comment>

    <xsl:call-template name="createSimpleUixPage">
      <xsl:with-param name="fullPageName" select="concat($pageName,'Page')"/>
      <xsl:with-param name="uixFileName" select="concat($pageName,'Page',$uixExtension)"/>
    </xsl:call-template>
  </xsl:template>

  <!-- table page process -->
  <xsl:template name="tablePage">
    <xsl:param name="pageName"/> 
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Create <xsl:value-of select="$pageName"/> table page process </xsl:comment>

    <xsl:call-template name="createSimpleUixPage">
      <xsl:with-param name="fullPageName" select="concat($pageName,'TablePage')"/>
      <xsl:with-param name="uixFileName" select="concat($pageName,'TablePage',$uixExtension)"/>
    </xsl:call-template>
  </xsl:template> <!-- end table page process -->


  <xsl:template name="tabsArgument">
    <!-- check whether group is child group, if this is the case,
         the tab number is the master group position (-1 because
         of zero-based index), and the global header is the position
         of the group itself. If the group is not a child group, the
         tab number is the position of the group itself -->
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <xsl:value-of select="concat($masterTabnr,',',position())"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat(position()-1,',0')"/>
        </xsl:otherwise>
      </xsl:choose>
  </xsl:template>

  <!-- P. Ebell - Simplified create UIX page process for v9i_2 -->
  <xsl:template name="createSimpleUixPage">
    <xsl:param name="className">
      <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.Forward'"/>
    </xsl:param>
    <xsl:param name="fullPageName"/>
    <xsl:param name="uixFileName"/>

   <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$fullPageName"/>
      </xsl:attribute>
      <xsl:attribute name="class">
        <xsl:value-of select="$className"/>
      </xsl:attribute>
      <xsl:attribute name="end">true</xsl:attribute>
      <!-- name of uix file -->
      <xsl:comment> Name of uix file </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($uixVirtualDir,$uixFileName)"/>
        </xsl:attribute>
      </argument>
    </process>
  </xsl:template>

    <!-- the internal error page process -->
  <xsl:template name="errorPage">

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> The default Error page process </xsl:comment>
    <process name="ErrorPage"
        class="oracle.jheadstart.controller.mvc.process.Forward" end="true">
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($uixVirtualDir,$defaultErrorPage)"/>
        </xsl:attribute>
      </argument>  
    </process>
  </xsl:template>

</xsl:stylesheet>
