<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : SaveDataProcessLib.xsl

Notes       :

Open Issues :

$revision_history$
15-apr-2004   Sandra Muller
  2.4         fix for bug 3464241: wrong order of arguments if saving detail group
01-sep-2003   Steven Davelaar
  2.3         Added support for queryType
01-aug-2003   Steven Davelaar
  2.2         Added childAttribute as constructor argument in SaveDataObjectSet
              when master-detail situation
17-mar-2003   Steven Davelaar
  2.1         
14-may-2002   Peter Ebell
  2.0         Pre-prod release
28-mrt-2002   Peter Ebell
  1.1         Rewritten for JHeadstart 9i. Removed obsolete revision history.
25-sep-2001   Steven Davelaar
  1.0         initial creation

*******************************************************************************
$add_to_revision_history
$
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="xml" indent="yes" />

  <!-- generic  save dataObject process -->

  <!-- P. Ebell 26-feb-2002 - Rename to saveDataObject later on -->
  <xsl:template name="saveSimpleDataObject">
    <xsl:param name="processName"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="pageFile"/>
    <xsl:param name="dmlButton"/>
    <xsl:param name="doCommit"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>

<!--    <xsl:param name="afterDeleteProcess"/> -->

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Save <xsl:value-of select="$dataObjectKey"/> process </xsl:comment>

    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!-- name of save process -->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.SaveDataObject'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>

      <!-- If the group is a detail group, add information about the master -->
      <xsl:if test="name(..)='Group'">

        <!-- fully qualified Master DataObject interface class name -->
        <xsl:comment> fully qualified Master DataObject interface class name </xsl:comment>
        <argument type="String">
          <xsl:attribute name="value">
            <xsl:value-of select="concat($modelBase,../@dataObject)"/>
          </xsl:attribute>
        </argument>

        <!-- sessionData select key of DataObject -->
        <xsl:comment> sessionData select key of Master DataObject </xsl:comment>
        <argument type="String">
          <xsl:attribute name="value">
            <xsl:value-of select="../@name"/>
          </xsl:attribute>
        </argument>

      </xsl:if>

      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> fully qualified dataObject interface class name </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($modelBase,@dataObject)"/>
        </xsl:attribute>
      </argument>

      <!-- If the group is a detail group, add some more information about the master -->
      <xsl:if test="name(..)='Group'">

        <xsl:comment> Child accessor attribute in master dataObject </xsl:comment>
        <argument type="String">
          <xsl:attribute name="value">
            <xsl:value-of select="@childAttribute"/>
          </xsl:attribute>
        </argument>

      </xsl:if>

      <!-- sessionData select key of DataObject -->
      <xsl:comment> sessionData select key of DataObject </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$dataObjectKey"/>
        </xsl:attribute>
      </argument>

      <!-- sessionData select key that contains Dml action (a.k.a name of save button) -->
      <xsl:comment> sessionData select key that contains Dml action (a.k.a name of save button)  </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$dmlButton"/>
        </xsl:attribute>
      </argument>

      <!-- Page Name -->
      <xsl:comment> Page File Name. This name must exactly match the UIX
           file name, in order for the save dataObject process to work correctly </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$pageFile"/>
        </xsl:attribute>
      </argument>

      <!-- Commit changes? -->
        <xsl:comment> Commit transaction? </xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">
          <xsl:value-of select="$doCommit"/>
        </xsl:attribute>
      </argument>

      <!-- add queryType argument when set and not a detail group -->
      <xsl:if test="@queryType!='' and name(..)!='Group'">
        <xsl:comment> queryType used to insert the dataobject </xsl:comment>
        <argument type="String">
          <xsl:attribute name="value">
            <xsl:value-of select="@queryType"/>
          </xsl:attribute>
        </argument>
      </xsl:if>
    </process>

    <!-- add transitions for save dataObject process -->
    <xsl:call-template name="persistentProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
    </xsl:call-template>

  </xsl:template> <!-- end generic Save Simple DataObject template-->


  <xsl:template name="deleteSimpleDataObject">
    <xsl:param name="processName"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="pageFile"/>
    <xsl:param name="doCommit"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Delete <xsl:value-of select="$dataObjectKey"/> process </xsl:comment>

    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!-- name of save process -->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.SaveDataObject'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>

      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> fully qualified dataObject interface class name </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($modelBase,@dataObject)"/>
        </xsl:attribute>
      </argument>

      <!-- sessionData select key of DataObject -->
      <xsl:comment> sessionData select key of DataObject </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$dataObjectKey"/>
        </xsl:attribute>
      </argument>

      <!-- Dml action -->
      <xsl:comment> Dml action </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="'delete'"/>
        </xsl:attribute>
      </argument>

      <!-- Page Name -->
      <xsl:comment> Page File Name. This name must exactly match the UIX
           file name, in order for the save dataObject process to work correctly </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$pageFile"/>
        </xsl:attribute>
      </argument>

      <!-- Commit changes? -->
        <xsl:comment> Commit transaction? </xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">
          <xsl:value-of select="$doCommit"/>
        </xsl:attribute>
      </argument>

    </process>

    <!-- add transitions for save dataObject process -->
    <xsl:call-template name="persistentProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
    </xsl:call-template>

    <!-- dataObjectDeleted condition ->
    <transition>
      <xsl:attribute name="src">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <xsl:attribute name="condition">dataObjectDeleted</xsl:attribute>
      <xsl:attribute name="dest">
        <xsl:value-of select="$afterDeleteProcess"/>
      </xsl:attribute>
    </transition>
    -->

  </xsl:template> <!-- end generic Save Simple DataObject template-->




  <!-- P. Ebell 18-mrt-2002 - Rename to saveChildDataObjectsForPage later on -->
  <xsl:template name="saveSimpleChildDataObjectsForPage">
    <xsl:param name="pageProcess"/>
    <xsl:param name="pageFile"/>

    <!-- check whether there is a child group with samePage = true.
         If so add saveChildDataObjectSet for this child group -->

    <xsl:variable name="childCount">
      <xsl:value-of select="count(Group[@samePage='true'])"/>
    </xsl:variable>

    <xsl:for-each select="Group[@samePage='true']">
    <!-- determine next process, this is either the get dataObject set
           for the next child group with samePage = true, or the
           display page process -->
      <xsl:variable name="nextProcess">
        <xsl:choose>
          <xsl:when test="./following-sibling::node()[name(.)='Group']/@samePage='true'">
            <xsl:value-of select="concat('Save',./following-sibling::node()/@name,'Table')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$pageProcess"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:call-template name="saveTable">
        <xsl:with-param name="processName" select="concat('Save',@name,'Table')"/>
        <xsl:with-param name="tableName" select="concat(@name,'Set')"/>
        <xsl:with-param name="dataObjectSetKey" select="concat(@name,'Set')"/>
        <xsl:with-param name="successProcess" select="$nextProcess"/>
        <xsl:with-param name="userErrorProcess" select="$pageProcess"/>
        <xsl:with-param name="pageFile" select="$pageFile"/>
        <xsl:with-param name="doCommit">
          <!-- check whether the group is the last child group on same
               page. If so, doCommit is true, otherwise it is false -->
          <xsl:choose>
            <xsl:when test="$childCount=position()">
              <xsl:value-of select="'true'"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="'false'"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>  <!-- end saveSimpleChildDataObjectsForPage template -->



  <!-- template to set success, failure and userError transitions -->
  <xsl:template name="persistentProcessTransitions">
    <xsl:param name="sourceProcess"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>

    <!-- call template to set success, failure transitions -->
    <xsl:call-template name="genericProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$sourceProcess"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
    </xsl:call-template>

    <!-- userError condition -->
    <transition>
      <xsl:attribute name="src">
        <xsl:value-of select="$sourceProcess"/>
      </xsl:attribute>
      <xsl:attribute name="condition">userError</xsl:attribute>
      <xsl:attribute name="dest">
        <xsl:value-of select="$userErrorProcess"/>
      </xsl:attribute>
    </transition>
  </xsl:template> <!-- end generic DataObject Transitions template-->



  <!-- saveMarlinTable process -->
  <xsl:template name="saveTable">
    <xsl:param name="processName"/>
    <xsl:param name="tableName"/>
    <xsl:param name="dataObjectSetKey"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="pageFile"/>
    <xsl:param name="doCommit" select="'true'"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Process to save the <xsl:value-of select="@name"/> table </xsl:comment>

    <!-- dataObject class name argument value depends on whether
         the group is a child group or not -->
    <xsl:variable name="dataObjectClass">
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <xsl:value-of select="concat($modelBase,../@dataObject)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat($modelBase,@dataObject)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!-- name of save process -->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.SaveDataObjectSet'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>
      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> Name of dataObject interface </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$dataObjectClass"/>
        </xsl:attribute>
      </argument>
      <!-- if group s child group, insert of new rows is handled through
           handler of parent dataObject. This requires three additional params:
           * the processinfo name of the master
           * the dataObject name of the child  
           * the childAttribute in master data object-->
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <!-- name of master dataObject in sessionData -->
          <xsl:comment> Name of master dataObject in sessionData </xsl:comment>
          <argument type="String">
            <xsl:attribute name="value">
              <xsl:value-of select="../@name"/>
            </xsl:attribute>
          </argument>
          <!-- name of detail view -->
          <xsl:comment> Child dataObject name </xsl:comment>
          <argument type="String">
            <xsl:attribute name="value">
              <xsl:value-of select="concat($modelBase,@dataObject)"/>
            </xsl:attribute>
          </argument>
          <xsl:comment> Child accessor attribute in master dataObject </xsl:comment>
          <argument type="String">
            <xsl:attribute name="value">
              <xsl:value-of select="@childAttribute"/>
            </xsl:attribute>
          </argument>
        </xsl:when>
      </xsl:choose>
      <!-- name of the table in UIX file -->
      <xsl:comment> Name of the table in uix page </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$tableName"/>
        </xsl:attribute>
      </argument>
      <!-- sessionData select key of DataObject Set -->
      <xsl:comment> Name of table dataObject set in sessionData </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$dataObjectSetKey"/>
        </xsl:attribute>
      </argument>
      <!-- Page Name -->
      <xsl:comment> Page File Name. This name must exactly match the UIX
           file name, in order for the save table process to work correctly </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$pageFile"/>
        </xsl:attribute>
      </argument>
      <!-- commit transaction? -->
      <xsl:comment> commit transaction? </xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">
          <xsl:value-of select="$doCommit"/>
        </xsl:attribute>
      </argument>

      <!-- add queryType argument when set and not a detail group -->
      <xsl:if test="@queryType!='' and name(..)!='Group'">
        <xsl:comment> queryType used to insert the dataobject </xsl:comment>
        <argument type="String">
          <xsl:attribute name="value">
            <xsl:value-of select="@queryType"/>
          </xsl:attribute>
        </argument>
      </xsl:if>

    </process>
    <!-- add transitions for save table rocess -->
    <xsl:call-template name="persistentProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
    </xsl:call-template>

  </xsl:template> <!-- end saveMarlinTable template-->

</xsl:stylesheet>
