<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : ServiceRouterLib.xsl

Notes       :

Open Issues :

$revision_history$
22-may-2003   STEVEN DAVELAAR
  2.9         Added support for detailDisclosure, tableRanges and sortable for detail
              groups in same page
17-apr-2003   STEVEN DAVELAAR
  2.8         cleaned up revision history
16-apr-2003   STEVEN DAVELAAR
  2.6         Include HandletableEvent transition when detailDisclosure set
17-mrt-2003   STEVEN DAVELAAR
  2.5         .
01-jul-2002   Peter Ebell
  2.4         Changed formValues 'targetInfoAlias' and 'targetInfoAliasValue' to 'targetCondition'
              and 'targetConditionValue' because infoAliases do not exist in the SimpleProcessRouter.
19-jun-2002   Peter Ebell
  2.3         Added groupPage transition to table layout groups. This way every type
              of group will have a groupPage transition, which is expected by the
              global headers.
11-jun-2002   Peter Ebell
  2.2         Added condition 'saveTable' to the 'remove' list instead of the 'keep' list
10-jun-2002   Peter Ebell
  2.1         Changed to use oracle.jheadstart.process.SimpleProcessRouter instead of
              oracle.clex.process.ProcessRouter. No longer any need for infoaliases and
              calculation of number of router transitions. Using new methods in XSLTUtils class
              to add transitions.
14-mei-2002   Peter Ebell
  2.0         - Major rewrite, because we no longer need separate processes for insert, update,
                delete and view.
              - Renamed all processes, transitions and infoaliases to use datasourceSetKey
                and datasourceKey names  
14-jan-2002   STEVEN DAVELAAR
  1.3         Changed references to clex.trial classes to clex
14-jan-2002   Peter Ebell
  1.2         Added explicit parameter passing between templates for compatibility with 
              JDeveloper 9i
02-nov-2001   STEVEN DAVELAAR
  1.1         Changed package name of Counter java class
25-sep-2001   STEVEN DAVELAAR
  1.0         initial creation

17-mar-2003   Steven Davelaar
  2.5         
01-jul-2002   Peter Ebell
  2.4         Changed formValues 'targetInfoAlias' and 'targetInfoAliasValue' to 'targetCondition'
              and 'targetConditionValue' because infoAliases do not exist in the SimpleProcessRouter.
19-jun-2002   Peter Ebell
  2.3         Added groupPage transition to table layout groups. This way every type
              of group will have a groupPage transition, which is expected by the
              global headers.
11-jun-2002   Peter Ebell
  2.2         Added condition 'saveTable' to the 'remove' list instead of the 'keep' list
10-jun-2002   Peter Ebell
  2.1         Changed to use oracle.jheadstart.controller.mvc.process.SimpleProcessRouter instead of
              oracle.clex.process.ProcessRouter. No longer any need for infoaliases and
              calculation of number of router transitions. Using new methods in XSLTUtils class
              to add transitions.
14-may-2002   Peter Ebell
  2.0         - Major rewrite, because we no longer need separate processes for insert, update,
                delete and view.
              - Renamed all processes, transitions and infoaliases to use dataObjectSetKey
                and dataObjectKey names  
28-mrt-2002   Peter Ebell
  1.1         Rewritten for JHeadstart 9i. Removed obsolete revision history.
25-sep-2001   Steven Davelaar
  1.0         initial creation

*******************************************************************************
$add_to_revision_history
$
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:cnt="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.Counter"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  exclude-result-prefixes="cnt utils">
  <xsl:output method="xml" indent="yes" />

  <xsl:template name="serviceRouter">
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <!-- the service level router -->
    <process name="ServiceRouter"
            class="oracle.jheadstart.controller.mvc.process.DynamicProcessRouter"
            end="false">
      <!-- A comma delimited list of processinfo objects that could hold the target group name -->
      <xsl:comment> A comma delimited list of processinfo objects that could hold the target group name </xsl:comment>
      <argument type="String" value="targetGroup, PageId"/>
      <!-- Name of processinfo object that holds an infoalias global name of the target group -->
      <xsl:comment>Name of processinfo object that holds an infoalias global name of the target group</xsl:comment>
      <argument type="String" value="targetCondition"/>
      <!-- Name of processinfo object that holds the value to be set for infoalias global name -->
      <xsl:comment> Name of processinfo object that holds the value to be set for infoalias global name </xsl:comment>
      <argument type="String" value="targetConditionValue"/>
      <!-- comma-delimited list of conditions that must be registered -->
      <xsl:comment> Comma-delimited list of conditions that must be registered </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:for-each select="Service//Group[@samePage!='true' or not(@samePage)]">
            <!-- we concat with empty string to work around strange
                 processor bug -->
            <xsl:copy-of select="concat(@name,'')"/>,
          </xsl:for-each>
        </xsl:attribute>
      </argument>
      <!-- default condition -->
      <xsl:comment> default condition </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat(Service//Group/@name,'')"/>
        </xsl:attribute>
      </argument>
      <!-- Remove all routing information after finishing the router? -->
      <xsl:comment> Remove all routing information after finishing the router? </xsl:comment>
      <argument type="Boolean" value="true"/>

      <xsl:if test="Service/@hideUnauthorizedGroupTabs='true'">
        <!-- Check user access to default group? --> 
        <xsl:comment> Check user access to default group? </xsl:comment>
        <argument type="Boolean" value="true"/>
      </xsl:if>  

   </process>

  <!-- add all group transitions to serviceRouter -->
   <xsl:for-each select="Service//Group[@samePage!='true' or not(@samePage)]">
     <xsl:call-template name="serviceGroupTransition"/>
   </xsl:for-each>

  </xsl:template> <!-- end service router template -->

  <!-- template to add one group transition to serviceRouter -->
  <xsl:template name="serviceGroupTransition">
     <transition src="ServiceRouter">
       <xsl:attribute name="condition">
          <xsl:copy-of select="concat(@name,'')"/>
        </xsl:attribute>
       <xsl:attribute name="dest">
          <xsl:copy-of select="concat(@name,'')"/>
       </xsl:attribute>
     </transition>
  </xsl:template>



  <!-- process router -->
  <xsl:template name="simpleProcessRouter">
    <xsl:param name="router"/>
    <xsl:param name="pageName"/>
    <xsl:param name="groupName"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="dataObjectSetKey"/>
    <xsl:param name="initialProcess"/>

    <!-- set initial transition -->
    <xsl:call-template name="transitionInitial">
      <xsl:with-param name="initialProcess" select="$initialProcess"/>      
      <xsl:with-param name="router" select="$router"/>      
    </xsl:call-template>
    
    <!-- set transition for findPage if applicable -->
    <xsl:call-template name="transitionsFindPage">
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
    </xsl:call-template>
    
    <!-- set transitions based on layout style -->
    <xsl:choose>
      <xsl:when test="@layoutStyle='table'">
        <xsl:call-template name="transitionsTablePage">
          <xsl:with-param name="router" select="$router"/>
        </xsl:call-template>
        <!-- add transition to navigate directly to the table Page,
             but call it 'groupPage'. This way every type of group
             has a 'groupPage' transition, which is expected by
             the global headers
        -->
        <xsl:call-template name="routerTransition">
          <xsl:with-param name="condition" select="'groupPage'"/>
          <xsl:with-param name="destination" select="concat($pageName,'TablePage')"/>
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="keep" select="'false'"/>
        </xsl:call-template>

      </xsl:when>
      <xsl:when test="@layoutStyle='table-form'">
        <xsl:call-template name="transitionsTablePage">
          <xsl:with-param name="router" select="$router"/>
        </xsl:call-template>

        <!-- add transition to the GetSelectedDataObject process -->  
        <xsl:call-template name="routerTransition">
          <xsl:with-param name="condition" select="'details'"/>
          <xsl:with-param name="destination" select="concat('Get',$dataObjectKey)"/>
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="keep" select="'false'"/>
        </xsl:call-template>

        <xsl:call-template name="transitionsFormPage">
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
          <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
        </xsl:call-template>

        <!-- add transition to navigate directly to the table Page -->
        <xsl:call-template name="routerTransition">
          <xsl:with-param name="condition" select="'tablePage'"/>
          <xsl:with-param name="destination" select="concat($pageName,'TablePage')"/>
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="keep" select="'false'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@layoutStyle='select-form'">
        <xsl:call-template name="transitionsSelectFormPage">
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
          <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@layoutStyle='form'">
        <xsl:call-template name="transitionsFormPage">
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
          <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>
    <!-- transition to navigate directly to find page --> 
    <xsl:if test="@findPage='true'">
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'findPage'"/>
        <xsl:with-param name="destination" select="concat('Find',$dataObjectKey,'Page')"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'false'"/>
      </xsl:call-template>
    </xsl:if>  

    <!-- transition to navigate to the SelectMasterRow process --> 
    <xsl:if test="../@layoutStyle='table'">
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'selectMasterRow'"/>
        <xsl:with-param name="destination" select="'SelectMasterRow'"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'false'"/>
      </xsl:call-template>
    </xsl:if>  

    <!-- All the transitions are added through methods in the XSLTUtils class.
         We can now send the router process to the output stream, and use the
         information in the XSLTUtils class to retrieve the keep- and remove
         strings and the actual transitions.
    -->
    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$router"/>
      </xsl:attribute>
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.SimpleProcessRouter'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>

        <!-- default condition -->
        <xsl:comment> default condition </xsl:comment>
        <argument type="String" value="initialState"/>

        <!-- infoTable parameters to remove -->
        <xsl:comment> infoTable parameters to remove.</xsl:comment>
        <argument type="String">
          <xsl:attribute name="value">
            <xsl:value-of select="utils:getRemoveConditions()"/>
          </xsl:attribute>
        </argument>  

        <!-- infoTable parameters to keep -->
        <xsl:comment> infoTable parameters to keep.
             We keep these events because other processes depend on their values </xsl:comment>
        <argument type="String">
          <xsl:attribute name="value">
            <xsl:value-of select="utils:getKeepConditions()"/>
          </xsl:attribute>
        </argument>  

    </process>

    <xsl:value-of  disable-output-escaping="yes" select="utils:getTransitions()"/>
   </xsl:template> <!-- end SIMPLE process router skeleton -->


  <!-- generic template for creating process router transition-->
  <xsl:template name="routerTransition">
    <xsl:param name="condition"/>
    <xsl:param name="destination"/>
    <xsl:param name="router"/>
    <xsl:param name="keep"/>

    <xsl:variable name="dummy">
      <xsl:value-of select="utils:addTransition($condition,$router,$destination,$keep)"/>
    </xsl:variable>
<!--
    <xsl:variable name="counter">
      <xsl:value-of select="cnt:getCounter(cnt:new())"/>
    </xsl:variable>

    <transition>
      <xsl:attribute name="src">
        <xsl:value-of select="$router"/>
      </xsl:attribute>
      <xsl:attribute name="condition">
        <xsl:value-of select="concat('tc',$counter)"/>
      </xsl:attribute>
      <xsl:attribute name="dest">
        <xsl:value-of select="$destination"/>
      </xsl:attribute>
    </transition>
-->
  </xsl:template>




  <!-- initial process router transitions -->
  <xsl:template name="transitionInitial">
    <xsl:param name="initialProcess"/>
    <xsl:param name="router"/>
    <!-- initialize java counter function used in routerTransition template
    <xsl:variable name="dummy">
      <xsl:value-of select="cnt:getNewCounter(cnt:new())"/>
    </xsl:variable>
    -->

    <xsl:variable name="dummy">
      <xsl:value-of select="utils:getNewTransitions()"/>
    </xsl:variable>

    <!-- initial transition -->
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'initialState'"/>
      <xsl:with-param name="destination" select="$initialProcess"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'none'"/>
    </xsl:call-template>

  </xsl:template>



  <!-- process router transitions form page navigation -->
  <xsl:template name="transitionsFindPage">
    <xsl:param name="router"/>
    <xsl:param name="dataObjectSetKey"/>

    <xsl:if test="@findPage='true'">
      <!-- find dataObject set that matches criteria in find page -->
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'find'"/>
        <xsl:with-param name="destination" select="concat('Find',$dataObjectSetKey)"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'false'"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>




  <!-- process router transitions select form page navigation -->
  <xsl:template name="transitionsSelectFormPage">
    <xsl:param name="router"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="dataObjectSetKey"/>

    <!-- get dataObject from set -->
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'select'"/>
      <xsl:with-param name="destination" select="concat('Get',$dataObjectKey)"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'false'"/>
    </xsl:call-template>
    <xsl:call-template name="transitionsFormPage">
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
      <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
    </xsl:call-template>
    <!-- transition to navigate directly to select page --> 
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'selectPage'"/>
      <xsl:with-param name="destination" select="concat('Select',$dataObjectKey,'Page')"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'false'"/>
    </xsl:call-template>
  </xsl:template> <!-- end template for select-form page transitions -->


  <!-- process router transitions select form page navigation -->
  <xsl:template name="transitionsFormPage">
    <xsl:param name="router"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="dataObjectSetKey"/>

    <!-- get dataObject from set -->
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'browse'"/>
      <xsl:with-param name="destination" select="concat('Browse',$dataObjectSetKey)"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'true'"/>
    </xsl:call-template>

    <xsl:if test="not(@insert='false')">
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'insert'"/>
        <xsl:with-param name="destination" select="concat('GetDefault',$dataObjectKey)"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'false'"/>
      </xsl:call-template>
    </xsl:if>
    
    <xsl:call-template name="transitionsSaveDataObject">
       <xsl:with-param name="router" select="$router"/>
       <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
    </xsl:call-template>
    <!-- transition to navigate directly to group page --> 
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'groupPage'"/>
      <xsl:with-param name="destination" select="concat($dataObjectKey,'Page')"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'false'"/>
    </xsl:call-template>

    <!-- S. davelaar 22-05-2003
         check whether there is at least one child group with samePage = true 
         and sortable, tableranges or
         detailDisclosure set to true. If so, we add one HandleTableEcent processs
         transition for all these children. We casnnot use separate processes because the
         routing request parameter is always "event". There is another request 
         parameter named "source" that holds the name of the child dataObjectSet.
         So, I also modified HandleTableEvent to check the source parameter if
         if the dataObjectSet cionstructior argument is not set -->
    <xsl:if test="count(Group[@samePage='true'])!='0'
                  and (Group[@sortable='true'] or Group[@useTableRange='true'] 
                  or Group[@detailDisclosure='true'])">

      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'event'"/>
        <xsl:with-param name="destination" select="concat('Handle',@name,'ChildTableEvent')"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'true'"/>
      </xsl:call-template>
    </xsl:if>              
    
  </xsl:template> <!-- end template for form page transitions -->

  <!-- process router transitions form page navigation -->
  <xsl:template name="transitionsSaveDataObject">
    <xsl:param name="router"/>
    <xsl:param name="dataObjectKey"/>

    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'save'"/>
      <xsl:with-param name="destination" select="concat('Save',$dataObjectKey)"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'true'"/>
    </xsl:call-template>

      <xsl:if test="not(@delete='false')">
        <xsl:call-template name="routerTransition">
          <xsl:with-param name="condition" select="'delete'"/>
          <xsl:with-param name="destination" select="concat('Delete',$dataObjectKey)"/>
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="keep" select="'false'"/>
        </xsl:call-template>
      </xsl:if>     
  </xsl:template> <!-- end template for save dataObject transitions -->


  <!-- process router transition for saving dataObject set -->
  <xsl:template name="transitionsSaveDataObjectSet">
    <xsl:param name="router"/>
    
    <!-- save dataObject set-->
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'saveTable'"/>
      <xsl:with-param name="destination" select="concat('Save',@name,'Table')"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'false'"/>
    </xsl:call-template>
  </xsl:template>

  <!-- process router transition for saving dataObject set -->
  <xsl:template name="transitionsTablePage">
    <xsl:param name="router"/>

    <xsl:call-template name="transitionsSaveDataObjectSet">
      <xsl:with-param name="router" select="$router"/>
    </xsl:call-template>

    <xsl:if test="@sortable='true' or @useTableRange='true' or @detailDisclosure='true'">
       <!-- handle table event -->
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'event'"/>
        <xsl:with-param name="destination" select="concat('Handle',@name,'TableEvent')"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'true'"/>
      </xsl:call-template>
    </xsl:if>  

     <!-- tc4: select master row
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="destination" select="'SelectMasterRow'"/>
      <xsl:with-param name="router" select="$router"/>
    </xsl:call-template>
 -->    
  </xsl:template>

</xsl:stylesheet>
