<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : StrutsGetDataProcessLib.xsl

Notes       :

$revision_history$
18-may-2004   Sandra Muller
  1.16        action CreateDefaultDataObjectSet: if newRows is empty set emptyRows to 0
              (otherwise emptyRows value="", which results in warning in struts-config)
16-feb-2004   Peter Ebell
  1.15        Use getDataObjectInterface method on XSLTUtils instead of concatinating modelPersistenceBase
              with dataObjectInterface attributes.
19-nov-2003   Steven Davelaar
  1.14        D0 not set indexParameterName with UIX when selecting master row
              in getSelectedDataObject
19-aug-2003   Aard-Jan van Kesteren
  1.13        Solved bug 3102758. For the action getChildDataObjectSet in case of a 
              DataObjectNotFound result, always do the successProcess instead of the
              dataObjectNotFound process. This way, we can be sure that all the child tables
              on a form with multiple child tables have been initialized. 
07-aug-2003   Aard-Jan van Kesteren
  1.12        Added support for maxNumberQueryHits
06-aug-2003   Steven Davelaar
  1.11        Added beanName parameter to getDefaultDataObjectSet
04-aug-2003   Steven Davelaar
  1.10        
23-jul-2003   Aard-Jan van Kesteren
  1.9         Solved bug 3054375. Also generate a readOnlyBean for a FindDataObjectSet action, 
              to allow for having a select page after a find page.
30-jun-2003   STEVEN DAVELAAR
  1.8         Added support for queryType in FindDataObjectSet
20-jun-2003   STEVEN DAVELAAR
  1.7         Added comment to childAttribute
23-may-2003   STEVEN DAVELAAR
  1.6         childAttribute property in GetChildDataObjectSet now taken
              from childAttribute group property
05-mei-2003   Aard-Jan van Kesteren
  1.5         Checked in after upload from D:\jhsdtdev\design\xslt\StrutsGetDataProcessLib.xsl
              - New CreateFormBean action for find pages without choice field
22-apr-2003   Aard-Jan van Kesteren
  1.4         Checked in after upload from D:\jhsdtdev\design\xslt\StrutsGetDataProcessLib.xsl
              - In the oracle.jheadstart.controller.struts.action.FindDataObjectSet action, the pageName
              is now the name of the find uix page in case of viewType = UIX. In the case of JSP's the
              pageName is the name of a form bean.
              - In the oracle.jheadstart.controller.struts.action.GetSelectedDataObject action, the
              property indexParameterName will only be generated if viewType = JSP or viewType = UIX and
              the current uix page is a select page.
18-apr-2003   Aard-Jan van Kesteren
  1.3         Checked in after upload from D:\jhsdtdev\design\xslt\StrutsGetDataProcessLib.xsl
              Generate form-beans only if view type = 'JSP'
18-apr-2003   STEVEN DAVELAAR
  1.2         Added support for queryType in both "base" and "lookup" getdataObjectSet
24-feb-2003   Aard-Jan van Kesteren
  1.0         Initial creation (based on GetDataProcessLib.xsl)

*******************************************************************************
$add_to_revision_history
$
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  exclude-result-prefixes="cnt utils"  
  >
  <xsl:output method="xml" indent="yes" />


  <!-- get dataObject set process -->
  <xsl:template name="getDataObjectSet">
      <xsl:param name="processName"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="userErrorProcess"/>
      <xsl:param name="dataObjectNotFoundProcess"/>
      <xsl:param name="resultKey"/>
      <xsl:param name="groupName"/>

      <xsl:param name="actionName">
        <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
      </xsl:param>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> action </xsl:comment>

  <!-- First set variables that determine number and sequence of
       constructor arguments
       Note: only set rangeSize if useTableRange is set to true -->
    <xsl:variable name="rangeSize">
      <xsl:choose>
        <xsl:when test="@useTableRange='true'">
          <xsl:value-of select="@tableRangeSize"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="newRows">
      <xsl:choose>
        <xsl:when test="@newRows &gt; 0">
          <xsl:value-of select="@newRows"/>        
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>0</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <!-- should dataObject set be updateable? -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>  

    <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of get dataObject set process -->
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.controller.struts.action.GetDataObjectSet'"/>
      </xsl:attribute>

      <xsl:if test="(@layoutStyle != 'select-form') and (/Service/@viewType = 'JSP')">
 
        <xsl:attribute name="scope">
          <xsl:value-of select="'session'"/>
        </xsl:attribute>

        <xsl:attribute name="name">
          <xsl:value-of select="concat($groupName, 'Bean')"/>
        </xsl:attribute>      
      </xsl:if>

      <xsl:if test="@layoutStyle = 'select-form'">
        <xsl:if test="/Service/@viewType = 'JSP'">             
          <set-property property="readOnlyBean" value="{concat($resultKey, 'Bean')}"/>
        </xsl:if>
      </xsl:if>
      
      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> name of dataObject interface class </xsl:comment>
      <set-property property="dataObjectInterface">
        <xsl:attribute name="value">
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,@dataObject)"/>
        </xsl:attribute>
      </set-property>
      <!-- process result select key of DataObjectSet -->
      <xsl:comment> key used to store result DataObjectSet in SessionData </xsl:comment>
      <set-property property="dataObjectSetName">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </set-property>

      <!-- is DataObjectSet updateable? -->
      <xsl:comment> is DataObjectSet updateable? </xsl:comment>
      <set-property property="isUpdatable">
        <xsl:attribute name="value">
          <xsl:value-of select="$updateable"/>
        </xsl:attribute>
      </set-property>

      <!-- Add rangesize or new rows when set
            Note: ranges and new rows are not supported in combination -->

      <xsl:choose>
        <xsl:when test="$rangeSize!=''">
          <xsl:comment> range size </xsl:comment>
          <set-property property="rangeSize">
            <xsl:attribute name="value">
              <xsl:value-of select="$rangeSize"/>
            </xsl:attribute>
          </set-property>
        </xsl:when>
        <xsl:when test="$newRows!='' and not(@insert='false')">
        
          <xsl:comment> number of new rows (empty lines) displayed </xsl:comment>
          <set-property property="emptyRows">
            <xsl:attribute name="value">
              <xsl:value-of select="$newRows"/>
            </xsl:attribute>
          </set-property>

        </xsl:when>
      </xsl:choose>

      <!-- check for queryType and add additional query arguments when set  -->
      <xsl:if test="@queryType!=''">

        <xsl:comment> Name of the query type</xsl:comment>
        <set-property property="queryType">
          <xsl:attribute name="value">
              <xsl:value-of select="@queryType"/>
          </xsl:attribute>
        </set-property>
      </xsl:if>  

      <xsl:if test="@queryTypeArguments!=''">

        <xsl:comment> SessionData attributes that are used as query type arguments </xsl:comment>
        <set-property property="attributeNames">
          <xsl:attribute name="value">
              <xsl:value-of select="@queryTypeArguments"/>
          </xsl:attribute>
        </set-property>

        <xsl:comment> use attribute name as key? </xsl:comment>
        <set-property property="useAttributeNameAsKey">
          <xsl:attribute name="value">false</xsl:attribute>
        </set-property>
        
      </xsl:if>  

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$dataObjectNotFoundProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>
    </xsl:call-template>
    </action>
  </xsl:template> <!-- end getDataSOurceSet template-->

  <!-- get lookup dataObject set process -->
  <xsl:template name="getLookupDataObjectSet">
      <xsl:param name="successProcess"/>
      <xsl:param name="userErrorProcess"/>
      <xsl:param name="dataObjectNotFoundProcess"/>
      <xsl:param name="resultKey"
             select="@name"/>
      <xsl:param name="groupName"/>
      <xsl:param name="processName"
             select="concat(concat('Get', $groupName), @name)"/>


      <xsl:param name="actionName">
        <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
      </xsl:param>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> action </xsl:comment>

    <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of get dataObject set process -->
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.controller.struts.action.GetDataObjectSet'"/>
      </xsl:attribute>
      <xsl:attribute name="name">
        <xsl:value-of select="concat('Find', $groupName)"/>
      </xsl:attribute>

      <xsl:if test="/Service/@viewType = 'JSP'">             
        <set-property property="readOnlyBean" value="{concat($resultKey, 'Bean')}"/>
      </xsl:if>
      
      <!-- fully qualified lookup dataObject interface class name -->
      <xsl:comment> name of dataObject interface class </xsl:comment>
      <set-property property="dataObjectInterface">
        <xsl:attribute name="value">
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,@dataObject)"/>
        </xsl:attribute>
      </set-property>
      <!-- process result select key of DataObjectSet
           Name of Lookup is used as select key -->
      <xsl:comment> key used to store result DataObjectSet in SessionData </xsl:comment>
      <set-property property="dataObjectSetName">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </set-property>

      <!-- Is updatable? -->
      <xsl:comment>Is Updatable?</xsl:comment>
      <set-property property="isUpdatable">
        <xsl:attribute name="value">false</xsl:attribute>
      </set-property>

      
      <!-- Lookup data source sets always have one new row at top -->
      <xsl:comment> number of new rows (empty lines) </xsl:comment>
      <set-property property="emptyRows">
        <xsl:attribute name="value">1</xsl:attribute>
      </set-property>
      <xsl:comment> show new row (empty line) at top? </xsl:comment>
      <set-property property="emptyRowsAtTop">
        <xsl:attribute name="value">true</xsl:attribute>
      </set-property>

      <!-- check for queryType and add additional query arguments when set  -->
      <xsl:if test="@queryType!=''">

        <xsl:comment> Name of the query type</xsl:comment>
        <set-property property="queryType">
          <xsl:attribute name="value">
              <xsl:value-of select="@queryType"/>
          </xsl:attribute>
        </set-property>
      </xsl:if>  

      <xsl:if test="@queryTypeArguments!=''">

        <xsl:comment> SessionData attributes that are used as query type arguments </xsl:comment>
        <set-property property="attributeNames">
          <xsl:attribute name="value">
              <xsl:value-of select="@queryTypeArguments"/>
          </xsl:attribute>
        </set-property>

        <xsl:comment> use attribute name as key? </xsl:comment>
        <set-property property="useAttributeNameAsKey">
          <xsl:attribute name="value">false</xsl:attribute>
        </set-property>
        
      </xsl:if>  

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$dataObjectNotFoundProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>             
    </xsl:call-template>

    </action>

  </xsl:template> <!-- end getLookupDataObjectSet template  -->


  <!-- get dataObject set process -->
  <xsl:template name="createFindFormBean">
      <xsl:param name="processName"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="groupName"/>
      <xsl:param name="actionName">
        <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
      </xsl:param>      
             
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> action </xsl:comment>

  <!-- First set variables that determine number and sequence of
       constructor arguments
       Note: only set rangeSize if useTableRange is set to true -->

     <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of find dataObject set process -->
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.controller.struts.action.CreateFormBean'"/>
      </xsl:attribute>
      <xsl:attribute name="name">
        <xsl:value-of select="concat('Find', $groupName)"/>
      </xsl:attribute>

      <forward>
        <xsl:attribute name="name">success</xsl:attribute>
        <xsl:attribute name="path">
          <xsl:value-of select="utils:getActionName($successProcess, $groupName)"/>
        </xsl:attribute>
      </forward>
    </action>
  </xsl:template> <!-- end createFindFormBean action-->


  <!-- get dataObject set process -->
  <xsl:template name="findDataObjectSet">
      <xsl:param name="processName"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="userErrorProcess"/>
      <xsl:param name="dataObjectNotFoundProcess"/>
      <xsl:param name="resultKey"/>
      <xsl:param name="findPage"/>
      <xsl:param name="groupName"/>

      <xsl:param name="actionName">
        <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
      </xsl:param>      
             
    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> action </xsl:comment>

  <!-- First set variables that determine number and sequence of
       constructor arguments
       Note: only set rangeSize if useTableRange is set to true -->
    <xsl:variable name="rangeSize">
      <xsl:choose>
        <xsl:when test="@useTableRange='true'">
          <xsl:value-of select="@tableRangeSize"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="newRows" select="@newRows"/>
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>  

    <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of find dataObject set process -->
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.controller.struts.action.FindDataObjectSet'"/>
      </xsl:attribute>

      <xsl:if test="/Service/@viewType = 'JSP'">
        <xsl:attribute name="scope">
          <xsl:value-of select="'session'"/>
        </xsl:attribute>

        <xsl:attribute name="name">
          <xsl:value-of select="concat($groupName, 'Bean')"/>
        </xsl:attribute>      
      </xsl:if>

      <xsl:if test="@layoutStyle = 'select-form'">
        <xsl:if test="/Service/@viewType = 'JSP'">             
          <set-property property="readOnlyBean" value="{concat($resultKey, 'Bean')}"/>
        </xsl:if>
      </xsl:if>
      
      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> name of dataObject interface class </xsl:comment>
      <set-property property="dataObjectInterface">
        <xsl:attribute name="value">
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,@dataObject)"/>
        </xsl:attribute>
      </set-property>
      <!-- process result select key of DataObjectSet -->
      <xsl:comment> key used to store result DataObjectSet in SessionData </xsl:comment>
      <set-property property="dataObjectSetName">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </set-property>
      <!-- file name of find page -->

      <xsl:choose>
        <xsl:when test="/Service/@viewType = 'UIX'">                   
          <xsl:comment> file name of find page </xsl:comment>
          <set-property property="pageName">
            <xsl:attribute name="value">
              <xsl:value-of select="concat('Find', $groupName, 'Page.uix')"/>
            </xsl:attribute>
          </set-property>
        </xsl:when>
        <xsl:otherwise>
          <xsl:comment>Name of Form Bean that contains the properties used to populate 
          the page fields that map of attributes of the datasource that is saved.  
          </xsl:comment>
          <set-property property="pageName">
            <xsl:attribute name="value">
              <xsl:value-of select="concat('Find', $groupName)"/>
            </xsl:attribute>
          </set-property>        
        </xsl:otherwise>
      </xsl:choose>
      
      <!-- Only Add rangesize when set -->
      <xsl:choose>
        <xsl:when test="$rangeSize!=''">
          <xsl:comment> range size </xsl:comment>
          <set-property property="rangeSize">
            <xsl:attribute name="value">
              <xsl:value-of select="$rangeSize"/>
            </xsl:attribute>
          </set-property>
        </xsl:when>
        <xsl:when test="$newRows!='' and not(@insert='false')">
          <xsl:comment> number of new rows (empty lines) displayed </xsl:comment>
          <set-property property="emptyRows">
            <xsl:attribute name="value">
              <xsl:value-of select="$newRows"/>
            </xsl:attribute>
          </set-property>
          <!-- empty rows at top? -->
          <xsl:comment> add new rows (empty lines) at top? </xsl:comment>
          <set-property property="emptyRowsAtTop">
            <xsl:attribute name="value">false</xsl:attribute>
          </set-property>
        </xsl:when>
      </xsl:choose>

      <!-- Is the result set updateable? -->
      <xsl:comment> Is the result set updateable? </xsl:comment>
      <set-property property="isUpdatable">
        <xsl:attribute name="value">
          <xsl:value-of select="$updateable"/>
        </xsl:attribute>  
      </set-property>

      <xsl:if test="@queryType!=''">
        <xsl:comment> queryType </xsl:comment>
        <set-property property="queryType">
          <xsl:attribute name="value">
            <xsl:value-of select="@queryType"/>
          </xsl:attribute>  
        </set-property>
      </xsl:if>  

      <xsl:if test="@maxNumberQueryHits != ''">
        <xsl:comment>The maximum number of query hits</xsl:comment>
        <set-property property="maxRows" value="{@maxNumberQueryHits}"/>
      </xsl:if>

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$dataObjectNotFoundProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>             
    </xsl:call-template>

    </action>

  </xsl:template> <!-- end findDataSOurceSet template-->


    <!-- get child dataObject set process -->
  <xsl:template name="getChildDataObjectSet">
      <xsl:param name="processName"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="userErrorProcess"/>
      <xsl:param name="dataObjectNotFoundProcess"/>
      <xsl:param name="masterKey"/>
      <xsl:param name="resultKey"/>
      <xsl:param name="groupName"/>

      <xsl:param name="actionName">
        <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
      </xsl:param>      

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> action </xsl:comment>

    <xsl:variable name="rangeSize">
      <xsl:choose>
        <xsl:when test="@useTableRange='true'">
          <xsl:value-of select="@tableRangeSize"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <!-- Should dataObject set be updateable? -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>  
    
    <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of get dataObject set process -->
      <xsl:attribute name="type">
        <xsl:text>oracle.jheadstart.controller.struts.action.GetChildDataObjectSet</xsl:text>
      </xsl:attribute>

      <xsl:if test="(@layoutStyle != 'select-form') and (/Service/@viewType = 'JSP')">
 
        <xsl:attribute name="scope">
          <xsl:value-of select="'session'"/>
        </xsl:attribute>

        <xsl:attribute name="name">
<!--          <xsl:value-of select="concat($groupName, 'Bean')"/>
S. Davelaar, does not wotk in samep page!
-->
          <xsl:value-of select="concat(@name, 'Bean')"/>
        </xsl:attribute>      
      </xsl:if>

      <xsl:if test="@layoutStyle = 'select-form' and /Service/@viewType = 'JSP'">
          <set-property property="readOnlyBean" value="{concat($resultKey, 'Bean')}"/>
      </xsl:if>
      
      <!-- fully qualified master dataObject interface class name -->
      <xsl:comment> name of master dataObject interface </xsl:comment>
      <set-property property="masterDataObjectInterface">
        <xsl:attribute name="value">
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,../@dataObject)"/>
        </xsl:attribute>
      </set-property>
      <!-- name under which master dataObject is stored in infoTabl -->
      <xsl:comment> select key of master dataObject in SessionData</xsl:comment>
      <set-property property="masterDataObjectName">
        <xsl:attribute name="value">
          <xsl:value-of select="$masterKey"/>
        </xsl:attribute>
      </set-property>
      <!-- name of detail view resource name -->
      <xsl:comment> name of detail dataObject interface </xsl:comment>
      <set-property property="dataObjectInterface">
        <xsl:attribute name="value">
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,@dataObject)"/>
        </xsl:attribute>                              
      </set-property>
      <!-- process result select key of ChildDataObjectSet -->
      <xsl:comment> select key of result child DataObjectSet in SessionData </xsl:comment>
      <set-property property="dataObjectSetName">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </set-property>
      <xsl:comment> attribute in master object that returns the collection of child objects</xsl:comment>
      <set-property property="childAttribute">
        <xsl:attribute name="value">
          <xsl:value-of select="@childAttribute"/>
        </xsl:attribute>
      </set-property>
      <!-- is child DataObjectSet updateable? -->
      <xsl:comment> is child DataObjectSet updateable? </xsl:comment>
      <set-property property="isUpdatable">
        <xsl:attribute name="value">
          <xsl:value-of select="$updateable"/>
        </xsl:attribute>
      </set-property>

      <!-- if rangeSize is specified, we set the ranbgeSize as
           arguments. Otherwise we set the numbe rof new rows and
           rows at top as ast twpo arguments (ranges are not supported
           in combination with new rows (inserts) -->
      <xsl:choose>
        <xsl:when test="$rangeSize!=''">
          <!-- table range size argument -->
          <xsl:comment> table range size </xsl:comment>
          <set-property property="rangeSize">
            <xsl:attribute name="value">
              <xsl:value-of select="$rangeSize"/>
            </xsl:attribute>
          </set-property>
        </xsl:when>
        <xsl:otherwise>
          <!-- number of empty rows -->
          <xsl:comment> number of new rows (i.e. empty lines) </xsl:comment>
          <set-property property="emptyRows">
            <xsl:attribute name="value">
              <xsl:choose>
                <xsl:when test="@newRows!='' and not(@insert='false') and contains(@layoutStyle,'table')">
                  <xsl:value-of select="@newRows"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="'0'"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:attribute>
          </set-property>
          <!-- empty rows at top? -->
          <xsl:comment> add new rows (empty lines) at top? </xsl:comment>
          <set-property property="emptyRowsAtTop">
            <xsl:attribute name="value">false</xsl:attribute>
          </set-property>
        </xsl:otherwise>
      </xsl:choose>
      
          
    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$successProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>             
    </xsl:call-template>

    </action>

  </xsl:template> <!-- end getChildDataObjectSet template-->


  <!-- Create Default DataObject process -->
  <xsl:template name="getDefaultDataObject">
    <xsl:param name="processName"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="resultKey"/>
    <xsl:param name="groupName"/>

    <xsl:param name="actionName">
      <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
    </xsl:param>    

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> action </xsl:comment>

    <action path="{$actionName}" type="oracle.jheadstart.controller.struts.action.CreateDefaultDataObject">
      <xsl:if test="/Service/@viewType = 'JSP'">      
        <xsl:attribute name="name">
          <xsl:value-of select="concat($groupName, 'Bean')"/>
        </xsl:attribute>
        <xsl:attribute name="scope">
          <xsl:value-of select="'session'"/>
        </xsl:attribute>
      </xsl:if>

      <!-- process result select -->
      <xsl:comment> select key of result DataObject in SessionData </xsl:comment>
      <set-property property="dataObjectName" value="{$resultKey}"/>

     <!-- resource name -->
      <xsl:comment> name of dataObject interface </xsl:comment>
      <set-property property="dataObjectInterface" value="{utils:getDataObjectInterface($modelBase,@dataObject)}"/>

    <xsl:call-template name="persistentProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>      
    </xsl:call-template>

    </action>

  </xsl:template> <!-- end GetDefaultDataObject template-->

  <!-- Create Default DataObjectSet process -->
  <xsl:template name="getDefaultDataObjectSet">
    <xsl:param name="processName"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="resultKey"/>
    <xsl:param name="beanName" select="''"/>
    <xsl:param name="groupName"/>

    <xsl:param name="actionName">
      <xsl:value-of select="utils:getActionName($processName, $groupName)"/>
    </xsl:param>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> action </xsl:comment>

    <action path="{$actionName}" type="oracle.jheadstart.controller.struts.action.CreateDefaultDataObjectSet">
      <xsl:if test="/Service/@viewType = 'JSP'">
        <xsl:attribute name="name">
          <xsl:value-of select="$beanName"/>
        </xsl:attribute>
        <xsl:attribute name="scope">
          <xsl:value-of select="'session'"/>
        </xsl:attribute>
      </xsl:if>

      <!-- process result select -->
      <xsl:comment> select key of result DataObject in SessionData </xsl:comment>
      <set-property property="dataObjectSetName" value="{$resultKey}"/>

     <!-- resource name -->
      <xsl:comment> name of dataObject interface </xsl:comment>
      <set-property property="dataObjectInterface" value="{utils:getDataObjectInterface($modelBase,@dataObject)}"/>

     <!-- number of rows in default dataObjectSet -->
      <xsl:comment> number of rows in default dataObjectSet </xsl:comment>
      <set-property property="emptyRows">
        <xsl:attribute name="value">
          <xsl:choose>
            <xsl:when test="@newRows!=''">
              <xsl:value-of select="@newRows"/>        
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>0</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
      </set-property>

      <xsl:call-template name="persistentProcessTransitions">
        <xsl:with-param name="sourceProcess" select="$processName"/>
        <xsl:with-param name="successProcess" select="$successProcess"/>
        <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
        <xsl:with-param name="groupName" select="$groupName"/>      
      </xsl:call-template>

    </action>

  </xsl:template> <!-- end GetDefaultDataObjectSet template-->

  <xsl:template name="getDataObjectTransitions">

    <xsl:param name="sourceProcess"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="dataObjectNotFoundProcess"/>
    <xsl:param name="groupName"/>

    <xsl:call-template name="persistentProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$sourceProcess"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>      
    </xsl:call-template>

    <!-- dataObjectNotFound condition -->
    <forward>
      <xsl:attribute name="name">dataObjectNotFound</xsl:attribute>
      <xsl:attribute name="path">
        <xsl:value-of select="utils:getActionName($dataObjectNotFoundProcess, $groupName)"/>
      </xsl:attribute>
    </forward>
  </xsl:template> <!-- end getDataObjectTransitions template-->

  <!-- get dataObject process ->
  <xsl:template name="getDataObject">
    <xsl:param name="processName"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="dataObjectNotFoundProcess"/>
    <xsl:param name="selectKey"
             select="concat(@dataObject,@identifier)"/>
    <xsl:param name="resultKey"
             select="@dataObject"/>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$processName"/> process </xsl:comment>

    <process>
      <xsl:attribute name="name">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <!- name of get dataObject process ->
      <xsl:attribute name="class">
        <xsl:value-of select="'oracle.jheadstart.controller.mvc.process.GetDataObject'"/>
      </xsl:attribute>
      <xsl:attribute name="end">false</xsl:attribute>
      <!- fully qualified dataObject interface class name ->
      <xsl:comment> name of dataObject interface class</xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($package
                                      ,'.resource.'
                                      ,@dataObject)"/>
        </xsl:attribute>
      </argument>
      <!- the primary key column of the DataObject used to get the object ->
      <xsl:comment> select key of the identifier (primary key) of the DataObject in SessionData</xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$selectKey"/>
        </xsl:attribute>
      </argument>
      <!- process result select key of DataObject ->
      <xsl:comment> select key of result DataObject in SessionData </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </argument>
      <!- is usage data source updatable  ->
      <xsl:comment> is DataObject updatable? </xsl:comment>
      <argument type="Boolean" value="true"/>

     </process>

    <!- call template to add process transitions ->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess"
             select="$processName"/>
      <xsl:with-param name="successProcess"
             select="$successProcess"/>
      <xsl:with-param name="userErrorProcess"
             select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess"
             select="$dataObjectNotFoundProcess"/>
    </xsl:call-template>

  </xsl:template> <!- end getDataSOurce for edit template->
-->

  <!-- get DataObject from Set process -->
  <xsl:template name="getSelectedDataObject">
    <xsl:param name="dataObjectSetKey"/>
    <xsl:param name="dataObjectName"/>
    <xsl:param name="processName"/>
<!--  <xsl:param name="buttonName"/> -->
    <xsl:param name="resultKey"/>
    <xsl:param name="selectKey"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="dataObjectNotFoundProcess"/>
    <xsl:param name="groupName"/>

    <xsl:param name="actionName">
      <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
    </xsl:param>    

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> action </xsl:comment>

    <!-- should dataObject set be updateable? -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>  

    <action path="{$actionName}" 
            type="oracle.jheadstart.controller.struts.action.GetSelectedDataObject"
    >

      <xsl:if test="/Service/@viewType = 'JSP'">      

        <xsl:attribute name="name">
          <xsl:value-of select="concat($resultKey, 'Bean')"/>
        </xsl:attribute>
        <xsl:attribute name="scope">
          <xsl:value-of select="'session'"/>
        </xsl:attribute>
      </xsl:if>


      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> name of dataObject set to get dataObject from</xsl:comment>
      <set-property property="dataObjectSetName" value="{$dataObjectSetKey}"/>

      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> name of dataObject interface class</xsl:comment>
      <set-property property="dataObjectInterface" value="{utils:getDataObjectInterface($modelBase,@dataObject)}"/>

      <!-- when groupName=resultKey we use this action to select from select page, so we need
           indexParameterName. If they are not the same, we use this action to select master row
           in which case we should not specify indexParameterName beacuse we use
           SingleSelectionBan inside getSelectedDataObject-->
      <xsl:if test="(/Service/@viewType = 'JSP') or (@layoutStyle = 'select-form' and $groupName=$resultKey)">      
        <!-- the primary key column of the DataObject used to get the object -->
        <xsl:comment> select key of the identifier (primary key) of the DataObject in SessionData</xsl:comment>
        <set-property property="indexParameterName" value="{concat($resultKey, 'Rownum')}"/>
      </xsl:if>

      <!-- name of the button 
      <xsl:comment> name of the button </xsl:comment>
      <argument type="String" value="{$buttonName}"/>
      -->
      
      <!-- process result select key of DataObject -->
      <xsl:comment> select key of result DataObject in SessionData </xsl:comment>
      <set-property property="dataObjectName" value="{$resultKey}"/>

      <!-- is the dataObject updateable? -->
      <xsl:comment> is the DataObject updateable? </xsl:comment>
      <set-property property="isUpdatable" value="{$updateable}"/>

    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess" select="$dataObjectNotFoundProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>      
    </xsl:call-template>

     </action>

  </xsl:template> <!-- end get DataObject From Set-->



  <!-- Browse DataObject from Set process -->
  <xsl:template name="browseDataObjectSet">
    <xsl:param name="processName"/>
    <xsl:param name="dataObjectSetKey"/>
    <xsl:param name="buttonName"/>
    <xsl:param name="resultKey"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="dataObjectNotFoundProcess"/>
    <xsl:param name="groupName"/>

    <xsl:param name="actionName">
      <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
    </xsl:param>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> action </xsl:comment>

    <!-- should dataObject set be updateable? -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="not(@update='false')">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'false'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable> 

    <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of get dataObject process -->
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.controller.struts.action.BrowseDataObjectSet'"/>
      </xsl:attribute>

      <xsl:if test="/Service/@viewType = 'JSP'">            
        <xsl:attribute name="name">
          <xsl:value-of select="concat($groupName, 'Bean')"/>
        </xsl:attribute>
        <xsl:attribute name="scope">
          <xsl:value-of select="'session'"/>
        </xsl:attribute>
      </xsl:if>

      <!-- name of dataObject set to get dataObject from -->
      <xsl:comment> name of dataObject set to get dataObject from</xsl:comment>
      <set-property property="dataObjectSetName">
        <xsl:attribute name="value">
          <xsl:value-of select="$dataObjectSetKey"/>
        </xsl:attribute>
      </set-property>

      <!-- process result select key of DataObject -->
      <xsl:comment> select key of result DataObject in SessionData </xsl:comment>
      <set-property property="dataObjectName">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </set-property>

      <!-- name of the button -->
      <xsl:comment> SessionData select key of browse action (name of the browse button </xsl:comment>
      <set-property property="browseActionParameterName">
        <xsl:attribute name="value">
          <xsl:value-of select="$buttonName"/>
        </xsl:attribute>
      </set-property>
     
      <!-- Is dataObject updateable -->
      <xsl:comment>Should the result dataObject be updateable?</xsl:comment>
      <set-property property="isUpdatable">
        <xsl:attribute name="value">
          <xsl:value-of select="$updateable"/>
        </xsl:attribute>
      </set-property>
    <!-- call template to add process transitions -->
    <xsl:call-template name="getDataObjectTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="dataObjectNotFoundProcess" select="$dataObjectNotFoundProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>      
    </xsl:call-template>

    </action>

  </xsl:template> <!-- end Browse DataObject Set-->









  <!-- get selected table row for edit process -->
  <xsl:template name="getSelectedRowForEdit">
      <xsl:param name="processName"
        select="concat('Get',@dataObject,'ForEdit')"/>
      <xsl:param name="successProcess"/>
      <xsl:param name="tableName"
             select="concat(@dataObject,'Set')"/>
      <xsl:param name="tableDataSetKey"
             select="concat(@dataObject,'Set')"/>
      <xsl:param name="resultKey"
             select="@dataObject"/>
      <xsl:param name="groupName"/>

    <xsl:param name="actionName">
      <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
    </xsl:param>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select="$actionName"/> process </xsl:comment>

    <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of get dataObject process -->
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.controller.struts.action.GetSelectedRow'"/>
      </xsl:attribute>
      <!-- table name -->
      <xsl:comment> table name </xsl:comment>
      <set-property property="tableName">
        <xsl:attribute name="value">
          <xsl:value-of select="$tableName"/>
        </xsl:attribute>
      </set-property>
      <!-- dataObject set name used to populate table -->
      <xsl:comment> key of table data source set in SessionData</xsl:comment>
      <set-property property="tableDataSetKey">
        <xsl:attribute name="value">
          <xsl:value-of select="$tableDataSetKey"/>
        </xsl:attribute>
      </set-property>
      <!-- process result select key of selected DataObject -->
      <xsl:comment> key of selected dataObject in SessionData </xsl:comment>
      <argument property="resultKey">
        <xsl:attribute name="value">
          <xsl:value-of select="$resultKey"/>
        </xsl:attribute>
      </argument>
     </action>

    <!-- call template to add process transitions -->
    <xsl:call-template name="genericProcessTransitions">
      <xsl:with-param name="sourceProcess"
        select="$processName"/>
      <xsl:with-param name="successProcess"
        select="$successProcess"/>
      <xsl:with-param name="groupName"/>      
    </xsl:call-template>

  </xsl:template> <!-- end selected row for edit template-->

</xsl:stylesheet>
