<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : StrutsSaveDataProcessLib.xsl

Notes       :

$revision_history$
16-feb-2004   Peter Ebell
  1.8         Use getDataObjectInterface method on XSLTUtils instead of concatinating modelPersistenceBase
              with dataObjectInterface attributes.
01-sep-2003   Steven Davelaar
  1.7         Added support for queryType
08-aug-2003   Steven Davelaar
  1.6         Adde deleteRowParameter to SaveDataObjectSet when view JSP
20-jun-2003   STEVEN DAVELAAR
  1.5         - Save action of detail group generated MVC Framework-style
              arguments instead of Struts set-property elements
              - childAttribute not set in SaveDataObject on detail group
22-apr-2003   AARDJ
  1.4         Checked in after upload from D:\jhsdtdev\design\xslt\StrutsSaveDataProcessLib.xsl
              - Added attribute pageName to action SaveDataObject in case viewType = UIX
              - Added attribute pageName to action SaveDataObjectSet in case viewType = UIX
18-apr-2003   AARDJ
  1.3         Checked in after upload from D:\jhsdtdev\design\xslt\StrutsSaveDataProcessLib.xsl
              Generate form-beans only if view type = 'JSP'
18-apr-2003   STEVEN DAVELAAR
  1.2         renamings
24-feb-2003  Aard-Jan van Kesteren
  1.0         Initial creation (based on SaveDataProcessLib.xsl)

*******************************************************************************
$add_to_revision_history
$
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  exclude-result-prefixes="cnt utils"  
  >
  <xsl:output method="xml" indent="yes" />

  <!-- generic  save dataObject process -->

  <!-- P. Ebell 26-feb-2002 - Rename to saveDataObject later on -->
  <xsl:template name="saveSimpleDataObject">
    <xsl:param name="processName"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="pageFile"/>
    <xsl:param name="dmlButton"/>
    <xsl:param name="doCommit"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="groupName"/>

    <xsl:param name="actionName">
      <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
    </xsl:param>    

<!--    <xsl:param name="afterDeleteProcess"/> -->

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Save <xsl:value-of select="$actionName"/> action </xsl:comment>

    <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of save process -->
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.controller.struts.action.SaveDataObject'"/>
      </xsl:attribute>
      
      <xsl:if test="/Service/@viewType = 'JSP'">            
        <xsl:attribute name="name">
          <xsl:value-of select="concat($groupName, 'Bean')"/>
        </xsl:attribute>
        <xsl:attribute name="scope">
          <xsl:value-of select="'session'"/>
        </xsl:attribute>
      </xsl:if>

      <!-- If the group is a detail group, add information about the master -->  
      <xsl:if test="name(..)='Group'">
        <!-- fully qualified Master DataObject interface class name -->
        <xsl:comment> fully qualified Master DataObject interface class name </xsl:comment>
        <set-property property="masterDataObjectInterface">
          <xsl:attribute name="value">
            <xsl:value-of select="utils:getDataObjectInterface($modelBase,../@dataObject)"/>
          </xsl:attribute>
        </set-property>

        <!-- SessionData select key of DataObject -->
        <xsl:comment> SessionData select key of Master DataObject </xsl:comment>
        <set-property property="masterDataObjectName">
          <xsl:attribute name="value">
            <xsl:value-of select="../@name"/>
          </xsl:attribute>
        </set-property>

        <xsl:comment> attribute in parent object that returns the collection of child objects</xsl:comment>
        <set-property property="childAttribute">
          <xsl:attribute name="value">
            <xsl:value-of select="@childAttribute"/>
          </xsl:attribute>
        </set-property>

      </xsl:if>  

      <!-- SessionData select key of DataObject -->
      <xsl:comment> SessionData select key of DataObject </xsl:comment>
      <set-property property="dataObjectName">
        <xsl:attribute name="value">
          <xsl:value-of select="$dataObjectKey"/>
        </xsl:attribute>
      </set-property>

      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> fully qualified dataObject interface class name </xsl:comment>
      <set-property property="dataObjectInterface">
        <xsl:attribute name="value">
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,@dataObject)"/>
        </xsl:attribute>
      </set-property>

      <!-- SessionData select key that contains Dml action (a.k.a name of save button) -->
      <xsl:comment> SessionData select key that contains Dml action (a.k.a name of save button)  </xsl:comment>
      <set-property property="persistActionParameterName">
        <xsl:attribute name="value">
          <xsl:value-of select="$dmlButton"/>
        </xsl:attribute>
      </set-property>

      <!-- Commit changes? -->
        <xsl:comment> Commit transaction? </xsl:comment>
      <set-property property="doCommit">
        <xsl:attribute name="value">
          <xsl:value-of select="$doCommit"/>
        </xsl:attribute>
      </set-property>

      <xsl:if test="/Service/@viewType = 'UIX'">
        <set-property property="pageName">
          <xsl:attribute name="value">
            <xsl:value-of select="concat($dataObjectKey, 'Page.uix')"/>
          </xsl:attribute>
        </set-property>
      </xsl:if>

      <xsl:if test="@queryType!=''">

        <xsl:comment> Name of the query type</xsl:comment>
        <set-property property="queryType">
          <xsl:attribute name="value">
              <xsl:value-of select="@queryType"/>
          </xsl:attribute>
        </set-property>
      </xsl:if>  
      
<!-- P. Ebell - Let's try it without this bit 
      <!- Uk Attribute that must be added to SessionData ->
      <xsl:comment> Uk Attribute that must be added to SessionData </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="@identifier"/>
        </xsl:attribute>
      </argument>

      <!- Name of Uk Attribute in SessionData ->
      <xsl:comment> Name of Uk Attribute in SessionData </xsl:comment>
      <argument type="String">
        <xsl:attribute name="value">
          <xsl:value-of select="concat($dataObjectKey,@identifier)"/>
        </xsl:attribute>
      </argument>
-->      

    <!-- add transitions for save dataObject process -->
    <xsl:call-template name="persistentProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>
    </xsl:call-template>

    <!-- resourceDeleted condition
    <transition>
      <xsl:attribute name="src">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <xsl:attribute name="condition">resourceDeleted</xsl:attribute>
      <xsl:attribute name="dest">
        <xsl:value-of select="$afterDeleteProcess"/>
      </xsl:attribute>
    </transition>
    -->

    </action>
    
  </xsl:template> <!-- end generic Save Simple DataObject template-->


  <xsl:template name="deleteSimpleDataObject">
    <xsl:param name="processName"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="pageFile"/>
    <xsl:param name="doCommit"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="groupName"/>

    <xsl:param name="actionName">
      <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
    </xsl:param>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Delete <xsl:value-of select="$dataObjectKey"/> action </xsl:comment>

    <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of save process -->
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.controller.struts.action.SaveDataObject'"/>
      </xsl:attribute>

      <!-- SessionData select key of DataObject -->
      <xsl:comment> SessionData select key of DataObject </xsl:comment>
      <set-property property="dataObjectName">
        <xsl:attribute name="value">
          <xsl:value-of select="$dataObjectKey"/>
        </xsl:attribute>
      </set-property>

      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> fully qualified dataObject interface class name </xsl:comment>
      <set-property property="dataObjectInterface">
        <xsl:attribute name="value">
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,@dataObject)"/>
        </xsl:attribute>
      </set-property>

      <!-- Dml action -->
      <xsl:comment> Dml action </xsl:comment>
      <set-property property="persistAction">
        <xsl:attribute name="value">
          <xsl:value-of select="'delete'"/>
        </xsl:attribute>
      </set-property>

      <!-- Page Name -->
      <xsl:comment> Page File Name. This name must exactly match the UIX
           file name, in order for the save dataObject action to work correctly </xsl:comment>
      <set-property property="pageName">
        <xsl:attribute name="value">
          <xsl:value-of select="$pageFile"/>
        </xsl:attribute>
      </set-property>

      <!-- Commit changes? -->
        <xsl:comment> Commit transaction? </xsl:comment>
      <set-property property="doCommit">
        <xsl:attribute name="value">
          <xsl:value-of select="$doCommit"/>
        </xsl:attribute>
      </set-property>

    <!-- add transitions for save dataObject process -->
    <xsl:call-template name="persistentProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>      
    </xsl:call-template>

    <!-- resourceDeleted condition ->
    <transition>
      <xsl:attribute name="src">
        <xsl:value-of select="$processName"/>
      </xsl:attribute>
      <xsl:attribute name="condition">resourceDeleted</xsl:attribute>
      <xsl:attribute name="dest">
        <xsl:value-of select="$afterDeleteProcess"/>
      </xsl:attribute>
    </transition>
    -->

    </action>

  </xsl:template> <!-- end generic Save Simple DataObject template-->




  <!-- P. Ebell 18-mrt-2002 - Rename to saveChildDataObjectsForPage later on -->
  <xsl:template name="saveSimpleChildDataObjectsForPage">
    <xsl:param name="pageProcess"/>
    <xsl:param name="pageFile"/>
    <xsl:param name="groupName"/>

    <!-- check whether there is a child group with samePage = true.
         If so add saveChildDataObjectSet for this child group -->

    <xsl:variable name="childCount">
      <xsl:value-of select="count(Group[@samePage='true'])"/>
    </xsl:variable>

    <xsl:for-each select="Group[@samePage='true']">
    <!-- determine next process, this is either the get dataObject set
           for the next child group with samePage = true, or the
           display page process -->
      <xsl:variable name="nextProcess">
        <xsl:choose>
          <xsl:when test="./following-sibling::node()[name(.)='Group']/@samePage='true'">
            <xsl:value-of select="concat('Save',./following-sibling::node()/@name,'Table')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$pageProcess"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>


      <xsl:call-template name="saveTable">
        <xsl:with-param name="processName" select="concat('Save',@name,'Table')"/>
        <xsl:with-param name="tableName" select="concat(@name,'Set')"/>
        <xsl:with-param name="dataObjectSetKey" select="concat(@name,'Set')"/>
        <xsl:with-param name="successProcess" select="$nextProcess"/>
        <xsl:with-param name="userErrorProcess" select="$pageProcess"/>
        <xsl:with-param name="pageFile" select="$pageFile"/>
        <xsl:with-param name="groupName" select="$groupName"/>
        <xsl:with-param name="doCommit">
          <!-- check whether the group is the last child group on same
               page. If so, doCommit is true, otherwise it is false -->
          <xsl:choose>
            <xsl:when test="$childCount=position()">
              <xsl:value-of select="'true'"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="'false'"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:for-each>
  </xsl:template>  <!-- end saveSimpleChildDataObjectsForPage template -->



  <!-- template to set success, failure and userError transitions -->
  <xsl:template name="persistentProcessTransitions">
    <xsl:param name="sourceProcess"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="groupName"/>

    <!-- call template to set success, failure transitions -->
    <xsl:call-template name="genericProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$sourceProcess"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>      
    </xsl:call-template>

    <!-- userError condition -->
    <forward>
      <xsl:attribute name="name">userError</xsl:attribute>
      <xsl:attribute name="path">
        <xsl:value-of select="utils:getActionName($userErrorProcess, $groupName)"/>
      </xsl:attribute>
    </forward>
  </xsl:template> <!-- end generic DataObject Transitions template-->

  <!-- saveMarlinTable process -->
  <xsl:template name="saveTable">
    <xsl:param name="processName"/>
    <xsl:param name="tableName"/>
    <xsl:param name="dataObjectSetKey"/>
    <xsl:param name="successProcess"/>
    <xsl:param name="userErrorProcess"/>
    <xsl:param name="pageFile"/>
    <xsl:param name="doCommit" select="'true'"/>
    <xsl:param name="groupName"/>

    <xsl:param name="actionName">
      <xsl:value-of select="utils:getActionName($processName, $groupName)"/>        
    </xsl:param>

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Action to save the <xsl:value-of select="@name"/> table </xsl:comment>

    <!-- resource class name argument value depends on whether
         the group is a child group or not -->
    <xsl:variable name="resourceClass">
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,../@dataObject)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,@dataObject)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>


    <action>
      <xsl:attribute name="path">
        <xsl:value-of select="$actionName"/>
      </xsl:attribute>
      <!-- name of save process -->
      <xsl:attribute name="type">
        <xsl:value-of select="'oracle.jheadstart.controller.struts.action.SaveDataObjectSet'"/>
      </xsl:attribute>

      <xsl:if test="/Service/@viewType = 'JSP'">            
        <xsl:attribute name="name">
          <xsl:value-of select="concat(@name, 'Bean')"/>
        </xsl:attribute>
        <xsl:attribute name="scope">
          <xsl:value-of select="'session'"/>
        </xsl:attribute>      
      </xsl:if>

      <!-- fully qualified dataObject interface class name -->
      <xsl:comment> Name of dataobject interface class </xsl:comment>
      <set-property property="dataObjectInterface">
        <xsl:attribute name="value">
          <xsl:value-of select="utils:getDataObjectInterface($modelBase,@dataObject)"/>
        </xsl:attribute>
      </set-property>

      <!-- if group s child group, insert of new rows is handled through
           handler of parent resource. This requires two additional params:
           * the processinfo name of the master
           * the resource name of the child  -->
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <!-- name of master dataObject in SessionData -->
          <xsl:comment> Name of master dataObject in SessionData </xsl:comment>
          <set-property property="masterDataObjectName">
            <xsl:attribute name="value">
              <xsl:value-of select="../@name"/>
            </xsl:attribute>
          </set-property>
          <!-- name of detail view -->
          <xsl:comment> Child dataObject name </xsl:comment>
          <set-property property="masterDataObjectInterface">
            <xsl:attribute name="value">
              <xsl:value-of select="$resourceClass"/>
            </xsl:attribute>
          </set-property>
          
          <xsl:comment> attribute in parent object that returns the collection of child objects</xsl:comment>
          <set-property property="childAttribute">
            <xsl:attribute name="value">
              <xsl:value-of select="@childAttribute"/>
            </xsl:attribute>
          </set-property>          
        </xsl:when>
      </xsl:choose>
      
      <!-- name of the table in UIX file -->
      <!--
      <xsl:comment> Name of the table in uix page </xsl:comment>
      <set-property property="dataObjectInterface">
        <xsl:attribute name="value">
          <xsl:value-of select="$tableName"/>
        </xsl:attribute>
      </set-property>
      -->
      
      <!-- SessionData select key of DataObject Set -->
      <xsl:comment> Name of table dataObject set in SessionData </xsl:comment>
      <set-property property="dataObjectSetName">
        <xsl:attribute name="value">
          <xsl:value-of select="$dataObjectSetKey"/>
        </xsl:attribute>
      </set-property>

     

      <xsl:if test="/Service/@viewType = 'UIX'">
      <!-- Page Name -->
        <xsl:comment> Page File Name. This name must exactly match the UIX
             file name, in order for the save table action to work correctly </xsl:comment>
        <set-property property="pageName">
          <xsl:attribute name="value">
            <xsl:value-of select="$pageFile"/>
          </xsl:attribute>
        </set-property>
      </xsl:if>

      
      <xsl:if test="/Service/@viewType = 'JSP' and @delete!='false'">            
        <xsl:comment> Request parameter name that is flag to delete a row </xsl:comment>
        <set-property property="deleteRowParameter">
          <xsl:attribute name="value">
            <xsl:value-of select="concat(@name,'DeleteRow')"/>
          </xsl:attribute>
        </set-property>
      </xsl:if>

      <xsl:if test="@queryType!=''">

        <xsl:comment> Name of the query type</xsl:comment>
        <set-property property="queryType">
          <xsl:attribute name="value">
              <xsl:value-of select="@queryType"/>
          </xsl:attribute>
        </set-property>
      </xsl:if>  

      <!-- commit transaction? -->
      <!--
      <xsl:comment> commit transaction? </xsl:comment>
      <argument type="Boolean">
        <xsl:attribute name="value">
          <xsl:value-of select="$doCommit"/>
        </xsl:attribute>
      </argument>
      -->

    <!-- add transitions for save table rocess -->
    <xsl:call-template name="persistentProcessTransitions">
      <xsl:with-param name="sourceProcess" select="$processName"/>
      <xsl:with-param name="successProcess" select="$successProcess"/>
      <xsl:with-param name="userErrorProcess" select="$userErrorProcess"/>
      <xsl:with-param name="groupName" select="$groupName"/>      
    </xsl:call-template>

    </action>

  </xsl:template> <!-- end saveMarlinTable template-->

</xsl:stylesheet>
