<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : StrutsServiceRouterLib.xsl

Notes       :

$revision_history$
24-nov-2003   Steven Davelaar
  1.10        Generate checkAccess=true property in ModuleRouter when
              hideUnauthorizedGroupTabs is true
29-aug-2003   Steven Davelaar
  1.9         Renamed ServiceRouter to ModuleRouter
21-aug-2003   Peter Ebell
  1.8         Replaced all occurences of static URL pattern "file://" by a call to
              utils:getURL, to allow for compatibility with Linux.
04-aug-2003   Steven Davelaar
  1.7         Set defaultPreviousForward to false (see bug 3079615)
              in ServiceRouter
28-mei-2003   AARDJ
  1.6         Generate a handle table event for a form if it has child groups
              with samePage = 'true'
22-mei-2003   AARDJ
  1.5         Added lov support
07-mei-2003   AARDJ
  1.4         Checked in after upload from D:\jhsdtdev\design\xslt\StrutsServiceRouterLib.xsl
18-apr-2003   AARDJ
  1.3         Checked in after upload from D:\jhsdtdev\design\xslt\StrutsServiceRouterLib.xsl
              Generate form-beans only if view type = 'JSP'
18-apr-2003   STEVEN DAVELAAR
  1.2         Added support for UIX table detail disclosure
24-feb-2003  Aard-Jan van Kesteren
  1.0         Initial creation (based on ServiceRouter.xsl)

*******************************************************************************
$add_to_revision_history
$
-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:cnt="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.Counter"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  exclude-result-prefixes="cnt utils"
  >
  <xsl:output method="xml" indent="yes" />

  <xsl:template name="moduleRouter">

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Check Page Time Stamp </xsl:comment>
    
    <action>
      <xsl:attribute name="path">    
        <xsl:text>/</xsl:text>
        <xsl:value-of select="concat('Start', Service/@name)"/>
      </xsl:attribute>
      <xsl:attribute name="type">
        <xsl:text>oracle.jheadstart.controller.struts.action.CheckPageTimeStamp</xsl:text>
      </xsl:attribute>

    <!-- Open the domain file definition file and check whether the are domains.
         If so, let the CheckPageTimeStamp action create the DomainsBean
    -->
    <xsl:variable name="domainFile" select="utils:getURL(/Service/@domainFilePath)"/>
    <xsl:variable name="domainDoc" select="document($domainFile)"/>

    <xsl:if test="count($domainDoc/DomainSet/Domain) != 0">
      <xsl:attribute name="name">DomainsBean</xsl:attribute>
    </xsl:if>
      
    <forward>
      <xsl:attribute name="name">
        <xsl:text>success</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="path">
        <xsl:text>ModuleRouter</xsl:text>
      </xsl:attribute>
    </forward>

    </action>    

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Module Router </xsl:comment>    
    
    <action>
      <xsl:attribute name="path">    
        <xsl:text>ModuleRouter</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="type">
        <xsl:text>oracle.jheadstart.controller.struts.action.DynamicActionRouter</xsl:text>
      </xsl:attribute>
    <set-property>
      <xsl:attribute name="property">
        <xsl:text>forwardParameterName</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="value">
        <xsl:text>targetGroup</xsl:text>
      </xsl:attribute>
    </set-property>
    <set-property>
      <xsl:attribute name="property">
        <xsl:text>attributeNameParameter</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="value">
        <xsl:text>targetCondition</xsl:text>
      </xsl:attribute>
    </set-property>
    <set-property>
      <xsl:attribute name="property">
        <xsl:text>attributeValueParameter</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="value">
        <xsl:text>targetConditionValue</xsl:text>
      </xsl:attribute>
    </set-property>
    <set-property>
      <xsl:attribute name="property">
        <xsl:text>defaultForward</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="value">
        <xsl:value-of select="Service/Group/@name"/>
      </xsl:attribute>
    </set-property>
    <set-property>
      <xsl:attribute name="property">
        <xsl:text>defaultPreviousForward</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="value">
        <xsl:text>false</xsl:text>
      </xsl:attribute>
    </set-property>
    <xsl:if test="Service/@hideUnauthorizedGroupTabs='true'">
      <set-property property="checkAccess" value="true"/>
      <!-- add comma-delimuted list of top level groups used to
           determine default forward if defaultFoward itself
           is not authorized -->
      <set-property property="forwardNames">
        <xsl:attribute name="value">
          <xsl:for-each select="Service/Group[position() != 1]">
            <xsl:if test="position()!=1">,</xsl:if>
            <xsl:value-of select="@name"/>
          </xsl:for-each>
        </xsl:attribute>
      </set-property>
    </xsl:if>
    <xsl:for-each select="//Group[@samePage!='true' or not(@samePage)]">
    <forward>
      <xsl:attribute name="name">
        <xsl:value-of select="@name"/>
      </xsl:attribute>
      <xsl:attribute name="path">
        <xsl:value-of select="@name"/>      
        <xsl:text>Router</xsl:text>
      </xsl:attribute>
    </forward>
    </xsl:for-each>

    <!--
      Below the fowards to the various lov routers.
      Complex lovs are currently only supported for jsp's
    -->
      <xsl:if test="Service/@viewType = 'JSP'">
        <xsl:for-each select="//Lookup[@displayType = 'lov']">
          <forward name="{concat('Lov', @name)}" path="{concat('Lov', @name, 'Router')}"/>
        </xsl:for-each>
      </xsl:if>
    </action>
  </xsl:template>


  <!-- process router -->
  <xsl:template name="simpleProcessRouter">
    <xsl:param name="router"/>
    <xsl:param name="pageName"/>
    <xsl:param name="groupName"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="dataObjectSetKey"/>
    <xsl:param name="initialProcess"/>

    <!-- set initial transition -->
    <xsl:call-template name="transitionInitial">
      <xsl:with-param name="initialProcess" select="$initialProcess"/>      
      <xsl:with-param name="router" select="$router"/>      
    </xsl:call-template>
    
    <!-- set transition for findPage if applicable -->
    <xsl:call-template name="transitionsFindPage">
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
    </xsl:call-template>
    
    <!-- set transitions based on layout style -->
    <xsl:choose>
      <xsl:when test="@layoutStyle='table'">
        <xsl:call-template name="transitionsTablePage">
          <xsl:with-param name="router" select="$router"/>
        </xsl:call-template>
        <!-- add transition to navigate directly to the table Page,
             but call it 'groupPage'. This way every type of group
             has a 'groupPage' transition, which is expected by
             the global headers
        -->
        <xsl:call-template name="routerTransition">
          <xsl:with-param name="condition" select="'groupPage'"/>
          <xsl:with-param name="destination" select="concat($pageName,'TablePage')"/>
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="keep" select="'false'"/>
        </xsl:call-template>

      </xsl:when>
      <xsl:when test="@layoutStyle='table-form'">
        <xsl:call-template name="transitionsTablePage">
          <xsl:with-param name="router" select="$router"/>
        </xsl:call-template>

        <!-- add transition to the GetSelectedDataObject process -->  
        <xsl:call-template name="routerTransition">
          <xsl:with-param name="condition" select="'details'"/>
          <xsl:with-param name="destination" select="concat('Get',$dataObjectKey)"/>
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="keep" select="'false'"/>
        </xsl:call-template>

        <xsl:call-template name="transitionsFormPage">
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
          <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
        </xsl:call-template>

        <!-- add transition to navigate directly to the table Page -->
        <xsl:call-template name="routerTransition">
          <xsl:with-param name="condition" select="'tablePage'"/>
          <xsl:with-param name="destination" select="concat($pageName,'TablePage')"/>
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="keep" select="'false'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@layoutStyle='select-form'">
        <xsl:call-template name="transitionsSelectFormPage">
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
          <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="@layoutStyle='form'">
        <xsl:call-template name="transitionsFormPage">
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
          <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>
    <!-- transition to navigate directly to find page --> 
    <xsl:if test="@findPage='true'">
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'findPage'"/>
        <xsl:with-param name="destination" select="concat('Find',$dataObjectKey,'Page')"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'false'"/>
      </xsl:call-template>
    </xsl:if>  

    <!-- transition to navigate to the SelectMasterRow process --> 
    <xsl:if test="../@layoutStyle='table'">
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'selectMasterRow'"/>
        <xsl:with-param name="destination" select="'SelectMasterRow'"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'false'"/>
      </xsl:call-template>
    </xsl:if>  

    <!-- All the transitions are added through methods in the XSLTUtils class.
         We can now send the router process to the output stream, and use the
         information in the XSLTUtils class to retrieve the keep- and remove
         strings and the actual transitions.
    -->

    <xsl:text>&#xa;</xsl:text>
    <xsl:text>&#xa;</xsl:text>
    <xsl:comment> Start <xsl:value-of select='@name'/>Router action </xsl:comment>
     <action>
       <xsl:attribute name="path">
         <xsl:value-of select='@name'/>
         <xsl:text>Router</xsl:text>
       </xsl:attribute>
       <xsl:attribute name="type">
         <xsl:text>oracle.jheadstart.controller.struts.action.ActionRouter</xsl:text>
       </xsl:attribute>       
       <set-property> 
         <xsl:attribute name="property">
           <xsl:text>defaultForward</xsl:text>
         </xsl:attribute>
         <xsl:attribute name="value">
           <xsl:text>initial</xsl:text>
         </xsl:attribute>         
       </set-property>
       <xsl:value-of disable-output-escaping="yes" select="utils:getTransitions()"/>
       <xsl:text>&#xa;      </xsl:text>            
     </action>    
    
   </xsl:template> <!-- end SIMPLE process router skeleton -->


  <!-- generic template for creating process router transition-->
  <xsl:template name="routerTransition">
    <xsl:param name="condition"/>
    <xsl:param name="destination"/>
    <xsl:param name="router"/>
    <xsl:param name="keep"/>

    <xsl:variable name="dummy">
      <xsl:value-of select="utils:addTransitionStruts($condition,$router,$destination,$keep)"/>
    </xsl:variable>
<!--
    <xsl:variable name="counter">
      <xsl:value-of select="cnt:getCounter(cnt:new())"/>
    </xsl:variable>

    <transition>
      <xsl:attribute name="src">
        <xsl:value-of select="$router"/>
      </xsl:attribute>
      <xsl:attribute name="condition">
        <xsl:value-of select="concat('tc',$counter)"/>
      </xsl:attribute>
      <xsl:attribute name="dest">
        <xsl:value-of select="$destination"/>
      </xsl:attribute>
    </transition>
-->
  </xsl:template>




  <!-- initial process router transitions -->
  <xsl:template name="transitionInitial">
    <xsl:param name="initialProcess"/>
    <xsl:param name="router"/>
    <!-- initialize java counter function used in routerTransition template
    <xsl:variable name="dummy">
      <xsl:value-of select="cnt:getNewCounter(cnt:new())"/>
    </xsl:variable>
    -->

    <xsl:variable name="dummy">
      <xsl:value-of select="utils:getNewTransitions()"/>
    </xsl:variable>

    <!-- initial transition -->
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'initial'"/>
      <xsl:with-param name="destination" select="$initialProcess"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'none'"/>
    </xsl:call-template>

  </xsl:template>



  <!-- process router transitions form page navigation -->
  <xsl:template name="transitionsFindPage">
    <xsl:param name="router"/>
    <xsl:param name="dataObjectSetKey"/>

    <xsl:if test="@findPage='true'">
      <!-- find data source set that matches criteria in find page -->
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'find'"/>
        <xsl:with-param name="destination" select="concat('Find',$dataObjectSetKey)"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'false'"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>




  <!-- process router transitions select form page navigation -->
  <xsl:template name="transitionsSelectFormPage">
    <xsl:param name="router"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="dataObjectSetKey"/>

    <!-- get dataObject from set -->
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'select'"/>
      <xsl:with-param name="destination" select="concat('Get',$dataObjectKey)"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'false'"/>
    </xsl:call-template>
    <xsl:call-template name="transitionsFormPage">
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
      <xsl:with-param name="dataObjectSetKey" select="$dataObjectSetKey"/>
    </xsl:call-template>
    <!-- transition to navigate directly to select page --> 
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'selectPage'"/>
      <xsl:with-param name="destination" select="concat('Select',$dataObjectKey,'Page')"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'false'"/>
    </xsl:call-template>
  </xsl:template> <!-- end template for select-form page transitions -->


  <!-- process router transitions select form page navigation -->
  <xsl:template name="transitionsFormPage">
    <xsl:param name="router"/>
    <xsl:param name="dataObjectKey"/>
    <xsl:param name="dataObjectSetKey"/>

    <!-- get dataObject from set -->
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'browse'"/>
      <xsl:with-param name="destination" select="concat('Browse',$dataObjectSetKey)"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'true'"/>
    </xsl:call-template>

    <xsl:if test="not(@insert='false')">
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'insert'"/>
        <xsl:with-param name="destination" select="concat('GetDefault',$dataObjectKey)"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'false'"/>
      </xsl:call-template>
    </xsl:if>

    <xsl:if test="Group[@samePage='true' and (@sortable='true' or @useTableRange='true' or @detailDisclosure='true')]">
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'event'"/>
        <xsl:with-param name="destination" select="concat('Handle',@name,'TableEvent')"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'true'"/>     
      </xsl:call-template>
    </xsl:if>
    
    <xsl:call-template name="transitionsSaveDataObject">
       <xsl:with-param name="router" select="$router"/>
       <xsl:with-param name="dataObjectKey" select="$dataObjectKey"/>
    </xsl:call-template>
    <!-- transition to navigate directly to group page --> 
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'groupPage'"/>
      <xsl:with-param name="destination" select="concat($dataObjectKey,'Page')"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'false'"/>
    </xsl:call-template>
  </xsl:template> <!-- end template for form page transitions -->

  <!-- process router transitions form page navigation -->
  <xsl:template name="transitionsSaveDataObject">
    <xsl:param name="router"/>
    <xsl:param name="dataObjectKey"/>

    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'save'"/>
      <xsl:with-param name="destination" select="concat('Save',$dataObjectKey)"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'true'"/>
    </xsl:call-template>

      <xsl:if test="not(@delete='false')">
        <xsl:call-template name="routerTransition">
          <xsl:with-param name="condition" select="'delete'"/>
          <xsl:with-param name="destination" select="concat('Delete',$dataObjectKey)"/>
          <xsl:with-param name="router" select="$router"/>
          <xsl:with-param name="keep" select="'false'"/>
        </xsl:call-template>
      </xsl:if>     
  </xsl:template> <!-- end template for save dataObject transitions -->


  <!-- process router transition for saving data source set -->
  <xsl:template name="transitionsSaveDataObjectSet">
    <xsl:param name="router"/>
    
    <!-- save data source set-->
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="condition" select="'saveTable'"/>
      <xsl:with-param name="destination" select="concat('Save',@name,'Table')"/>
      <xsl:with-param name="router" select="$router"/>
      <xsl:with-param name="keep" select="'false'"/>
    </xsl:call-template>
  </xsl:template>

  <!-- process router transition for saving data source set -->
  <xsl:template name="transitionsTablePage">
    <xsl:param name="router"/>

    <xsl:call-template name="transitionsSaveDataObjectSet">
      <xsl:with-param name="router" select="$router"/>
    </xsl:call-template>

    <xsl:if test="@sortable='true' or @useTableRange='true' or @detailDisclosure='true'">
       <!-- handle table event -->
      <xsl:call-template name="routerTransition">
        <xsl:with-param name="condition" select="'event'"/>
        <xsl:with-param name="destination" select="concat('Handle',@name,'TableEvent')"/>
        <xsl:with-param name="router" select="$router"/>
        <xsl:with-param name="keep" select="'true'"/>
      </xsl:call-template>
    </xsl:if>  

     <!-- tc4: select master row
    <xsl:call-template name="routerTransition">
      <xsl:with-param name="destination" select="'SelectMasterRow'"/>
      <xsl:with-param name="router" select="$router"/>
    </xsl:call-template>
 -->    
  </xsl:template>

</xsl:stylesheet>
