<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : UIXFormLib.xsl

Notes       :

Open Issues :


$revision_history$
06-aug-2003   Steven Davelaar
  2.8         
15-mei-2003   STEVEN DAVELAAR
  2.7         Improved nls support
17-mar-2003   Steven Davelaar
  2.6         
09-jan-2003   Peter Ebell
  2.5         Fixed problem with previous version where hidden form items would appear once
              in EVERY region.
23-oct-2002   Sigrid Gylseth
  2.4         Bugfix 2407595. Ensured that all hidden items are included below the 
              </labeledFieldLayout> tag.
11-jun-2002   Peter Ebell
  2.3         Put the display title on the delete button on samePage layout
11-jun-2002   Peter Ebell
  2.2         Added support for Regions on form page layout.
15-may-2002   Peter Ebell
  2.1         Always use template getDisplayTitle for buttons etc.
14-may-2002   Peter Ebell
  2.0         Pre-prod release
28-mrt-2002   Peter Ebell
  1.1         Rewritten for JHeadstart 9i. Removed obsolete revision history.
25-sep-2001   Steven Davelaar
  1.0         initial creation (copied most part from obsolete
              VOToUIXForm.xsl)

*******************************************************************************
$add_to_revision_history
$
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  exclude-result-prefixes="utils">
  <xsl:import href="UIXLibrary.xsl"/>

  <xsl:template name="formLayout">
    <xsl:param name="columns"/>
    <xsl:param name="voDoc"/>
    <xsl:param name="layoutStyle"/>
    
    <xsl:variable name="width">
      <xsl:choose>
        <xsl:when test="@formWidth!=''">
          <xsl:value-of select="@formWidth"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'10%'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <spacer height="14"/>

    <xsl:call-template name="formItems">
      <xsl:with-param name="voDoc" select="$voDoc"/>
      <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
      <xsl:with-param name="columns" select="$columns"/>
      <xsl:with-param name="width" select="$width"/>
    </xsl:call-template>
<!--    <xsl:call-template name="hiddenFieldsFormLayout"/> -->
  </xsl:template>

  <xsl:template name="findFormLayout">
    <xsl:param name="columns"/>
    <xsl:param name="voDoc"/>
    <xsl:param name="layoutStyle"/>

    <xsl:variable name="width">
      <xsl:choose>
        <xsl:when test="@formWidth!=''">
          <xsl:value-of select="@formWidth"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="'10%'"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <script text="ignoreChanges=true;"/>

    <spacer height="14"/>
    <labeledFieldLayout>
      <xsl:attribute name="columns">
        <xsl:value-of select="$columns"/>
      </xsl:attribute>
      <xsl:attribute name="width">
        <xsl:value-of select="$width"/>
      </xsl:attribute>
      <contents>
         <xsl:call-template name="findFormItems">
           <xsl:with-param name="voDoc" select="$voDoc"/>
           <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
           <xsl:with-param name="hiddenDisplay" select="'false'"/>
         </xsl:call-template>
      </contents>
    </labeledFieldLayout>   
    <xsl:call-template name="findFormItems">
       <xsl:with-param name="voDoc" select="$voDoc"/>
       <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
       <xsl:with-param name="hiddenDisplay" select="'true'"/>
    </xsl:call-template>
    <!-- hidden field for find button 
    <formValue>
      <xsl:attribute name="name">
        <xsl:value-of select="concat('find',$pageName)"/>
      </xsl:attribute>
    </formValue>
-->
  </xsl:template>


  <xsl:template name="formItems">
    <xsl:param name="voDoc"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="columns"/>
    <xsl:param name="width"/>

    <!-- we need to store the group node in a variable because we will
         loose the "Group" context from here on
    -->
    <xsl:variable name="group" select="current()"/>
    <xsl:variable name="dataObjectName" select="@name"/>

    <xsl:choose>
      <xsl:when test="Region">
<!--    <xsl:message>We have regions in group <xsl:value-of select="@name"/></xsl:message> -->
        <!-- First display the items that do not belong to a region -->
        <labeledFieldLayout columns="{$columns}" width="{$width}">
          <contents>
            <xsl:for-each select="$voDoc/ViewObject/ViewAttribute">
              <xsl:variable name="region" select="Properties/Property[@Name='REGION']/@Value"/>
    <!--      <xsl:message>Item: <xsl:value-of select="@name"/> has region: <xsl:value-of select="$region"/></xsl:message>            -->
              <xsl:if test="not($region)">
    <!--        <xsl:message>No Region!!</xsl:message> -->
                <xsl:call-template name="handleItem">
                  <xsl:with-param name="voDoc" select="$voDoc"/>
                  <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
                  <xsl:with-param name="group" select="$group"/>
                  <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
                  <xsl:with-param name="hiddenDisplay" select="'false'"/>
                </xsl:call-template>  
              </xsl:if>  
            </xsl:for-each>
          </contents>
        </labeledFieldLayout>  

        <!-- Loop over all the regions, and display only the fields in these regions -->
        <xsl:for-each select="Region">
          <xsl:variable name="regionTitle" select="@title"/>
          <xsl:variable name="regionName" select="@name"/>
          <xsl:variable name="regionWidth">
            <xsl:choose>
              <xsl:when test="@width">
                <xsl:value-of select="@width"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="'10%'"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>

          <spacer height="14"/>
          <header text="{$regionTitle}">
            <contents>
              <spacer height="14"/>
              <labeledFieldLayout columns="{@columns}">
                <xsl:if test="$regionWidth">
                  <xsl:attribute name="width">
                    <xsl:value-of select="$regionWidth"/> 
                  </xsl:attribute>
                </xsl:if>  
                <contents>
<!--              <xsl:message>Processing region: <xsl:value-of select="@title"/></xsl:message> -->
                  <xsl:for-each select="$voDoc/ViewObject/ViewAttribute[Properties/Property[@Name='REGION' and @Value=$regionName]]">
<!--                <xsl:message>Item: <xsl:value-of select="@Name"/> </xsl:message> -->
                    <xsl:call-template name="handleItem">
                      <xsl:with-param name="voDoc" select="$voDoc"/>
                      <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
                      <xsl:with-param name="group" select="$group"/>
                      <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
                      <xsl:with-param name="hiddenDisplay" select="'false'"/>
                    </xsl:call-template>  
                  </xsl:for-each>
                </contents>
              </labeledFieldLayout> 
            </contents>  
          </header>  
        </xsl:for-each>
        <!-- Add all hidden fields after all the other items and regions   -->
        <xsl:for-each select="$voDoc/ViewObject/ViewAttribute">
           <xsl:call-template name="handleItem">
             <xsl:with-param name="voDoc" select="$voDoc"/>
             <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
             <xsl:with-param name="group" select="$group"/>
             <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
             <xsl:with-param name="hiddenDisplay" select="'true'"/>
           </xsl:call-template>  
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <labeledFieldLayout columns="{$columns}" width="{$width}">
          <contents>
            <xsl:for-each select="$voDoc/ViewObject/ViewAttribute">
               <xsl:call-template name="handleItem">
                 <xsl:with-param name="voDoc" select="$voDoc"/>
                 <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
                 <xsl:with-param name="group" select="$group"/>
                 <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
                 <xsl:with-param name="hiddenDisplay" select="'false'"/>
               </xsl:call-template>  
            </xsl:for-each>
          </contents>
        </labeledFieldLayout>  
            <xsl:for-each select="$voDoc/ViewObject/ViewAttribute">
               <xsl:call-template name="handleItem">
                 <xsl:with-param name="voDoc" select="$voDoc"/>
                 <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
                 <xsl:with-param name="group" select="$group"/>
                 <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
                 <xsl:with-param name="hiddenDisplay" select="'true'"/>
               </xsl:call-template>  
            </xsl:for-each>
      </xsl:otherwise>
    </xsl:choose>  
  </xsl:template>



  <!-- Template to hande a single form item. -->
  <xsl:template name="handleItem">
    <xsl:param name="voDoc"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="group"/>
    <xsl:param name="dataObjectName"/>
    <xsl:param name="hiddenDisplay"/>
   
      <xsl:variable name="eoFile">
        <xsl:call-template name="getEoFileName">
          <xsl:with-param name="voDoc" select="$voDoc"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="entityUsage" select="@EntityUsage"/>
      <xsl:variable name="eoAttributeName" select="@EntityAttrName"/>
      <xsl:variable name="eoDoc" select="document($eoFile)"/>
      <xsl:variable name="entityAttributeNode"  select="$eoDoc/Entity/Attribute[$entityUsage!='' and @Name=$eoAttributeName]|current()[$entityUsage='']"/>

      <xsl:variable name="display">
        <xsl:call-template name="getCustomProperty">
          <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
          <xsl:with-param name="propertyName" select="'DISPLAY'"/>
          <xsl:with-param name="defaultValue" select="'true'"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:choose>
      <xsl:when test="($hiddenDisplay='true' and $display='hidden') or ($hiddenDisplay='false' and $display='true') ">  
<!--        <xsl:when test="$display='true' or $display='hidden'"> -->
          <xsl:call-template name="createFormElement">
            <xsl:with-param name="attributeName" select="@Name"/>
            <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
            <xsl:with-param name="operator" select="''"/>
            <xsl:with-param name="isFindElement">false</xsl:with-param>
            <xsl:with-param name="display" select="$display"/>
            <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
            <xsl:with-param name="group" select="$group"/>
            <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
          </xsl:call-template>
        </xsl:when>
      </xsl:choose>
  </xsl:template>    

  <xsl:template name="findFormItems">
    <xsl:param name="voDoc"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="hiddenDisplay"/>  
    <!-- we need to store the group node in a variable because we will
         loose the "Group" context from here on
    -->
    <xsl:variable name="group" select="current()"/>
    <xsl:variable name="dataObjectName" select="@dataObject"/>
    
    <xsl:for-each select="$voDoc/ViewObject/ViewAttribute[not(@IsQueriable='false')]">

      <xsl:variable name="eoFile">
        <xsl:call-template name="getEoFileName">
          <xsl:with-param name="voDoc" select="$voDoc"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="entityUsage" select="@EntityUsage"/>
      <xsl:variable name="eoAttributeName" select="@EntityAttrName"/>
      <xsl:variable name="eoDoc" select="document($eoFile)"/>
      <xsl:variable name="entityAttributeNode"  select="$eoDoc/Entity/Attribute[$entityUsage!='' and @Name=$eoAttributeName]|current()[$entityUsage='']"/>
      
      <xsl:variable name="isQueriable" select="$entityAttributeNode/@IsQueriable"/>

      <!-- check whether IsQueriable flag is set to false at EO level -->
      <xsl:if test="not ($isQueriable='false')">
        <xsl:variable name="display">
          <xsl:call-template name="getCustomProperty">
            <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
            <xsl:with-param name="propertyName" select="'DISPLAY'"/>
            <xsl:with-param name="defaultValue" select="'true'"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name="rangeQuery">
          <xsl:call-template name="getCustomProperty">
            <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
            <xsl:with-param name="propertyName" select="'RANGE_QUERY'"/>
            <xsl:with-param name="defaultValue" select="'false'"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name="queryOperator">
          <xsl:call-template name="getCustomProperty">
            <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
            <xsl:with-param name="propertyName" select="'QUERY_OPERATOR'"/>
            <xsl:with-param name="defaultValue" select="'like'"/>
          </xsl:call-template>
        </xsl:variable>

      <xsl:if test="($hiddenDisplay='true' and $display='hidden') or ($hiddenDisplay='false' and $display='true') ">  
<!--       <xsl:if test="$display='true' or $display='hidden'">  -->
          <!-- if rangeQuery is true, we create two find fields
               a "from" and a "to" field -->
          <xsl:choose>
            <xsl:when test="$rangeQuery='true'">
              <xsl:call-template name="createRangeAttributes">
                <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
                <xsl:with-param name="display" select="$display"/>
                <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
                <xsl:with-param name="group" select="$group"/>
                <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
              </xsl:call-template>    
            </xsl:when>
            <xsl:otherwise>
              <xsl:call-template name="createFormElement">
                <xsl:with-param name="attributeName" select="concat($findItemPrefix,@Name)"/>
                <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
                <xsl:with-param name="operator" select="$queryOperator"/>
                <xsl:with-param name="isFindElement">true</xsl:with-param>
                <xsl:with-param name="display" select="$display"/>
                <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
                <xsl:with-param name="group" select="$group"/>
                <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="createRangeAttributes">
    <xsl:param name="entityAttribute"/>
    <xsl:param name="display"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="group"/>
    <xsl:param name="dataObjectName"/>

    <!-- set variable that holds prompt name -->
    <xsl:variable name="prompt">
      <xsl:call-template name="getCustomProperty">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="propertyName" select="'PROMPT'"/>
        <xsl:with-param name="defaultValue" select="@Name"/>
      </xsl:call-template>
    </xsl:variable>
    <!-- create from attribute -->
    <!-- we first create a variable promptFrom and then pass it as
         template parameter. If we directly use the utils:getNlsKey
         construct within xsl:param we get error XSL-1008: Namespace prefix
         utils used but not declared -->
    <xsl:variable name="promptFrom"
       select="utils:getNlsKey('SEARCH_FROM',$prompt,concat('SEARCH_FROM_',@Name))"/>

    <xsl:call-template name="createFormElement">
      <xsl:with-param name="attributeName" select="concat($findItemPrefix,@Name,'From')"/>
      <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      <xsl:with-param name="elementPrompt" select="$promptFrom"/>
      <xsl:with-param name="operator" select="'gt'"/>
      <xsl:with-param name="isFindElement">true</xsl:with-param>
      <xsl:with-param name="display" select="$display"/>
      <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
      <xsl:with-param name="group" select="$group"/>
      <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
    </xsl:call-template>
    <!-- create to attribute -->
    <!-- we first create a variable promptTo and then pass it as
         template parameter. If we directly use the utils:getNlsKey
         construct within xsl:param we get error XSL-1008: Namespace prefix
         utils used but not declared -->
    <xsl:variable name="promptTo"
       select="utils:getNlsKey('SEARCH_TO',$prompt,concat('SEARCH_TO_',@Name))"/>

    <xsl:call-template name="createFormElement">
      <xsl:with-param name="attributeName" select="concat($findItemPrefix,@Name,'To')"/>
      <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      <xsl:with-param name="elementPrompt" select="$promptTo"/>
      <xsl:with-param name="operator" select="'lt'"/>
      <xsl:with-param name="isFindElement">true</xsl:with-param>
      <xsl:with-param name="display" select="$display"/>
      <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
      <xsl:with-param name="group" select="$group"/>
      <xsl:with-param name="dataObjectName" select="$dataObjectName"/>
    </xsl:call-template>

  </xsl:template>

  <!-- Add hidden form fields for new, save, ok and delete button 
  <xsl:template name="hiddenFieldsFormLayout">
    <!- hidden field for new button ->
    <formValue>
      <xsl:attribute name="name">
        <xsl:value-of select="concat('new',$pageName,'Page')"/>
      </xsl:attribute>
    </formValue>
    <!- hidden field for save button ->
    <formValue>
      <boundAttribute name="name">
        <if>
          <dataObject source="jheadstart:staticData" select="isInserting"/>
          <dataObject source="pageModeData" select="insertButton"/>
          <dataObject source="pageModeData" select="updateButton"/>
        </if>
      </boundAttribute>
    </formValue>

    <formValue>
      <xsl:attribute name="name">
        <xsl:value-of select="$okbutton"/>
      </xsl:attribute>
    </formValue>
    <!- only add hidden field for delete button
         if deleteAllowed ->
    <xsl:choose>
      <xsl:when test="@delete='true'">
        <formValue>
          <xsl:attribute name="name">
            <xsl:value-of select="$deletebutton"/>
          </xsl:attribute>
        </formValue>
      </xsl:when>
    </xsl:choose>

  </xsl:template>
-->

  <!-- add save, delete, OK buttons
  -->
  <xsl:template name="formButtons">
    <!-- determine the display title -->
    <xsl:variable name="displayTitle">
      <xsl:call-template name="getDisplayTitle"/>
    </xsl:variable>

    <xsl:attribute name="formName">
      <xsl:value-of select="$formName"/>
    </xsl:attribute>

    <xsl:call-template name="nlsTextAttribute">
      <xsl:with-param name="attributeName" select="'newButtonLabel'"/>
      <xsl:with-param name="key">
        <xsl:value-of select="utils:getNlsKey('NEW_BUTTON_LABEL',$displayTitle,concat('NEW_BUTTON_LABEL_',@name))"/>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="nlsTextAttribute">
      <xsl:with-param name="attributeName" select="'saveButtonLabel'"/>
      <xsl:with-param name="key">
        <xsl:value-of select="utils:getNlsKey('SAVE_BUTTON_LABEL')"/>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:variable name="delDisplayTitle">
      <xsl:choose>
        <xsl:when test="Group[@samePage='true']">
          <xsl:value-of select="$displayTitle"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="''"/>
        </xsl:otherwise>  
      </xsl:choose>
    </xsl:variable>  

    <xsl:call-template name="nlsTextAttribute">
      <xsl:with-param name="attributeName" select="'deleteButtonLabel'"/>
      <xsl:with-param name="key">
        <xsl:value-of select="utils:getNlsKey('DELETE_BUTTON_LABEL',$delDisplayTitle,concat('DELETE_BUTTON_LABEL_',@name))"/>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="nlsTextAttribute">
      <xsl:with-param name="attributeName" select="'okButtonLabel'"/>
      <xsl:with-param name="key">
        <xsl:value-of select="utils:getNlsKey('OK_BUTTON_LABEL')"/>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:attribute name="okButtonName">
      <xsl:choose>
        <xsl:when test="@layoutStyle='table-form'">
          <xsl:value-of select="'tablePage'"/>
        </xsl:when>  
        <xsl:when test="@layoutStyle='select-form'">
          <xsl:value-of select="'selectPage'"/>
        </xsl:when>  
        <xsl:otherwise>
          <xsl:value-of select="'groupPage'"/>
        </xsl:otherwise>  
      </xsl:choose>
    </xsl:attribute>
  </xsl:template>

  <!-- add save, delete, OK buttons
  -->
  <xsl:template name="findFormButtons">
    <!-- determine the display title -->
    <xsl:variable name="displayTitle">
      <xsl:call-template name="getDisplayTitle"/>
    </xsl:variable>

   <xsl:attribute name="formName">
     <xsl:value-of select="$formName"/>
   </xsl:attribute>

   
    <xsl:call-template name="nlsTextAttribute">
      <xsl:with-param name="attributeName" select="'findButtonLabel'"/>
      <xsl:with-param name="key">
        <xsl:value-of select="utils:getNlsKey('FIND_BUTTON_LABEL',$displayTitle,concat('FIND_BUTTON_LABEL_',@name))"/>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="nlsTextAttribute">
      <xsl:with-param name="attributeName" select="'newButtonLabel'"/>
      <xsl:with-param name="key">
        <xsl:value-of select="utils:getNlsKey('NEW_BUTTON_LABEL',$displayTitle,concat('NEW_BUTTON_LABEL_',@name))"/>
      </xsl:with-param>
    </xsl:call-template>

   <xsl:attribute name="showInsertButton">
     <xsl:choose>
       <xsl:when test="not(@layoutStyle='table')">
         <xsl:value-of select="'true'"/>
       </xsl:when>  
       <xsl:otherwise>
         <xsl:value-of select="'false'"/>
       </xsl:otherwise>  
     </xsl:choose>
   </xsl:attribute>

  </xsl:template>

</xsl:stylesheet>
