<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : UIXGlobalHeaders.xsl

Notes       :

Open Issues :

$revision_history$
24-okt-2003   Steven Davelaar
  2.10        Exclude detail groups in same page
25-aug-2003   Peter Ebell
  2.9         - Changed syntax for binding with groupAccessDataObject.
              - No longer use groupIncludeFile property to determine if a group is 'secure'.
04-aug-2003   Steven Davelaar
  2.8         Changed nls key to use group name 
28-jul-2003   Steven Davelaar
  2.7         Use tabName property first as label
15-mei-2003   STEVEN DAVELAAR
  2.6         Added addToNlsBundle parameter in call to nlsTextAttribute template
01-mei-2003   STEVEN DAVELAAR
  2.5         Added nls support
17-mar-2003   Steven Davelaar
  2.4         
01-jul-2002   Peter Ebell
  2.3         Changed references to formValues 'targetInfoAlias(Value)' to 'targetCondition(Value)'
              because infoAliases do not exist in the SimpleProcessRouter.
19-jun-2002   Peter Ebell
  2.2         Do not use 'initialState', as this is not a 'real' transition.
10-jun-2002   Peter Ebell
  2.1         - Dynamically bind the disabled attribute of the global header links.
              - Made rendered attribute dependent on roleBasedSecurity and groupIncludeFile properties
14-may-2002   Peter Ebell
  2.0         Global header is now a template instead of an included UIX document
28-mrt-2002   Peter Ebell
  1.1         Rewritten for JHeadstart 9i. Removed obsolete revision history.
31-may-2001   Steven Davelaar
  1.0         Initial creation

*******************************************************************************
$add_to_revision_history
$
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
  xmlns="http://xmlns.oracle.com/uix/ui"
  xmlns:data="http://xmlns.oracle.com/uix/ui"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  exclude-result-prefixes="utils"
  xmlns:jheadstart="http://www.oracle.com/jheadstart">
  <xsl:import href="UIXLibrary.xsl"/>
  <!-- import UIXPage for boundAttribute template called indirectly
       by nlsTextAttribute -->
  <xsl:import href="UIXPage.xsl"/>
  <xsl:output method="xml" indent="yes" />
  <xsl:param name="groupName"></xsl:param>

  <xsl:variable name="hideUnauthorizedGroupTabs" select="Service/@hideUnauthorizedGroupTabs"/>

  <xsl:variable name="groupIncludeFile" select="Service/@groupIncludeFile"/>

  <xsl:variable name="globalHeaderName" select="concat($groupName,'GlobalHeader')"/>

  <xsl:template match="/">

    <templateDefinition xmlns="http://xmlns.oracle.com/uix/ui"
            xmlns:ui="http://xmlns.oracle.com/uix/ui"
            xmlns:data="http://xmlns.oracle.com/uix/ui"
            xmlns:templates="http://www.oracle.com/jheadstart/templates"
            targetNamespace="http://www.oracle.com/jheadstart/templates"
            localName="{$globalHeaderName}">

      <!-- define the template's type information -->
      <type base="ui:pageHeader">
        <attribute name="selectedIndex" javaType="int"/>
      </type>
      <content>
        <globalHeader data:selectedIndex="selectedIndex@ui:rootAttr">
          <contents>
            <xsl:call-template name="masterGroupLink"/>
          </contents>
        </globalHeader>
      </content>  
    </templateDefinition>
  </xsl:template>

  <xsl:template name="masterGroupLink">
   <xsl:for-each select="Service/Group[@name=$groupName]">

    <xsl:variable name="masterTargetProcess" select="'groupPage'"/>
<!--<xsl:text>edit</xsl:text>
      <xsl:value-of select="@name"/>
      <xsl:text>Page</xsl:text>
    </xsl:variable>
-->    
    <xsl:call-template name="groupLink">
      <xsl:with-param name="targetProcess" select="$masterTargetProcess"/>
    </xsl:call-template>
    
    <xsl:call-template name="detailGroupLinks"/>

   </xsl:for-each>
  </xsl:template>

  <xsl:template name="detailGroupLinks">
   <xsl:for-each select="Group[@samePage='false']">
    <xsl:call-template name="groupLink">
       <xsl:with-param name="targetProcess" select="''"/>
    </xsl:call-template>
   </xsl:for-each>
  </xsl:template>

  <xsl:template name="groupLink">
   <xsl:param name="targetProcess"/>

<!-- PEBELL No longer check for groupIncludeFile because with Struts it is not used
   <!- determine whether the group is secure ->
   <xsl:variable name="secureGroup">
     <xsl:choose>
       <xsl:when test="not($hideUnauthorizedGroupTabs='true')">
         <xsl:value-of select="'false'"/>
       </xsl:when>
       <xsl:when test="$groupIncludeFile">
         <xsl:value-of select="'true'"/>
       </xsl:when>
       <xsl:when test="@groupIncludeFile">
         <xsl:value-of select="'true'"/>
       </xsl:when>
       <xsl:otherwise>
         <xsl:value-of select="'false'"/>
       </xsl:otherwise>
     </xsl:choose> 
   </xsl:variable>
-->

   <xsl:variable name="displayLabel">
      <xsl:choose>
          <xsl:when test="@tabName!=''">
            <xsl:value-of select="@tabName"/>
          </xsl:when>
        <xsl:when test="@displayTitle!=''">
          <xsl:value-of select="@displayTitle"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@dataObject"/>
        </xsl:otherwise>
      </xsl:choose>
   </xsl:variable>

   <!-- add dummy text here to get output-escaping really disabled -->
   <xsl:text disable-output-escaping="yes">
   </xsl:text>
    <link>

      <xsl:call-template name="nlsTextAttribute">
        <xsl:with-param name="attributeName" select="'text'"/>
        <xsl:with-param name="key">
          <xsl:value-of select="concat(@name,'_LEVEL_2_TAB')"/>
        </xsl:with-param>
        <xsl:with-param name="value">
          <xsl:value-of select="$displayLabel"/>
        </xsl:with-param>
        <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
      </xsl:call-template>
   
      <xsl:attribute name="destination">
        <xsl:text>javascript:go2(['targetGroup','targetCondition'],['</xsl:text>
        <xsl:value-of select="@name"/>
        <xsl:text>','</xsl:text>
        <xsl:value-of select="$targetProcess"/>
        <xsl:text>']);</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="data:disabled">
        <xsl:value-of select="'isInserting@dataObject@ui:rootAttr'"/>
      </xsl:attribute>
      <xsl:if test="$hideUnauthorizedGroupTabs='true'">
        <xsl:attribute name="data:rendered">
          <xsl:value-of select="concat(@name,'@GroupAccessDataObject@jheadstart:userDataObject')"/>
        </xsl:attribute>
      </xsl:if>
    </link>
  </xsl:template>

  <xsl:template name="dataBoundAttribute">
    <xsl:param name="name"/>
    <xsl:param name="value"/>
    <xsl:attribute name="{$name}">
      <xsl:value-of select="$value"/>
    </xsl:attribute>
  </xsl:template>

</xsl:stylesheet>
