<!--
*******************************************************************************
Copyright (c) 2001 Oracle Corporation

File        : UIXLibrary.xsl

Notes       :

Open Issues :

$revision_history$
24-nov-2003   Steven Davelaar
  2.30        Added rendered boundAttribute to image and fileDownload
20-nov-2003   Sandra Muller
  2.29        added requestContextPath to lovFrame.jsp, notes.jsp, editor.gif
              (for use in Struts modules)
14-nov-2003   Steven Davelaar
  2.28        Added templates checkMultipartAttributes and checkGroupMultipartAttributes
              Added logic to set display type to fileUpload, fileDownload or image
07-sep-2003   Steven Davelaar
  2.27        Fixed bug in setting isPersistent, now set to "true" instead 
              "while_insert"!
29-aug-2003   Steven Davelaar
  2.26        Renamed infoTable to sessionData
21-aug-2003   Peter Ebell
  2.25        Replaced all occurences of static URL pattern "file://" by a call to
              utils:getURL, to allow for compatibility with Linux.
11-aug-2003   Steven Davelaar
  2.24        - Removed code that checks for PERSISTS custom property.
              Instead, improved algorithm to determine updatability. 
              We now check whether attribute is lookup attribute of
              readOnly EO, which means it is non updatable as well.
              This fixes bug 3080053 UIX WITH LOV ON LOOKUP FIELD: WHEN SAVING 
              RECORD, GET JBO-27008 ERROR as well.
              - renamed custom properties persistent to isPersistent,
              operator to queryOperator, and caseSensitive to isCaseSensitive
07-aug-2003   Aard-Jan van Kesteren
  2.23        
06-aug-2003   Steven Davelaar
  2.22        
06-aug-2003   Steven Davelaar
  2.21        fix bug 3083520 in nlsTextAttrobute: always nls enable
              search_from/ search_to prompts
04-aug-2003   Steven Davelaar
  2.20        Fixed bug in nlsTextAttribute introduced in previous fix
04-aug-2003   Steven Davelaar
  2.19        Changed nlsTextAttribute, always generate NLS-enabled in case
              of heading, button label, etc.
29-jul-2003   Steven Davelaar
  2.18        
29-jul-2003   Steven Davelaar
  2.17        Added support for datetimeField display type
29-jul-2003   Steven Davelaar
  2.16        Databind date pattern
              Improved algorithm for display type. Dates and choices are no longer converted
              to styledtext when readOnly. If there is a LookUp defined in app struc, we always 
              use the lookup style (choice / lov) defined with the Lookup as display type
30-jun-2003   Steven Davelaar
  2.15        Added support for queryType in LOV call
23-may-2003   Steven Davelaar
  2.14        Only display tip attribute when Service-level property
              showHintText is set to true
              D0 not set readOnly attribute on find page 
15-mei-2003   Steven Davelaar
  2.13        changed @nls to @jheadstart:nls
03-mei-2003   Steven Davelaar
  2.12        Improved nls support
01-mei-2003   Steven Davelaar
  2.11        Added nls support
17-mar-2003   Steven Davelaar
  2.9         
09-jan-2003   Peter Ebell
  2.8         Set dataObject property on EVERY form element.
25-oct-2002   Sigrid Gylseth
  2.7         Bugfix: 2417892. Changed template readOnlyBinding to take into accout updatable
              property on VO.
02-aug-2002   Peter Ebell
  2.6         Comment out all code where "requestContextPath" was added
              to urls. Now use relative url to jheadstart objects.
21-jul-2002   Peter Ebell
  2.5         - Added maxlength parameter to LOV call for editor field
              - Non-enterable LOV field now displays message on onFocus event instead of onChange
11-jun-2002   Peter Ebell
  2.4         Disable the LOV field if the underlying item is not persistent/updateable
10-jun-2002   Peter Ebell
  2.3         - Added support for custom properties HINT_TEXT and CASE_INSENSITIVE
              - Took out workaround with respect to required date fields in table
15-may-2002   Peter Ebell
  2.2         Added hint for better code to getDisplayTitle
14-may-2002   Peter Ebell
  2.1         Pre-prod release
28-mrt-2002   Peter Ebell
  2.0         Rewritten for JHeadstart 9i. Removed obsolete revision history.
18-apr-2001   Steven Davelaar
  1.0         Initial creation
*******************************************************************************
$add_to_revision_history
$
-->
<xsl:stylesheet
  xmlns="http://xmlns.oracle.com/uix/ui"
  xmlns:utils="http://www.oracle.com/XSL/Transform/java/oracle.jheadstart.tools.util.XSLTUtils"
  xmlns:string="http://www.oracle.com/XSL/Transform/java/java.lang.String"
  exclude-result-prefixes="utils, string"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:template name="createFormElement">
    <xsl:param name="attributeName" select="@Name"/>   
    <xsl:param name="entityAttribute"/>
    <xsl:param name="isFindElement">false</xsl:param>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="operator"/>
    <xsl:param name="display"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="group"/>
    <xsl:param name="dataObjectName"/>

    <xsl:variable name="tableDataSet" select="concat($dataObjectName,'Set')"/>
    <xsl:variable name="voAttribute" select="@Name"/>
    <xsl:variable name="entityUsage" select="@EntityUsage"/>
    <!--
       S. Davelaar: 10-08-2003 
       Find elements are always updatable.
       Lookup attributes part of readOnly EO usage are always non updatable
    -->
    <xsl:variable name="updateable">
      <xsl:choose>
        <xsl:when test="$isFindElement='true'">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:when test="/ViewObject/EntityUsage[@Name=$entityUsage]/@ReadOnly='true'">
          <xsl:value-of select="'false'"/>
        </xsl:when>
        <xsl:when test="@IsUpdateable!=''">
          <xsl:value-of select="@IsUpdateable"/>
        </xsl:when>
        <xsl:otherwise>true</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="eoAttributeName" select="@EntityAttrName"/>
    <xsl:variable name="columnType" select="$entityAttribute/@ColumnType"/>
    <xsl:variable name="eoDisplayType" select="$entityAttribute/Properties/Property[@Name='DISPLAY_TYPE']/@Value"/>
    <xsl:variable name="voDisplayType" select="Properties/Property[@Name='DISPLAY_TYPE']/@Value"/>

    <xsl:variable name="listValidator"
      select="$entityAttribute/ListValidationBean/AttrArray/Item">
     </xsl:variable>


    <!-- The check whether a Lookup is defined for
    the current attribute, depends on the Lookup displayType.
    If the displayType is lov, we match with the baseDisplayAttribute,
    otherwise we match with the baseValueAttribute -->

    <xsl:variable name="lookupLovValues"
      select="$group/Lookup[@displayType='lov' and @baseDisplayAttribute=$voAttribute]"/>

    <xsl:variable name="lookupChoiceValues"
      select="$group/Lookup[not(@displayType='lov') and @baseValueAttribute=$voAttribute]"/>

    <xsl:variable name="domain">
      <xsl:call-template name="getCustomProperty">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="propertyName" select="'DOMAIN'"/>
        <xsl:with-param name="defaultValue" select="''"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="displayType">
      <xsl:call-template name="setDisplayType">
        <xsl:with-param name="lookupLovValues" select="$lookupLovValues"/>
        <xsl:with-param name="updateable" select="$updateable"/>
        <xsl:with-param name="voDisplayType" select="$voDisplayType"/>
        <xsl:with-param name="eoDisplayType" select="$eoDisplayType"/>
        <xsl:with-param name="columnType" select="$columnType"/>
        <xsl:with-param name="listValidator" select="$listValidator"/>
        <xsl:with-param name="lookupChoiceValues" select="$lookupChoiceValues"/>
        <xsl:with-param name="domain" select="$domain"/>
        <xsl:with-param name="isFindElement" select="$isFindElement"/>
      </xsl:call-template>
    </xsl:variable>
<!--    <xsl:message>Attribute: <xsl:value-of select="$attributeName"/>, display type = <xsl:value-of select="$displayType"/></xsl:message> -->

    <xsl:variable name="required" select="$entityAttribute/@IsNotNull"/>

    <!-- Create the appropriate form element based on the display type  -->
    <xsl:choose>
      <xsl:when test="$display='hidden'">
        <xsl:call-template name="hiddenField">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='textInput'">
        <xsl:call-template name="textInput">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='fileUpload'">
        <xsl:call-template name="fileUpload">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='image'">
        <xsl:call-template name="image">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='fileDownload'">
        <xsl:call-template name="fileDownload">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='numberField'">
        <xsl:call-template name="numberField">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='secret'">
        <xsl:call-template name="secret">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='dateField'">
        <xsl:call-template name="dateField">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='datetimeField'">
        <xsl:call-template name="datetimeField">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='choice'">
        <xsl:call-template name="choice">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="lookupChoiceValues" select="$lookupChoiceValues"/>
          <xsl:with-param name="domain"          select="$domain"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
          <xsl:with-param name="listValidator"   select="$listValidator"/>
       </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='list'">
        <xsl:call-template name="list">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="lookupChoiceValues" select="$lookupChoiceValues"/>
          <xsl:with-param name="domain"          select="$domain"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
          <xsl:with-param name="listValidator"   select="$listValidator"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='lov'">
        <xsl:call-template name="lov">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="lookupLovValues" select="$lookupLovValues"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
          <xsl:with-param name="tableDataSet"    select="$tableDataSet"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='displayField'">
        <xsl:call-template name="styledText">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='editor'">
        <xsl:call-template name="editor">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="required"        select="$required"/>
          <xsl:with-param name="displayType"     select="$displayType"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="operator"        select="$operator"/>
          <xsl:with-param name="voAttribute"     select="$voAttribute"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="tableDataSet"    select="$tableDataSet"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$displayType='checkBox'">
        <xsl:call-template name="checkBox">
          <xsl:with-param name="attributeName"   select="$attributeName"/>
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
          <xsl:with-param name="isFindElement"   select="$isFindElement"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="updateable"      select="$updateable"/>
          <xsl:with-param name="domain"          select="$domain"/>
          <xsl:with-param name="listValidator"   select="$listValidator"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
         ATTRIBUTE <xsl:value-of select="@Name"/>: INVALID DISPLAY TYPE: <xsl:value-of select="$displayType"/>!!!
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template> <!-- end template createFormElement -->

  <xsl:template name="setDisplayType">
    <xsl:param name="lookupLovValues"/>
    <xsl:param name="updateable"/>
    <xsl:param name="voDisplayType"/>
    <xsl:param name="eoDisplayType"/>
    <xsl:param name="columnType"/>
    <xsl:param name="listValidator"/>
    <xsl:param name="lookupChoiceValues"/>
    <xsl:param name="domain"/>
    <xsl:param name="isFindElement"/>

    <xsl:variable name="customDisplayType">
      <xsl:choose>
        <xsl:when test="$voDisplayType!=''">
          <xsl:value-of select="$voDisplayType"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$eoDisplayType"/>
        </xsl:otherwise>
      </xsl:choose>  
    </xsl:variable>  

    <xsl:choose>

      <!-- set display type to lov when Lookup associated with
           this attribute that has displayType set to Lov -->
      <xsl:when test="count($lookupLovValues)!=0">lov</xsl:when>

      <!-- S. davelaar, added 29-07-2003
           set display type to lovchoice when Lookup associated with
           this attribute that has displayType set to choice -->
      <xsl:when test="count($lookupChoiceValues)!=0">choice</xsl:when>


      <!-- a checkbox cannot be used on find page, because the user cannot
           see not indicate the difference between selecting the unchecked
           value as query criterium versus not using the checkbox attribute 
           as query criterium at all. Thereforem we use choice instead
           of checkbox on find page -->
      <xsl:when test="$customDisplayType!=''">
        <xsl:choose>
          <xsl:when test="$customDisplayType='checkBox' and $isFindElement='true'">
            <xsl:value-of select="'choice'"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$customDisplayType"/>
          </xsl:otherwise>
        </xsl:choose>  
      </xsl:when>

      <!-- if display type not set, and underlying EO attribute column
           is a date, than set display type to dateField

           S. Davelaar: 29-07-2003: even when field is readOnly we 
           use dateField with readOnly is true. If we would use styledText
           we would loose the proper date formatting!

           NOTE: dateField must be directly enclosed with the xsl:when
           tags, to prevent spaces being added to the value of displayType!!!
      -->
      <xsl:when test="$columnType='DATE'">dateField</xsl:when>

      <!-- if display type not set, and underlying EO attribute column
           is a BLOB, than set display type to uploadFile if vo attribute
           is updatavble. If not updatable, set to fileDownload
       -->
      <xsl:when test="$columnType='BLOB' and $updateable='false'">fileDownload</xsl:when>
      <xsl:when test="$columnType='BLOB' and $updateable='true'">fileUpload</xsl:when>

      <!-- default display type to textInput -->

      <!-- if no display type is set, default to choice if the attribute
      either has a listValidator, domain or Lookup with displayType<> lov

      
     S. Davelaar: 29-07-2003: even when choice is readOnly we 
     use choice with readOnly is true. If we would use styledText
     we cannot display the meaning, only the value.

      -->

      <xsl:when test="count($listValidator)!=0">choice</xsl:when>
      <xsl:when test="count($lookupChoiceValues)!=0">choice</xsl:when>
      <xsl:when test="$domain!=''">choice</xsl:when>

      <!-- if display type not set, and underlying EO attribute column
           is a number, than set display type to numberField
      -->
<!-- numberField commented out for now for two reaosns: results in ugly
     javascript error box with escape character, and , worse, this error
     is also raised on empty UiOnly REQUIRED number field which is undesirable, 
     in particula in table layout

        <xsl:when test="$columnType='NUMBER'">numberField</xsl:when>
-->
        <!-- default display type to textInput or styledText when read-only -->

      <!-- if VO attribute is not updateable, than set
      display type to displayField. We check this after lookups, so
      we can use a non-updateable lookup attribute for the lov 
      S. Davelaar 18-1-03: Non updateabe fields can still be queriable
      and should be displayed as input field on find page! -->
      <xsl:when test="$updateable='false' and $isFindElement='false'">
        <xsl:value-of select="'displayField'"/>
      </xsl:when>

        <xsl:otherwise>textInput</xsl:otherwise>
      </xsl:choose>
  </xsl:template> <!-- end setDisplayType -->


  <!-- Set generic attributes that applyto all types of form elements:
       name, id, prompt, required, dataObject, data:text, data:readOnly
   -->
  <xsl:template name="fieldProps">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"  select="'false'" />
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>
    <xsl:param name="group"/>

    <!-- Set name -->
    <xsl:attribute name="name">
      <xsl:value-of select="$attributeName"/>
    </xsl:attribute>

    <xsl:call-template name="promptAttribute">
      <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      <xsl:with-param name="prompt" select="$elementPrompt"/>
    </xsl:call-template>

    <xsl:call-template name="textAttribute">
      <xsl:with-param name="attributeName" select="$attributeName"/>
      <xsl:with-param name="layoutStyle"   select="$layoutStyle"/>
      <xsl:with-param name="isFindElement" select="$isFindElement"/>
      <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
    </xsl:call-template>

    <!-- add attributes that do not apply to find items -->
    <xsl:if test="$isFindElement!='true'">

      <xsl:attribute name="anchor">
        <xsl:value-of select="$attributeName"/>
      </xsl:attribute>  
      <xsl:call-template name="persistentAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="updateable" select="$updateable"/>
      </xsl:call-template>

      <!-- add dataObject property so the save process knows which
           items on the page belong to which dataObject -->
      <xsl:attribute name="dataObject">
        <xsl:choose>
          <xsl:when test="$layoutStyle='table'">
            <xsl:value-of select="concat($dataObjectName,'Set')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$dataObjectName"/>
          </xsl:otherwise>
        </xsl:choose>  
      </xsl:attribute>

      <!-- Set required attribute if underlying entity attribute is required
           Note: when the field is in a table layout, we set the value
           to uiOnly, which causes the asterisk to appear but does not
           check the requiredness using javascript. This allows for a
           table with new rows displayed without the user being forced
           to enter the reqire fields in all new rows. We use
           JHeadstart-speciifc javascript to check required fields in
           a table properly, excluding new rows that do not contain
           any data -->
      <xsl:if test="$required='true' and $displayType!='displayField'">
        <xsl:attribute name="required">
           <xsl:choose>
             <xsl:when test="$layoutStyle='table'">
               <xsl:value-of select="'uiOnly'"/>
             </xsl:when>
             <xsl:otherwise>
               <xsl:value-of select="'yes'"/>
             </xsl:otherwise>
           </xsl:choose>
        </xsl:attribute>
      </xsl:if>
    
    </xsl:if> <!-- end if attribute is not find item -->

    <xsl:variable name="hinttext">
      <xsl:call-template name="getCustomProperty">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="propertyName" select="'HINT_TEXT'"/>
        <xsl:with-param name="defaultValue" select="''"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:if test="$hinttext!='' and $showHintText='true'">
      <xsl:choose>
        <xsl:when test="not($layoutStyle='table')">
          <xsl:attribute name="tip">
            <xsl:value-of select="$hinttext"/>
          </xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="shortDesc">
            <xsl:value-of select="$hinttext"/>
          </xsl:attribute>
        </xsl:otherwise>  
      </xsl:choose> 
    </xsl:if>  

    <xsl:if test="$isFindElement='true'">
      <xsl:variable name="caseInsensitive">
        <xsl:call-template name="getCustomProperty">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="propertyName" select="'CASE_INSENSITIVE'"/>
          <xsl:with-param name="defaultValue" select="''"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:if test="$caseInsensitive='true'">
        <xsl:attribute name="isCaseInsensitive">
          <xsl:value-of select="'true'"/>
        </xsl:attribute>
      </xsl:if>
    </xsl:if>  

<!--
    <xsl:call-template name="readOnlyBinding">
      <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
      <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
    </xsl:call-template>
-->

    <!-- add attributes that only apply to find items -->
    <xsl:if test="$isFindElement='true'">
      <xsl:call-template name="findItemProperties">
        <xsl:with-param name="operator" select="$operator"/>
        <xsl:with-param name="voAttribute" select="$voAttribute"/>
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template name="findItemProperties">
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    
    <!-- add attributes that only apply to find items -->
    <xsl:attribute name="queryOperator">
      <xsl:value-of select="$operator"/>
    </xsl:attribute>
    <xsl:attribute name="persistentAttribute">
      <xsl:value-of select="$voAttribute"/>
    </xsl:attribute>


  </xsl:template>

  <xsl:template name="lov">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="lookupLovValues"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>
    <xsl:param name="tableDataSet"/>

     <!-- first intialize variables required for LOV onClick attribute
          Note that we don't need a avraiable for lookupDisplayAttribute
          as that will always be the current attribute -->
     <xsl:variable name="baseValueAttribute">
        <xsl:if test="$layoutStyle='find'">
          <xsl:value-of select="$findItemPrefix"/>
        </xsl:if>
       <xsl:value-of select="$lookupLovValues/@baseValueAttribute"/>
     </xsl:variable>
     <xsl:variable name="baseDisplayAttribute">
        <xsl:if test="$layoutStyle='find'">
          <xsl:value-of select="$findItemPrefix"/>
        </xsl:if>
       <xsl:value-of select="$lookupLovValues/@baseDisplayAttribute"/>
     </xsl:variable>
     <xsl:variable name="lookupValueAttribute">
       <xsl:value-of select="$lookupLovValues/@lookupValueAttribute"/>
     </xsl:variable>
     <xsl:variable name="lookupDisplayAttribute">
       <xsl:value-of select="$lookupLovValues/@lookupDisplayAttribute"/>
     </xsl:variable>

     <xsl:variable name="lovTitle">
      <xsl:choose>
       <xsl:when test="$lookupLovValues/@title!=''">
         <xsl:value-of select="$lookupLovValues/@title"/>
       </xsl:when>
       <xsl:otherwise>
         <xsl:value-of select="concat('Find+',$lookupLovValues/@dataObject)"/>
       </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

   <!-- This dummy text element is required to get proper escaping of the
        quotes in the editor onClick command. Very weird! -->
   <xsl:text disable-output-escaping="yes">
   </xsl:text>

    <messageLovField>
      <xsl:call-template name="fieldProps">
        <xsl:with-param name="attributeName"   select="$attributeName"/>
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
        <xsl:with-param name="isFindElement"   select="$isFindElement"/>
        <xsl:with-param name="required"        select="$required"/>
        <xsl:with-param name="displayType"     select="$displayType"/>
        <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
        <xsl:with-param name="operator"        select="$operator"/>
        <xsl:with-param name="voAttribute"     select="$voAttribute"/>
        <xsl:with-param name="updateable"      select="$updateable"/>
        <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
      </xsl:call-template>

      <xsl:call-template name="lovTypeInSupport">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="updateable"      select="$updateable"/>
      </xsl:call-template>

      <xsl:call-template name="columnsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      
      <xsl:call-template name="rowsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>

      <!-- After this next template, all simple attributes (i.e. data:attr) attributes
           are added. Now we add the readOnly binding which can be simple
           but in some cases is a boundAttribute child. After this template
           call, only boundAttribute child attributes may be added
      -->
      <xsl:if test="$isFindElement='false'">
        <!-- S. Dave;aar 29-07-2003: quick fix for nasty problem:
             when lov is on lookup attribute, the updatability is by
             definiton false because lookup atributes ina a VO are always
             non-updatable. Therefore, of updatable is false, we ignore this
             and do not set the readOnly property. 
             If the attribute wasn't a lookup attribute 
             after all, we also make it updatable. This is in itself not
             correct, but for the user it is easy to work around this
             limitation: remove the lov Lookup alltogether -->
        <xsl:if test="$updateable!='false'">
           <xsl:call-template name="readOnlyBinding">
            <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
            <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
            <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
          </xsl:call-template>
        </xsl:if>  
      </xsl:if>
      
      <xsl:call-template name="lovAttribute">
        <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
        <xsl:with-param name="lovTitle" select="$lovTitle"/>
        <xsl:with-param name="lookupLovValues" select="$lookupLovValues"/>
        <xsl:with-param name="lookupDisplayAttribute" select="$lookupDisplayAttribute"/>
        <xsl:with-param name="lookupValueAttribute" select="$lookupValueAttribute"/>
        <xsl:with-param name="baseDisplayAttribute" select="$baseDisplayAttribute"/>
        <xsl:with-param name="baseValueAttribute" select="$baseValueAttribute"/>
        <xsl:with-param name="tableDataSet" select="$tableDataSet"/>
      </xsl:call-template>  
    </messageLovField>
  </xsl:template>

  <!-- set text attribute
       Binding depends on whether item is displayed in find form
       or normal form -->
  <xsl:template name="textAttribute">
    <xsl:param name="attributeName"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="dataObjectName"/>

    <xsl:call-template name="dataBoundAttribute">
      <xsl:with-param name="name">data:text</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:call-template name="valueBinding">
          <xsl:with-param name="attributeName" select="$attributeName"/>
          <xsl:with-param name="layoutStyle"   select="$layoutStyle"/>
          <xsl:with-param name="isFindElement" select="$isFindElement"/>
          <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <!-- set value attribute
       Binding depends on whether item is displayed in find form
       or normal form -->
  <xsl:template name="valueAttribute">
    <xsl:param name="attributeName"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="dataObjectName"/>

    <xsl:call-template name="dataBoundAttribute">
      <xsl:with-param name="name">data:value</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:call-template name="valueBinding">
          <xsl:with-param name="attributeName" select="$attributeName"/>
          <xsl:with-param name="layoutStyle"   select="$layoutStyle"/>
          <xsl:with-param name="isFindElement" select="$isFindElement"/>
          <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:template>


  <xsl:template name="editorImage">
    <xsl:param name="layoutStyle"/>
    <xsl:param name="tableDataSet"/>
    <xsl:param name="attributeName"/>
    <xsl:param name="maxLength"/>

    <!-- This template is layout-specific, because in a table the
        attribute-name is a concatenation of the tablename, attributename
        and row num -->
   <end>
     <image>
       <boundAttribute name="source">
         <concat>
           <dataObject select="requestContextPath" source="jheadstart:sessionData"/>
           <xsl:text>/jheadstart/images/editor.gif</xsl:text>
         </concat>
       </boundAttribute>  
       <xsl:choose>
         <xsl:when test="$layoutStyle='table'">
           <boundAttribute name="destination">
             <concat>
               <xsl:text>javascript:editorPopup('</xsl:text>
               <dataObject select="requestContextPath" source="jheadstart:sessionData"/>
               <xsl:text>/jheadstart/notes.jsp','</xsl:text>
               <xsl:value-of select="$formName"/>
               <xsl:text>','</xsl:text>
               <xsl:value-of select="concat($tableDataSet,':',$attributeName,':')"/>
               <dataObject select="rownum"/>
               <xsl:text>',</xsl:text>
               <xsl:value-of select="$maxLength"/>
               <xsl:text>);</xsl:text>
             </concat>
           </boundAttribute>
         </xsl:when>
         <xsl:otherwise>
           <boundAttribute name="destination" >
             <concat>
               <xsl:text>javascript:editorPopup('</xsl:text>
               <dataObject select="requestContextPath" source="jheadstart:sessionData"/>
               <xsl:text>/jheadstart/notes.jsp','</xsl:text>
               <xsl:value-of select="$formName"/>
               <xsl:text>','</xsl:text>
               <xsl:value-of select="$attributeName"/>
               <xsl:text>',</xsl:text>
               <xsl:value-of select="$maxLength"/>
               <xsl:text>);</xsl:text>
             </concat>  
           </boundAttribute>
         </xsl:otherwise>
       </xsl:choose>
     </image>
   </end>
  </xsl:template>

  <!-- Template to construct onClick attribute that invokes generic LOV
       window. Example onClick attribute:

       onClick="lovPopup('/jheadstart/lovFrame.jsp?title=Find+Manager&amp;
                class=oracle.empdepdemo.dataObject.Employee&amp;displayAttr=Ename
                &amp;valueAttr=Empno;','EmployeeForm','Mgr'); return false;"/>
  -->
  <xsl:template name="lovAttribute">
    <xsl:param name="layoutStyle"/>
    <xsl:param name="lovTitle"/>
    <xsl:param name="lookupLovValues"/>
    <xsl:param name="lookupDisplayAttribute"/>
    <xsl:param name="lookupValueAttribute"/>
    <xsl:param name="baseDisplayAttribute"/>
    <xsl:param name="baseValueAttribute"/>
    <xsl:param name="tableDataSet"/>
    <!-- This template is layout-specific, because in a table the
        attribute-name is a concatenation of the tablename, attributename
        and row num -->
<!-- <xsl:message>AAP: <xsl:value-of select="$tableDataSet"/></xsl:message> -->
    <xsl:choose>
      <xsl:when test="$layoutStyle='table'">
        <boundAttribute name="onClick">
          <concat>
            <xsl:text>lovPopup('</xsl:text>
            <dataObject select="requestContextPath" source="jheadstart:sessionData"/>
            <xsl:text>/jheadstart/lovFrame.jsp?title=</xsl:text>
            <dataObject select="{utils:getNlsKey('LOV_TITLE',$lovTitle, concat('LOV_TITLE_',$lookupLovValues/@name))}" 
                        source="jheadstart:nls"/>
            <xsl:text disable-output-escaping="yes">&amp;amp;class=</xsl:text>
            <xsl:value-of select="$modelBase"/>
            <xsl:value-of select="$lookupLovValues/@dataObject"/>
            <xsl:text disable-output-escaping="yes">&amp;amp;queryType=</xsl:text>
            <xsl:value-of select="$lookupLovValues/@queryType"/>
            <xsl:text disable-output-escaping="yes">&amp;amp;displayAttr=</xsl:text>
            <xsl:value-of select="$lookupDisplayAttribute"/>
            <xsl:text disable-output-escaping="yes">&amp;amp;valueAttr=</xsl:text>
            <xsl:value-of select="$lookupValueAttribute"/>
            <xsl:text>','</xsl:text>
            <xsl:value-of select="$formName"/>
            <xsl:text>','</xsl:text>
            <xsl:value-of select="concat($tableDataSet,':',$baseDisplayAttribute,':')"/>
            <dataObject select="rownum"/>
            <xsl:text>','</xsl:text>
            <xsl:value-of select="concat($tableDataSet,':',$baseValueAttribute,':')"/>
            <dataObject select="rownum"/>
            <xsl:text>'); return false;</xsl:text>
          </concat>
        </boundAttribute>
      </xsl:when>
      <xsl:otherwise>
        <boundAttribute name="onClick" >
          <concat> 
            <xsl:text>lovPopup('</xsl:text>
            <dataObject select="requestContextPath" source="jheadstart:sessionData"/>
            <xsl:text>/jheadstart/lovFrame.jsp?title=</xsl:text>
            <dataObject select="{utils:getNlsKey('LOV_TITLE',$lovTitle, concat('LOV_TITLE_',$lookupLovValues/@name))}" 
                        source="jheadstart:nls"/>
            <xsl:text disable-output-escaping="yes">&amp;amp;class=</xsl:text>
            <xsl:value-of select="$modelBase"/>
            <xsl:value-of select="$lookupLovValues/@dataObject"/>
            <xsl:text disable-output-escaping="yes">&amp;amp;queryType=</xsl:text>
            <xsl:value-of select="$lookupLovValues/@queryType"/>
            <xsl:text disable-output-escaping="yes">&amp;amp;displayAttr=</xsl:text>
            <xsl:value-of select="$lookupDisplayAttribute"/>
            <xsl:text disable-output-escaping="yes">&amp;amp;valueAttr=</xsl:text>
            <xsl:value-of select="$lookupValueAttribute"/>
            <xsl:text>','</xsl:text>
            <xsl:value-of select="$formName"/>
            <xsl:text>','</xsl:text>
            <xsl:value-of select="$baseDisplayAttribute"/>
            <xsl:text>','</xsl:text>
            <xsl:value-of select="$baseValueAttribute"/>
            <xsl:text>'); return false;</xsl:text>
          </concat>
        </boundAttribute>  
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

<!--
               <radioGroup name="Job" data:text="text" data:value="text"
                  data:selectedValue="Job@jheadstart:Employee">
                <childData>
                 <t text="CLERK"/>
                 <t text="MANAGER"/>
                 <t text="SALESMAN"/>
                 <t text="PRESIDENT"/>
                 <t text="CLERK"/>
                </childData>
               </radioGroup>

   <messageRadioGroup prompt="Depje" name="Deptno"
      data:text="Dname" data:value="Deptno"
      data:selectedValue="Deptno@jheadstart:Employee"
      data:childData="DepartmentList@jheadstart:sessionData"/>


-->
  <xsl:template name="option">
    <xsl:param name="entityAttribute"/>
    <xsl:param name="attributeName"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="lookupChoiceValues"/>
    <xsl:param name="domain"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="dataObjectName"/>
    <xsl:param name="listValidator"/>

     <!-- add selectedValue attribute -->
    <xsl:call-template name="dataBoundAttribute">
      <xsl:with-param name="name">data:selectedValue</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:call-template name="valueBinding">
          <xsl:with-param name="attributeName" select="$attributeName"/>
          <xsl:with-param name="layoutStyle"   select="$layoutStyle"/>
          <xsl:with-param name="isFindElement" select="$isFindElement"/>
          <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
     <!-- call insertOnlyInTable here because it causes boundAttribute
          child element to be added, which must be added after normal attributes -->
      <xsl:if test="$isFindElement='false'">
        <xsl:call-template name="readOnlyBinding">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
     </xsl:if>      
     <xsl:choose>
       <xsl:when test="$lookupChoiceValues/@name!=''">
         <!-- Lookup defined in application structure -->
         <xsl:call-template name="lookupOption">
           <xsl:with-param name="lookupChoiceValues" select="$lookupChoiceValues"/>
         </xsl:call-template>
       </xsl:when>
       <xsl:when test="$domain!=''">
         <!-- Domain defined in DomainDefinitions.xml -->
         <xsl:call-template name="domainOption">
           <xsl:with-param name="domain" select="$domain"/>
         </xsl:call-template>
       </xsl:when>
       <xsl:otherwise>
         <!-- List of options defined in ListValidator of attribute -->
         <xsl:call-template name="listValidatorOption">
           <xsl:with-param name="listValidator"   select="$listValidator"/>
         </xsl:call-template>
       </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="listValidatorOption">
     <xsl:param name="listValidator"/>
     <contents>
      <!-- add one dummy option to show empty row in poplist -->
      <option value="" text=""/>
      <xsl:for-each select="$listValidator">
        <option>
          <xsl:attribute name="value">
           <xsl:value-of select="@Value"/>
         </xsl:attribute>
          <xsl:attribute name="text">
            <xsl:value-of select="@Value"/>
          </xsl:attribute>
        </option>
      </xsl:for-each>
     </contents>
  </xsl:template>

  <!-- lookupOption template.  -->
  <xsl:template name="lookupOption">
    <xsl:param name="lookupChoiceValues"/>
    
     <xsl:variable name="lookupName">
       <xsl:value-of select="$lookupChoiceValues/@name"/>
     </xsl:variable>
     <xsl:variable name="lookupValueAttribute">
       <xsl:value-of select="concat($lookupChoiceValues/@lookupValueAttribute,'')"/>
     </xsl:variable>
     <xsl:variable name="lookupDisplayAttribute">
       <xsl:value-of select="concat($lookupChoiceValues/@lookupDisplayAttribute,'')"/>
     </xsl:variable>
    <contents>
      <!-- set data:childData attribute
      <xsl:variable name="attributeName" select="'data:childData'"/>
      <xsl:variable name="attributeValue">
        <xsl:value-of select="concat($lookupName,'@jheadstart:sessionData')"/>
      </xsl:variable>
-->
      <xsl:call-template name="boundAttribute">
        <xsl:with-param name="attributeName" select="'data:childData'"/>
        <xsl:with-param name="attributeValue" select="concat($lookupName,'@jheadstart:sessionData')"/>
      </xsl:call-template>

      <option>
        <!-- set data:value attribute
        <xsl:variable name="attributeName" select="'data:value'"/>
        <xsl:variable name="attributeValue" select="$lookupValueAttribute"/>
        -->
        <xsl:call-template name="boundAttribute">
          <xsl:with-param name="attributeName" select="'data:value'"/>
          <xsl:with-param name="attributeValue" select="$lookupValueAttribute"/>
        </xsl:call-template>
        <!-- set data:text attribute -->
        <!-- Don't call boundAttribute template directly because
        it results in following error:
          XSL-1050: Duplicate variable 'attributeName' definition.

        <xsl:variable name="attributeName" select="'data:text'"/>
        <xsl:variable name="attributeValue" select="$lookupDisplayAttribute"/>
        <xsl:call-template name="boundAttribute"/>

          work around: wrap this in new template
        
        <xsl:call-template name="optionText"/>
        <xsl:variable name="attributeName" select="'data:text'"/>
        <xsl:variable name="attributeValue" select="$lookupDisplayAttribute"/>
        -->
        <xsl:call-template name="boundAttribute">
          <xsl:with-param name="attributeName" select="'data:text'"/>
          <xsl:with-param name="attributeValue" select="$lookupDisplayAttribute"/>
        </xsl:call-template>
      </option>
    </contents>
  </xsl:template>

<!-- P. Ebell - merged into lookupOption template

  <xsl:template name="optionText">
        <xsl:variable name="attributeName" select="'data:text'"/>
        <xsl:variable name="attributeValue" select="$lookupDisplayAttribute"/>
        <xsl:call-template name="boundAttribute">
          <xsl:with-param name="attributeName" select="$attributeName"/>
          <xsl:with-param name="attributeValue" select="$attributeValue"/>
        </xsl:call-template>
  </xsl:template>
-->

  <xsl:template name="domainOption">
    <xsl:param name="domain"/>
    
     <contents>
      <!-- add one dummy option to show empty row in poplist -->
      <option value="" text=""/>
      <xsl:for-each select="$domainDef/DomainSet/Domain[@name=$domain]/AllowableValue">
        <option>
          <xsl:attribute name="value">
           <xsl:value-of select="@value"/>
         </xsl:attribute>
          <xsl:attribute name="text">
            <xsl:value-of select="@meaning"/>
          </xsl:attribute>
        </option>
      </xsl:for-each>
     </contents>
  </xsl:template>

  <!-- set property -->
  <xsl:template name="setProperty">
    <xsl:attribute name="{$attributeName}">
      <xsl:value-of select="$propertyValue"/>
    </xsl:attribute>
  </xsl:template>

  <!-- Get custom property
       Invoke this template within xsl:variable element to get the
       custom property value assigned to the variable
       Variables that must exist:
       - entityDef, eoAttributeName, propertyName, defaultValue
   -->
  <xsl:template name="getCustomProperty">
    <xsl:param name="entityAttribute"/>
    <xsl:param name="propertyName"/>
    <xsl:param name="defaultValue"/>

    <xsl:variable name="eoType" select="$entityAttribute/@Type"/> 
    <xsl:variable name="eoProperty" select="$entityAttribute/Properties/Property[@Name=$propertyName]/@Value"/> 
    <xsl:variable name="voProperty" select="Properties/Property[@Name=$propertyName]/@Value"/>

    <!-- Figure out the value of the display height, First check the VO attribute
         for the custom property, then check the EO attribute for
         the same property. If both properties donot exist/are null, then
         assign defaultValue
    -->
      <xsl:choose>
        <xsl:when test="$voProperty!=''">
          <xsl:value-of select="$voProperty"/>
        </xsl:when>
        <xsl:when test="$eoProperty!=''">
          <xsl:value-of select="$eoProperty"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$defaultValue"/>
        </xsl:otherwise>
      </xsl:choose>

  </xsl:template> <!-- end get custom property template -->

  <!-- Set custom property
   -->
  <xsl:template name="customProperty">
    <xsl:param name="entityAttribute"/>
    <xsl:param name="attributeName"/>
    <xsl:param name="propertyName"/>
    <xsl:param name="defaultValue"/>
<!--
    <xsl:variable name="eoProperty" select="$entityDef/Entity/Attribute[@Name=$eoAttributeName]/Properties/Property[@Name=$propertyName]/@Value"/>
    <xsl:variable name="voProperty" select="Properties/Property[@Name=$propertyName]/@Value"/>

-->
    <!-- Figure out the value of the display height, First check the VO attribute
         for a custom DISPLAY_TYPE property, then check the EO attribute for
         the same property. If both properties donot exist/are null, then
         set the height to 1
    -->

<!--
    <xsl:variable name = "propertyValue">
      <xsl:choose>
        <xsl:when test="$voProperty!=''">
          <xsl:value-of select="$voProperty"/>
        </xsl:when>
        <xsl:when test="$eoProperty!=''">
          <xsl:value-of select="$eoProperty"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$defaultValue"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
-->    
    <xsl:variable name="propertyValue">
      <xsl:call-template name="getCustomProperty">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="propertyName" select="$propertyName"/>
        <xsl:with-param name="defaultValue" select="$defaultValue"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:attribute name="{$attributeName}">
       <xsl:value-of select="$propertyValue"/>
    </xsl:attribute>

  </xsl:template> <!-- end custom property template -->

  <!-- set persistent attribute -->
  <xsl:template name="persistentAttribute">
    <xsl:param name="entityAttribute"/>
    <xsl:param name="updateable"/>

    <xsl:attribute name="isPersistent">
      <xsl:choose>
        <!-- S. Davelaar, 08-09-2003: updatable can also be "while_insert",
             this should be trabslated to true! -->
        <xsl:when test="$updateable = 'while_insert'">
          <xsl:value-of select="'true'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$updateable"/>
        </xsl:otherwise>  
      </xsl:choose>
    </xsl:attribute>
  </xsl:template>



  <!-- set attributes for type in support for LOVs -->
  <xsl:template name="lovTypeInSupport">
    <xsl:param name="entityAttribute"/>
    <xsl:param name="updateable"/>

      <!-- if the lov field ends up being updateable, this means we
           need to enable type in support, otherwise prevent access to the field.
      -->

      <xsl:choose>
        <xsl:when test="$updateable='false'">
          <!-- Disable the field by means of a message            
          <xsl:attribute name="onChange">alert(getMessage('mustUseLov'));return false</xsl:attribute>
          <xsl:attribute name="onKeyPress">alert(getMessage('mustUseLov'));return false</xsl:attribute>
          -->
          <!-- onChange/onKeyPress is tricky to get to work on all browsers. We can display
               the message upon entering the field, and prevent access to it instead.
               This will not allow the user to scroll in the queried value
          -->
          <xsl:attribute name="onFocus">this.blur();alert(getMessage('mustUseLov'));return false</xsl:attribute>

          <!-- This is another option but I like the user friendly message   
          <xsl:attribute name="maximumLength">
            <xsl:value-of select="'0'"/>
          </xsl:attribute>
          -->          
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="maximumLengthAttribute">
            <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>

  </xsl:template>



  <!-- set prompt attribute -->
  <xsl:template name="promptAttribute">
    <!-- prompt to use can be specified as parameter so
         we can specify from and to prompts for range search
         in find form -->
    <xsl:param name="prompt"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="uixAttributeName"  select="'promptAndAccessKey'"/>
    <xsl:param name="prefixWithAmpersand"  select="'true'"/>

    <xsl:variable name="prefix">
      <xsl:if test="$prefixWithAmpersand='true'">
        <xsl:value-of  select="'&amp;amp;'"/>
      </xsl:if>
    </xsl:variable>
    <!-- create variable with prompt value -->
    <xsl:variable name="promptValue">           
      <!-- test whether prompt is null, somehow the default value
       does not work with getCustomProperty used within xsl:param
       element -->
      <xsl:choose>
        <xsl:when test="$prompt=''">
          <xsl:variable name="customPrompt">
            <!-- get prompt from custom property when not specified -->
            <xsl:call-template name="getCustomProperty">
              <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
              <xsl:with-param name="propertyName" select="'PROMPT'"/>
              <xsl:with-param name="defaultValue" select="@Name"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:value-of disable-output-escaping="yes" select="concat($prefix,$customPrompt)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of disable-output-escaping="yes" select="concat($prefix,$prompt)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>       

    <xsl:variable name="promptKey">
      <xsl:choose>
        <xsl:when test="$prompt!=''">
          <xsl:value-of select="$prompt"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="string:toUpperCase(concat($groupName,'_',@Name))"/>
        </xsl:otherwise>
      </xsl:choose>  
    </xsl:variable>

    <xsl:call-template name="nlsTextAttribute">
      <xsl:with-param name="attributeName" select="$uixAttributeName"/>
      <xsl:with-param name="key">
        <xsl:value-of select="$promptKey"/>
      </xsl:with-param>
      <xsl:with-param name="value">
        <xsl:value-of select="$promptValue"/>
      </xsl:with-param>
      <xsl:with-param name="addToNlsBundle">true</xsl:with-param>
    </xsl:call-template>

  </xsl:template>

  <!-- set rows attribute -->
  <xsl:template name="rowsAttribute">
    <xsl:param name="entityAttribute"/>

    <xsl:call-template name="customProperty">
      <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      <xsl:with-param name="attributeName" select="'rows'"/>
      <xsl:with-param name="propertyName" select="'DISPLAY_HEIGHT'"/>
      <xsl:with-param name="defaultValue" select="'1'"/>
    </xsl:call-template>
  </xsl:template>

  <!-- set size attribute -->
  <xsl:template name="sizeAttribute">
    <xsl:param name="entityAttribute"/>

    <xsl:call-template name="customProperty">
      <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      <xsl:with-param name="attributeName" select="'size'"/>
      <xsl:with-param name="propertyName" select="'DISPLAY_HEIGHT'"/>
      <xsl:with-param name="defaultValue" select="'5'"/>
    </xsl:call-template>
  </xsl:template>

  <!-- set columns attribute -->
  <xsl:template name="columnsAttribute">
    <xsl:param name="entityAttribute"/>

    <xsl:call-template name="customProperty">
      <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      <xsl:with-param name="attributeName" select="'columns'"/>
      <xsl:with-param name="propertyName" select="'DISPLAY_WIDTH'"/>
      <xsl:with-param name="defaultValue" select="$entityAttribute/@Precision"/>
    </xsl:call-template>
  </xsl:template>

  <!-- set maximumLength attribute -->
  <xsl:template name="maximumLengthAttribute">
    <xsl:param name="entityAttribute"/>

    <xsl:call-template name="customProperty">
      <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      <xsl:with-param name="attributeName" select="'maximumLength'"/>
      <xsl:with-param name="propertyName" select="'MAXIMUM_LENGTH'"/>
      <xsl:with-param name="defaultValue" select="$entityAttribute/@Precision"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="checkBox">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="updateable"/>
    <xsl:param name="domain"/>
    <xsl:param name="listValidator"/>
    <xsl:param name="dataObjectName"/>

    <messageCheckBox>
      <!-- Set name -->
      <xsl:attribute name="name">
        <xsl:value-of select="$attributeName"/>
      </xsl:attribute>  
      <xsl:call-template name="promptAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="prompt" select="$elementPrompt"/>
      </xsl:call-template>
      <xsl:attribute name="anchor">
        <xsl:value-of select="$attributeName"/>
      </xsl:attribute>  
      <xsl:call-template name="persistentAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="updateable" select="$updateable"/>
      </xsl:call-template>
          
      <!-- add dataObject property so the save process knows which
           items on the page belong to which dataObject -->
      <xsl:attribute name="dataObject">
        <xsl:choose>
          <xsl:when test="$layoutStyle='table'">
            <xsl:value-of select="concat($dataObjectName,'Set')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$dataObjectName"/>
          </xsl:otherwise>
        </xsl:choose>  
      </xsl:attribute>

      <!-- value is first allowable value in domain -->
      <xsl:attribute name="value">
         <xsl:choose>
           <xsl:when test="$domain!=''">
             <!-- take first value of Domain defined in DomainDefinitions.xml -->
             <xsl:value-of select="$domainDef/DomainSet/Domain[@name=$domain]/AllowableValue[1]/@value"/>
           </xsl:when>
           <xsl:otherwise>
             <!-- take first value defined in ListValidator of attribute -->
             <xsl:value-of select="$listValidator[1]/@Value"/>
           </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>

      <!-- uncheckedValue is second allowable value in domain -->
      <xsl:attribute name="uncheckedValue">
         <xsl:choose>
           <xsl:when test="$domain!=''">
             <!-- take first value of Domain defined in DomainDefinitions.xml -->
             <xsl:value-of select="$domainDef/DomainSet/Domain[@name=$domain]/AllowableValue[2]/@value"/>
           </xsl:when>
           <xsl:otherwise>
             <!-- take first value defined in ListValidator of attribute -->
             <xsl:value-of select="$listValidator[2]/@Value"/>
           </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>

      <!-- add selectedValue attribute -->
      <xsl:call-template name="dataBoundAttribute">
        <xsl:with-param name="name">data:selectedValue</xsl:with-param>
        <xsl:with-param name="value">
          <xsl:call-template name="valueBinding">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="layoutStyle"   select="$layoutStyle"/>
            <xsl:with-param name="isFindElement" select="$isFindElement"/>
            <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
          </xsl:call-template>
        </xsl:with-param>
      </xsl:call-template>

      <xsl:if test="$isFindElement='false'">
        <xsl:call-template name="readOnlyBinding">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
          <xsl:with-param name="propertyName"    select="'disabled'"/>
        </xsl:call-template>
      </xsl:if>      
    </messageCheckBox>
  </xsl:template>

  <xsl:template name="hiddenField">
    <xsl:param name="attributeName"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>

    <formValue>
      <xsl:attribute name="name">
        <xsl:value-of select="$attributeName"/>
      </xsl:attribute>

      <xsl:choose>
        <!-- add attributes that only apply to find items -->
        <xsl:when test="$isFindElement='true'">
          <xsl:call-template name="findItemProperties">
            <xsl:with-param name="operator" select="$operator"/>
            <xsl:with-param name="voAttribute" select="$voAttribute"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <!-- add attributes that do not apply to find items -->
          <xsl:call-template name="valueAttribute">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="layoutStyle"   select="$layoutStyle"/>
            <xsl:with-param name="isFindElement" select="$isFindElement"/>
            <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
          </xsl:call-template>
          <xsl:call-template name="persistentAttribute">
            <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
            <xsl:with-param name="updateable" select="$updateable"/>
          </xsl:call-template>

          <!-- add dataObject property so the save process knows which
               items on the page belong to which dataObject -->
          <xsl:attribute name="dataObject">
            <xsl:choose>
              <xsl:when test="$layoutStyle='table'">
                <xsl:value-of select="concat($dataObjectName,'Set')"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="$dataObjectName"/>
              </xsl:otherwise>
            </xsl:choose>  
          </xsl:attribute>

        </xsl:otherwise>
      </xsl:choose>
    </formValue>

  </xsl:template>

  <xsl:template name="textInput">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>

    <messageTextInput>
      <xsl:call-template name="fieldProps">
        <xsl:with-param name="attributeName"   select="$attributeName"/>
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
        <xsl:with-param name="isFindElement"   select="$isFindElement"/>
        <xsl:with-param name="required"        select="$required"/>
        <xsl:with-param name="displayType"     select="$displayType"/>
        <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
        <xsl:with-param name="operator"        select="$operator"/>
        <xsl:with-param name="voAttribute"     select="$voAttribute"/>
        <xsl:with-param name="updateable"      select="$updateable"/>
        <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
      </xsl:call-template>

      <xsl:call-template name="maximumLengthAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>

      <xsl:call-template name="columnsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      
      <xsl:call-template name="rowsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>

      <xsl:if test="$isFindElement='false'">
        <xsl:call-template name="readOnlyBinding">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:if>
    </messageTextInput>
  </xsl:template>

  <xsl:template name="fileUpload">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>

    <messageFileUpload>
      <xsl:call-template name="fieldProps">
        <xsl:with-param name="attributeName"   select="$attributeName"/>
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
        <xsl:with-param name="required"        select="$required"/>
        <xsl:with-param name="displayType"     select="$displayType"/>
        <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
        <xsl:with-param name="voAttribute"     select="$voAttribute"/>
        <xsl:with-param name="updateable"      select="$updateable"/>
        <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
      </xsl:call-template>

      <xsl:call-template name="maximumLengthAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      
      <xsl:call-template name="rowsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>

        <xsl:call-template name="readOnlyBinding">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>

    </messageFileUpload>
  </xsl:template>

  <xsl:template name="image">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>

    <xsl:variable name="width">
      <xsl:call-template name="getCustomProperty">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="propertyName" select="'DISPLAY_WIDTH'"/>
        <xsl:with-param name="defaultValue" select="''"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="height">
      <xsl:call-template name="getCustomProperty">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="propertyName" select="'DISPLAY_HEIGHT'"/>
        <xsl:with-param name="defaultValue" select="''"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:if test="$layoutStyle != 'table'">   
      <styledText>

        <xsl:call-template name="promptAttribute">
          <xsl:with-param name="entityAttribute" select="$entityAttribute" />
          <xsl:with-param name="uixAttributeName"  select="'textAndAccessKey'"/>
        </xsl:call-template>

     </styledText>   
    </xsl:if>

    <image>
      <xsl:if test="$width != ''">
        <xsl:attribute name="width"><xsl:value-of select="$width"/></xsl:attribute>
      </xsl:if>
      <xsl:if test="$height != ''">
        <xsl:attribute name="height"><xsl:value-of select="$height"/></xsl:attribute>
      </xsl:if>

      <xsl:choose>
        <xsl:when test="$layoutStyle = 'table'">
         <boundAttribute name="source">
           <concat>
             <xsl:text>DownloadFile.do?dataObjectSet=</xsl:text>
             <xsl:value-of select="$dataObjectName"/>
             <xsl:text>Set&amp;attribute=</xsl:text>
             <xsl:value-of select="$attributeName"/>
             <xsl:text>&amp;index=</xsl:text>
             <dataObject select="rownum"/>
           </concat>
         </boundAttribute>
          <boundAttribute name="rendered">
            <comparison type="notEquals">
              <defaulting>
                <dataObject select="{$attributeName}"/>
                  <fixed javaType="boolean">true</fixed>
              </defaulting>
              <fixed javaType="boolean">true</fixed>
            </comparison>
         </boundAttribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="source">
            <xsl:value-of select="concat('DownloadFile.do?dataObject=',$dataObjectName,'&amp;attribute=',$attributeName)"/>
         </xsl:attribute>
          <boundAttribute name="rendered">
            <comparison type="notEquals">
              <defaulting>
                <dataObject source="{concat('jheadstart:',$dataObjectName)}" select="{$attributeName}"/>
                <fixed javaType="boolean">true</fixed>
              </defaulting>
              <fixed javaType="boolean">true</fixed>
            </comparison>
         </boundAttribute>
        </xsl:otherwise>
      </xsl:choose>

    </image>
  </xsl:template>

  <xsl:template name="fileDownload">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>

    <!-- add dummy styled text so we get proper aligining in labeldFieldLayout -->
    <xsl:if test="$layoutStyle != 'table'">   
      <styledText text=""/>
    </xsl:if>
    
    <link>
      <xsl:call-template name="promptAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="uixAttributeName" select="'textAndAccessKey'"/>
      </xsl:call-template>

      <xsl:choose>
        <xsl:when test="$layoutStyle = 'table'">
         <boundAttribute name="destination">
           <concat>
             <xsl:text>DownloadFile.do?dataObjectSet=</xsl:text>
             <xsl:value-of select="$dataObjectName"/>
             <xsl:text>Set&amp;attribute=</xsl:text>
             <xsl:value-of select="$attributeName"/>
             <xsl:text>&amp;attachment=</xsl:text>
             <xsl:value-of select="$attributeName"/>
             <xsl:text>&amp;index=</xsl:text>
             <dataObject select="rownum"/>
           </concat>
         </boundAttribute>
          <boundAttribute name="rendered">
            <comparison type="notEquals">
              <defaulting>
                <dataObject select="{$attributeName}"/>
                  <fixed javaType="boolean">true</fixed>
              </defaulting>
              <fixed javaType="boolean">true</fixed>
            </comparison>
         </boundAttribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="destination">
            <xsl:value-of select="concat('DownloadFile.do?dataObject=',$dataObjectName,'&amp;attribute=',$attributeName,'&amp;attachment=',$attributeName)"/>
         </xsl:attribute>
          <boundAttribute name="rendered">
            <comparison type="notEquals">
              <defaulting>
                <dataObject source="{concat('jheadstart:',$dataObjectName)}" select="{$attributeName}"/>
                <fixed javaType="boolean">true</fixed>
              </defaulting>
              <fixed javaType="boolean">true</fixed>
            </comparison>
         </boundAttribute>
        </xsl:otherwise>
      </xsl:choose>

    </link>
  </xsl:template>

  <xsl:template name="numberField">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>

    <messageTextInput>
      <xsl:call-template name="fieldProps">
        <xsl:with-param name="attributeName"   select="$attributeName"/>
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
        <xsl:with-param name="isFindElement"   select="$isFindElement"/>
        <xsl:with-param name="required"        select="$required"/>
        <xsl:with-param name="displayType"     select="$displayType"/>
        <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
        <xsl:with-param name="operator"        select="$operator"/>
        <xsl:with-param name="voAttribute"     select="$voAttribute"/>
        <xsl:with-param name="updateable"      select="$updateable"/>
        <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
      </xsl:call-template>
      <xsl:call-template name="maximumLengthAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      <xsl:call-template name="columnsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      <xsl:call-template name="rowsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      <xsl:if test="$isFindElement='false'">
        <xsl:call-template name="readOnlyBinding">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:if>
      <onBlurValidater><decimal/></onBlurValidater>
    </messageTextInput>

  </xsl:template>




  <xsl:template name="secret">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>

    <messageTextInput secret="true">
      <xsl:call-template name="fieldProps">
        <xsl:with-param name="attributeName"   select="$attributeName"/>
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
        <xsl:with-param name="isFindElement"   select="$isFindElement"/>
        <xsl:with-param name="required"        select="$required"/>
        <xsl:with-param name="displayType"     select="$displayType"/>
        <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
        <xsl:with-param name="operator"        select="$operator"/>
        <xsl:with-param name="voAttribute"     select="$voAttribute"/>
        <xsl:with-param name="updateable"      select="$updateable"/>
        <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
      </xsl:call-template>
      <xsl:call-template name="maximumLengthAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      <xsl:call-template name="columnsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      <xsl:call-template name="rowsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      <xsl:if test="$isFindElement='false'">

        <xsl:call-template name="readOnlyBinding">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:if>  
    </messageTextInput>
  </xsl:template>


<!-- This dummy text element is required to get proper escaping of the
        quotes in the editor onClick command. Very weird! -->
<!--   <xsl:text disable-output-escaping="yes">
   </xsl:text>
-->

  <xsl:template name="editor">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="tableDataSet"/>
    <xsl:param name="dataObjectName"/>

    <messageTextInput>
      <xsl:call-template name="fieldProps">
        <xsl:with-param name="attributeName"   select="$attributeName"/>
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
        <xsl:with-param name="isFindElement"   select="$isFindElement"/>
        <xsl:with-param name="required"        select="$required"/>
        <xsl:with-param name="displayType"     select="$displayType"/>
        <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
        <xsl:with-param name="operator"        select="$operator"/>
        <xsl:with-param name="voAttribute"     select="$voAttribute"/>
        <xsl:with-param name="updateable"      select="$updateable"/>
        <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
      </xsl:call-template>
      <xsl:call-template name="maximumLengthAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      <xsl:call-template name="columnsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      <xsl:call-template name="rowsAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>

      <!-- At this point, all simple attributes (i.e. data:attr) attributes
           are added. Now we add the readOnly binding which can be simple
           but in some cases is a boundAttribute child. After this template
           call, only boundAttribute child attributes may be added
      -->
      <xsl:if test="$isFindElement='false'">
        <xsl:call-template name="readOnlyBinding">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
          <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
        </xsl:call-template>
      </xsl:if>
      
      <xsl:variable name="maxLength">
        <xsl:call-template name="getCustomProperty">
          <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
          <xsl:with-param name="propertyName" select="'MAXIMUM_LENGTH'"/>
          <xsl:with-param name="defaultValue" select="$entityAttribute/@Precision"/>
        </xsl:call-template>
      </xsl:variable>  

      <xsl:call-template name="editorImage">
        <xsl:with-param name="layoutStyle" select="$layoutStyle"/>
        <xsl:with-param name="tableDataSet" select="$tableDataSet"/>
        <xsl:with-param name="attributeName" select="$attributeName"/>
        <xsl:with-param name="maxLength" select="$maxLength"/>
      </xsl:call-template>

    </messageTextInput>
  </xsl:template>



  <!-- due to a bug in cabo we cannot set required to uiOnly for
       a date field in a table. This causes the data validation to fire
       rasing an error when the field is empty.
       Work around is to display the required gif ourselves -->
  <xsl:template name="dateField">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>
        <messageDateField>
          <xsl:call-template name="fieldProps">
            <xsl:with-param name="attributeName"   select="$attributeName"/>
            <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
            <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
            <xsl:with-param name="isFindElement"   select="$isFindElement"/>
            <xsl:with-param name="required"        select="$required"/>
            <xsl:with-param name="displayType"     select="$displayType"/>
            <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
            <xsl:with-param name="operator"        select="$operator"/>
            <xsl:with-param name="voAttribute"     select="$voAttribute"/>
            <xsl:with-param name="updateable"      select="$updateable"/>
            <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
          </xsl:call-template>
          <xsl:call-template name="valueAttribute">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="layoutStyle"   select="$layoutStyle"/>
            <xsl:with-param name="isFindElement" select="$isFindElement"/>
            <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
          </xsl:call-template>
          <xsl:if test="$isFindElement='false'">
            <xsl:call-template name="readOnlyBinding">
              <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
              <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
              <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
            </xsl:call-template>
          </xsl:if>
          <onSubmitValidater>
             <date dateStyle="shortish">
                <xsl:call-template name="dataBoundAttribute">
                  <xsl:with-param name="name" select="'data:pattern'"/>
                  <xsl:with-param name="value" select="'datepattern@jheadstart:dateformat'"/>
                </xsl:call-template>

             </date>
          </onSubmitValidater>
        </messageDateField>
<!--
      </xsl:otherwise>
    </xsl:choose>
-->    
  </xsl:template>

  <!-- due to a bug in cabo we cannot set required to uiOnly for
       a date field in a table. This causes the data validation to fire
       rasing an error when the field is empty.
       Work around is to display the required gif ourselves -->
  <xsl:template name="datetimeField">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="dataObjectName"/>
        <messageDateField>
          <xsl:call-template name="fieldProps">
            <xsl:with-param name="attributeName"   select="$attributeName"/>
            <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
            <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
            <xsl:with-param name="isFindElement"   select="$isFindElement"/>
            <xsl:with-param name="required"        select="$required"/>
            <xsl:with-param name="displayType"     select="$displayType"/>
            <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
            <xsl:with-param name="operator"        select="$operator"/>
            <xsl:with-param name="voAttribute"     select="$voAttribute"/>
            <xsl:with-param name="updateable"      select="$updateable"/>
            <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
          </xsl:call-template>
          <xsl:attribute name="isDateTimeField">true</xsl:attribute>
          <xsl:call-template name="valueAttribute">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="layoutStyle"   select="$layoutStyle"/>
            <xsl:with-param name="isFindElement" select="$isFindElement"/>
            <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
          </xsl:call-template>
          <xsl:if test="$isFindElement='false'">
            <xsl:call-template name="readOnlyBinding">
              <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
              <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
              <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
            </xsl:call-template>
          </xsl:if>
          <onSubmitValidater>
             <date dateStyle="shortish">
                <xsl:call-template name="dataBoundAttribute">
                  <xsl:with-param name="name" select="'data:pattern'"/>
                  <xsl:with-param name="value" select="'datetimepattern@jheadstart:dateformat'"/>
                </xsl:call-template>

             </date>
          </onSubmitValidater>
        </messageDateField>
<!--
      </xsl:otherwise>
    </xsl:choose>
-->    
  </xsl:template>

  <xsl:template name="choice">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="lookupChoiceValues"/>
    <xsl:param name="domain"/>
    <xsl:param name="dataObjectName"/>
    <xsl:param name="listValidator"/>

    <messageChoice>
      <xsl:call-template name="fieldProps">
        <xsl:with-param name="attributeName"   select="$attributeName"/>
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
        <xsl:with-param name="isFindElement"   select="$isFindElement"/>
        <xsl:with-param name="required"        select="$required"/>
        <xsl:with-param name="displayType"     select="$displayType"/>
        <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
        <xsl:with-param name="operator"        select="$operator"/>
        <xsl:with-param name="voAttribute"     select="$voAttribute"/>
        <xsl:with-param name="updateable"      select="$updateable"/>
        <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
      </xsl:call-template>
      <xsl:call-template name="option">
        <xsl:with-param name="entityAttribute"    select="$entityAttribute"/>
        <xsl:with-param name="layoutStyle"        select="$layoutStyle"/>
        <xsl:with-param name="attributeName"      select="$attributeName"/>
        <xsl:with-param name="lookupChoiceValues" select="$lookupChoiceValues"/>
        <xsl:with-param name="domain"             select="$domain"/>
        <xsl:with-param name="isFindElement"      select="$isFindElement"/>
        <xsl:with-param name="dataObjectName"       select="$dataObjectName"/>
        <xsl:with-param name="listValidator"   select="$listValidator"/>
      </xsl:call-template>
    </messageChoice>
  </xsl:template>

  <xsl:template name="list">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="required"/>
    <xsl:param name="displayType"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="operator"/>
    <xsl:param name="voAttribute"/>
    <xsl:param name="updateable"/>
    <xsl:param name="lookupChoiceValues"/>
    <xsl:param name="domain"/>
    <xsl:param name="dataObjectName"/>
    <xsl:param name="listValidator"/>

    <messageList>
      <xsl:call-template name="fieldProps">
        <xsl:with-param name="attributeName"   select="$attributeName"/>
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="elementPrompt"   select="$elementPrompt"/>
        <xsl:with-param name="isFindElement"   select="$isFindElement"/>
        <xsl:with-param name="required"        select="$required"/>
        <xsl:with-param name="displayType"     select="$displayType"/>
        <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
        <xsl:with-param name="operator"        select="$operator"/>
        <xsl:with-param name="voAttribute"     select="$voAttribute"/>
        <xsl:with-param name="updateable"      select="$updateable"/>
        <xsl:with-param name="dataObjectName"    select="$dataObjectName"/>
      </xsl:call-template>
      <xsl:call-template name="sizeAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
      </xsl:call-template>
      <xsl:call-template name="option">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="layoutStyle"     select="$layoutStyle"/>
        <xsl:with-param name="attributeName"   select="$attributeName"/>
        <xsl:with-param name="lookupChoiceValues" select="$lookupChoiceValues"/>
        <xsl:with-param name="domain" select="$domain"/>
        <xsl:with-param name="isFindElement"   select="$isFindElement"/>
        <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
        <xsl:with-param name="listValidator"   select="$listValidator"/>
      </xsl:call-template>
    </messageList>
  </xsl:template>

  <!-- styled text element -->
  <xsl:template name="styledText">
    <xsl:param name="attributeName"/>
    <xsl:param name="entityAttribute"/>
    <xsl:param name="elementPrompt"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="dataObjectName"/>

    <messageStyledText styleClass="OraFieldText">
      <xsl:call-template name="promptAttribute">
        <xsl:with-param name="entityAttribute" select="$entityAttribute"/>
        <xsl:with-param name="prompt" select="$elementPrompt"/>
      </xsl:call-template>
      <xsl:call-template name="textAttribute">
        <xsl:with-param name="attributeName" select="$attributeName"/>
        <xsl:with-param name="layoutStyle"   select="$layoutStyle"/>
        <xsl:with-param name="isFindElement" select="$isFindElement"/>
        <xsl:with-param name="dataObjectName"  select="$dataObjectName"/>
      </xsl:call-template>
    </messageStyledText>
  </xsl:template>

  <!-- template to set page header objects: images, global buttons, etc. -->
  <xsl:template name="pageHeader">

    <!-- fetch names of page header object uix files -->
    <xsl:variable name="globalButtons">
      <xsl:value-of select="/Service/@globalButtonsFile"/>
    </xsl:variable>
    <xsl:variable name="corpBranding">
      <xsl:value-of select="/Service/@corpBrandingFile"/>
    </xsl:variable>
    <xsl:variable name="prodBranding">
      <xsl:value-of select="/Service/@prodBrandingFile"/>
    </xsl:variable>
    <xsl:variable name="copyright">
      <xsl:value-of select="/Service/@copyrightFile"/>
    </xsl:variable>
    <xsl:variable name="tabBar">
      <xsl:value-of select="/Service/@tabBarFile"/>
    </xsl:variable>

    <xsl:if test="$globalButtons!=''">
      <globalButtons>
        <include>
          <xsl:variable name="attributeName" select="'data:node'"/>
          <xsl:variable name="attributeValue">
            <xsl:value-of select="concat($globalButtons,'@jheadstart:uixIncludes')"/>
          </xsl:variable>
          <xsl:call-template name="boundAttribute">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="attributeValue" select="$attributeValue"/>
          </xsl:call-template>
        </include>
      </globalButtons>
    </xsl:if>
    <xsl:if test="$corpBranding!=''">
      <corporateBranding>
        <include>
          <xsl:variable name="attributeName" select="'data:node'"/>
          <xsl:variable name="attributeValue">
            <xsl:value-of select="concat($corpBranding,'@jheadstart:uixIncludes')"/>
          </xsl:variable>
          <xsl:call-template name="boundAttribute">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="attributeValue" select="$attributeValue"/>
          </xsl:call-template>
        </include>
      </corporateBranding>
    </xsl:if>
    <xsl:if test="$prodBranding!=''">
      <productBranding>
        <include>
          <xsl:variable name="attributeName" select="'data:node'"/>
          <xsl:variable name="attributeValue">
            <xsl:value-of select="concat($prodBranding,'@jheadstart:uixIncludes')"/>
          </xsl:variable>
          <xsl:call-template name="boundAttribute">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="attributeValue" select="$attributeValue"/>
          </xsl:call-template>
        </include>
      </productBranding>
    </xsl:if>
    <xsl:if test="$copyright!=''">
      <copyright>
        <include>
          <xsl:variable name="attributeName" select="'data:node'"/>
          <xsl:variable name="attributeValue">
            <xsl:value-of select="concat($copyright,'@jheadstart:uixIncludes')"/>
          </xsl:variable>
          <xsl:call-template name="boundAttribute">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="attributeValue" select="$attributeValue"/>
          </xsl:call-template>
        </include>
      </copyright>
    </xsl:if>
    <xsl:if test="$tabBar!=''">
      <tabs>
        <include>
           <xsl:variable name="attributeName" select="'data:node'"/>
          <xsl:variable name="attributeValue">
            <xsl:value-of select="concat($tabBar,'@jheadstart:uixIncludes')"/>
          </xsl:variable>
          <xsl:call-template name="boundAttribute">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="attributeValue" select="$attributeValue"/>
          </xsl:call-template>
        </include>
      </tabs>
    </xsl:if>
    <!-- add global header if not null -->
    <xsl:choose>
      <xsl:when test="$globalHeader!=''">
        <pageHeader>
          <include>
            <xsl:variable name="attributeName" select="'data:node'"/>
            <xsl:variable name="attributeValue">
              <xsl:value-of select="concat($globalHeader,'@jheadstart:uixIncludes')"/>
            </xsl:variable>
          <xsl:call-template name="boundAttribute">
            <xsl:with-param name="attributeName" select="$attributeName"/>
            <xsl:with-param name="attributeValue" select="$attributeValue"/>
          </xsl:call-template>
          </include>
        </pageHeader>
      </xsl:when>
    </xsl:choose>

  </xsl:template> <!-- end page header template -->

  <!-- bind value of form element
       This binding depends on wehther the form element is within a table
       or not. Outside a table, we have to include the namespace and
       dataobject in the binding. Within a table, this is inherited from
       the table data set.
  -->
  <xsl:template name="valueBinding">
    <xsl:param name="attributeName"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="isFindElement"/>
    <xsl:param name="dataObjectName"/>

    <xsl:choose>
      <xsl:when test="$layoutStyle='table'">
        <xsl:value-of select="$attributeName"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:choose>
         <xsl:when test="$isFindElement='true'">
            <xsl:value-of select="concat($attributeName,'@jheadstart:sessionData')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat($attributeName,'@jheadstart:',$dataObjectName)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- Set value of readOnly binding
       This value depends on wehther the form element is within a table
       or not. Outside a table, we have to include the namespace and
       dataobject in the binding. Within a table, this is inherited from
       the table data set.
   -->

  <xsl:template name="readOnlyBinding">
    <xsl:param name="entityAttribute"/>
    <xsl:param name="layoutStyle"/>
    <xsl:param name="dataObjectName"/>
    <xsl:param name="propertyName" select="'readOnly'"/>

    <xsl:variable name="voAttUpdateableMode" select="@IsUpdateable"/>
    <xsl:variable name="eoAttUpdateableMode" select="$entityAttribute/@IsUpdateable"/>
    <!--
    If the updatable property of the vo attribute gives more restrictions to the
    end user than in the EO, then this should be taken into consideration. 
    However, the end user can never get less restrictions than give through the EO.
    -->
    <xsl:variable name="fieldUpdateableMode">
      <xsl:choose>
        <xsl:when test="$voAttUpdateableMode='false' or $eoAttUpdateableMode='false'">
          <xsl:value-of select="'false'"/>
        </xsl:when>
        <xsl:when test="$voAttUpdateableMode='while_insert' or $eoAttUpdateableMode='while_insert'"> 
          <xsl:value-of select="'while_insert'"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$voAttUpdateableMode"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$layoutStyle='table'">
        <xsl:choose>
          <!-- S. Davelaar 29-07-03: added check for false, which means we do 
               not have to data-bind the readOnly property -->               
          <xsl:when test="$fieldUpdateableMode='false'">
            <xsl:attribute name="readOnly">true</xsl:attribute>
          </xsl:when>
          <xsl:when test="$fieldUpdateableMode='while_insert'">
             <boundAttribute name="{$propertyName}">
               <or>
                 <dataObject select="isUpdating"/>
                 <dataObject select="isViewing"/>
               </or>
             </boundAttribute>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="dataBoundAttribute">
              <xsl:with-param name="name" select="concat('data:',$propertyName)"/>
              <xsl:with-param name="value" select="'isViewing'"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>  
      </xsl:when>
      <xsl:otherwise>
        <!-- Geen table layout -->
        <xsl:choose>
          <!-- S. Davelaar 29-07-03: added check for false, which means we do 
               not have to data-bind the readOnly property -->               
          <xsl:when test="$fieldUpdateableMode='false'">
            <xsl:attribute name="readOnly">true</xsl:attribute>
          </xsl:when>
          <xsl:when test="$fieldUpdateableMode='while_insert'">
              <xsl:call-template name="dataBoundAttribute">
                <xsl:with-param name="name" select="concat('data:',$propertyName)"/>
                <xsl:with-param name="value">
                  <xsl:value-of select="concat('isNotInserting@jheadstart:',$dataObjectName)"/>
                </xsl:with-param>
              </xsl:call-template>
          </xsl:when> 
          <xsl:otherwise>
            <xsl:call-template name="dataBoundAttribute">
              <xsl:with-param name="name" select="concat('data:',$propertyName)"/>
              <xsl:with-param name="value" select="concat('isViewing@jheadstart:',$dataObjectName)"/>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>  
      </xsl:otherwise>
    </xsl:choose>  
</xsl:template>

<!--
  <xsl:template name="insertOnlyInTable">
    <xsl:param name="entityAttribute"/>
    <xsl:param name="layoutStyle"/>

    <xsl:variable name="isUpdateable" select="$entityAttribute/@IsUpdateable"/>
    <xsl:if test="$isUpdateable='while_insert' and $layoutStyle='table'">
       <boundAttribute name="readOnly">
         <or>
           <dataObject select="isUpdating"/>
           <dataObject source="jheadstart:staticData" select="isViewing"/>
         </or>
       </boundAttribute>
     </xsl:if>
  </xsl:template>


  <xsl:template name="routerForm">
    <xsl:text disable-output-escaping="yes">
    </xsl:text>
    <!- reset of specific fields still needed despite reset
         all form values with onLoad on body ->
    <script>
      <xsl:attribute name="text">
        <xsl:text>reset=['target','targetProcess'];form='router';</xsl:text>
      </xsl:attribute>
    </script>
    <form name="router" method="POST">
      <contents>
        <formValue name="target"/>
        <formValue name="targetProcess"/>
      </contents>
    </form>
  </xsl:template>
-->

  <xsl:template name="getEoFileName">
    <xsl:param name="voDoc"/>
    <xsl:variable name="entityUsageName" select="@EntityUsage"/>
    <xsl:choose>
      <xsl:when test="$entityUsageName!=''">
        <xsl:variable name="entityUsage" select="$voDoc/ViewObject/EntityUsage[@Name=$entityUsageName]/@Entity"/>
        <xsl:variable name="entityUsagePath" select="translate($entityUsage,'.','/')"/>
        <xsl:value-of select="utils:getURL(concat($sourceDir,$entityUsagePath,'.xml'))"/>
      </xsl:when> 
      <xsl:otherwise>
        <xsl:value-of select="$domainfile"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="getDisplayTitle">
<!-- THIS COULD BE BETTER?
    <xsl:variable name="selectText">
      <xsl:choose>
        <xsl:when test="@displayTitle!=''">
          <xsl:value-of select="@displayTitle"/>
        </xsl:when>
        <xsl:when test="@tabName!=''">
          <xsl:value-of select="@tabName"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@dataObject"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
-->  
    <xsl:variable name="tabName">
      <xsl:choose>
        <xsl:when test="@tabName!=''">
          <xsl:value-of select="@tabName"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@dataObject"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="displayTitle">
      <xsl:choose>
        <xsl:when test="@displayTitle!=''">
          <xsl:value-of select="@displayTitle"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$tabName"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:value-of select="$displayTitle"/>
  </xsl:template>

  <xsl:template name="nlsTextAttribute">
    <xsl:param name="attributeName"/>
    <xsl:param name="key"/>
    <xsl:param name="value" select="$key"/>
    <xsl:param name="addToNlsBundle" select="'false'"/>

      <xsl:choose>

        <xsl:when test="$addToNlsBundle='false'">
           <!-- entry already added to bundle through call to getNlsKey
                this also means, we must always generate as nls-enabled,
                regardless of nls setting. (This is the case for headings
                and button laberls, etc. -->
          <xsl:call-template name="dataBoundAttribute">
            <xsl:with-param name="name">
              <xsl:value-of select="concat('data:',$attributeName)"/>
            </xsl:with-param>
            <xsl:with-param name="value">
              <xsl:value-of select="concat(string:toUpperCase($key),'@jheadstart:nls')"/>
            </xsl:with-param>
          </xsl:call-template>

        </xsl:when>

        <!-- add entry to bundle, we use the key as value  -->
        <xsl:when test="$nls='true' and $addToNlsBundle='true'">
            <!-- add key value pair in XSLTUtils, so we can generate
                 resource bundle from it -->
            <xsl:variable name="dummy">
              <xsl:value-of select="utils:addNlsText(string:toUpperCase($key),$value)"/>
            </xsl:variable>
            <xsl:call-template name="dataBoundAttribute">
              <xsl:with-param name="name">
                <xsl:value-of select="concat('data:',$attributeName)"/>
              </xsl:with-param>
              <xsl:with-param name="value">
                <xsl:value-of select="concat(string:toUpperCase($key),'@jheadstart:nls')"/>
              </xsl:with-param>
            </xsl:call-template>
        </xsl:when>
        <!-- dirty fix for bug 3083520 INCORRECT RANGE QUERY PROMPS IN FIND
             PAGES (UIX). Problem is that we have called getNlsKey to get
             search from/to prompts. getNlsKey now already returns a
             message bundle selectKey, regardless of nls setting. However, when
             generating form elements, the prompts are only nls-enabled when
             nls is set to true. The serach from/.to must ALWAYS be nls enabled
             therefore, we explicitly check for these prompts here. More
             elegant is to pass a "forceNls" parameter but this must be cascaded
             to a zillion of templates ... -->        
        <xsl:when test="string:startsWith($key,'SEARCH_FROM') ='true'
                        or string:startsWith($key,'SEARCH_TO') ='true'">
            <!-- add key value pair in XSLTUtils, so we can generate
                 resource bundle from it -->
            <xsl:call-template name="dataBoundAttribute">
              <xsl:with-param name="name">
                <xsl:value-of select="concat('data:',$attributeName)"/>
              </xsl:with-param>
              <xsl:with-param name="value">
                <xsl:value-of select="concat(string:toUpperCase($key),'@jheadstart:nls')"/>
              </xsl:with-param>
            </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="{$attributeName}">
            <xsl:value-of select="$value"/>
          </xsl:attribute>
        </xsl:otherwise>
       </xsl:choose> 

  </xsl:template>

  <xsl:template name="checkMultipartAttributes">
     <!-- first check for multi part aiitrbiutes of current group, then
          check for multipart attrbutes in child groups displayed on the
          same page -->
      <xsl:variable name="voFile" select="utils:getURL(concat($bc4jDir,@viewObject,'.xml'))"/>
      <xsl:call-template name="checkGroupMultipartAttributes">
        <xsl:with-param name="voFile"  select="$voFile"/>
      </xsl:call-template>

    <!-- loop over same page child groups -->
    <xsl:for-each select="Group[@samePage='true']">
      <xsl:variable name="voFile" select="utils:getURL(concat($bc4jDir,@viewObject,'.xml'))"/>
      <xsl:call-template name="checkGroupMultipartAttributes">
        <xsl:with-param name="voFile"  select="$voFile"/>
      </xsl:call-template>
    </xsl:for-each>
  
  </xsl:template>

  <xsl:template name="checkGroupMultipartAttributes">
    <xsl:param name="voFile"/>

      <xsl:variable name="voDoc" select="document($voFile)"/>
      <xsl:for-each select="$voDoc/ViewObject/ViewAttribute[not(@IsUpdateable='false')]">

        <xsl:variable name="eoFile">
          <xsl:call-template name="getEoFileName">
            <xsl:with-param name="voDoc" select="$voDoc"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="entityUsage" select="@EntityUsage"/>
        <xsl:variable name="eoAttributeName" select="@EntityAttrName"/>
        <xsl:variable name="eoDoc" select="document($eoFile)"/>
        <xsl:variable name="entityAttributeNode"  select="$eoDoc/Entity/Attribute[$entityUsage!='' and @Name=$eoAttributeName]|current()[$entityUsage='']"/>

        <xsl:variable name="display">
          <xsl:call-template name="getCustomProperty">
            <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
            <xsl:with-param name="propertyName" select="'DISPLAY'"/>
            <xsl:with-param name="defaultValue" select="'true'"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="displayType">
          <xsl:call-template name="getCustomProperty">
            <xsl:with-param name="entityAttribute" select="$entityAttributeNode"/>
            <xsl:with-param name="propertyName" select="'DISPLAY_TYPE'"/>
            <xsl:with-param name="defaultValue" select="''"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:if test="$display='true' and ($displayType='fileUpload' or $entityAttributeNode/@ColumnType='BLOB')">
           <xsl:value-of select="@Name"/>
        </xsl:if>

      </xsl:for-each>
  
  </xsl:template>

</xsl:stylesheet>

