/*******************************************************************************
Copyright (c) 2000 Oracle Corporation/2004 AMIS Services BV

File        : form.js

Author      : Maurits Blok

Notes       : Some functions can only used in UIX pages others only in JSP pages
              and some in both. This is indicated by a note, e.g. note: UIX & JSP

Open Issues : functions that are not called by any generated coded (directly or
              indirectly) should be refactored out and put into a specific reuse
              library.

$revision_history$
19-jul-2004   Sandra Muller
  1.54AMIS1.0 New function popupAndGo
26-apr-2004   Sandra Muller
  1.54        bugfix 3584358: pass pageTimeStamp=ignore in jspLovPopup
19-dec-2003   Jan Kettenis
  1.53        - Made loop in getFormElement more efficient
              - Made baseDescriptorItem an Array
              - Changed jspLovPopup() to pass multi descriptor item names and search arguments to LoV
              - Changed chooseLov() to multi descriptor item values back to calling form
28-nov-2003   Jan Kettenis
  1.52        Changed jspLovPopup() to take lovKeyAttributeName, lovDescriptorAttributeNames,
              searchItems as extra parameters. These parameters are also in the lovPopup.jsp.
              Also renamed LoV global variables.
17-nov-2003   Steven Davelaar
  1.51        Added calendarPath as argument to calendarPopup, this
              is needed because we must pass web context path to support
              struts sub modules
21-aug-2003   Aard-Jan van Kesteren
  1.50        Added documentation
21-aug-2003   Aard-Jan van Kesteren
  1.49        Added comments
14-aug-2003   Aard-Jan van Kesteren
  1.48        Added format functionality in calendarPopup
14-aug-2003   Aard-Jan van Kesteren
  1.47        Dropped formName in jspEditorPopup and calendarPopup
05-aug-2003   Aard-Jan van Kesteren
  1.46        Dropped var targetGroup + setter + the targetGroup in go3
01-aug-2003   Aard-Jan van Kesteren
  1.45        Added function getFormElement and getFormElementValue
01-aug-2003   Aard-Jan van Kesteren
  1.44
30-jul-2003   Aard-Jan van Kesteren
  1.43        - Changed go3 to set the value of form.targetGroup to (global variable) targetGroup
                in case of JSP page
              - Added functions submitJspForm(), closeWindows(), setTargetGroup(), setLovField(),
                setLovValue(), jspEditorPopup(), jspLovPopup(), chooseLovValue(), calendarPopup()
30-jul-2003   Jan Kettenis
  1.42        - Added constructTableItemName() and refactored isEmptyDisplayLine() and
                checkForEmptyRequiredRowItems() to cope for naming of table items in JSP pages.
              - Indicated which functions can be used in UIX or JSP pages.
24-jul-2003   Jan Kettenis
  1.41        Second try
              Changed hasChanges() to check for the existence of HasErrors.
24-jul-2003   Jan Kettenis
  1.39        - Added global variable viewType with default UIX
              - Added functions getItemName() and stripRownumFromName()
              - Changed checkForChanges() to use ignoreChangedFields
              - Changed resetValues() to ignore radio buttons radio groups, checkboxes and
                select-one items, as these kind of items cannot be reset by setting the
                value to "".
16-jan-2003   Jan Kettenis
  1.38        Refactored
              Introduced array requiredRowItems with item names including table name, as opposed to
              (now depricated requiredFields). As a result of this, refactored checkRequiredRowItems()
              as a router for newly created checkForEmptyRequiredRowItems() and
              checkForEmptyRequiredRowItems(). The latter function uses array requiredRowItems and
              function getScreenPrompt().
07-jan-2003   Peter Ebell
  1.37b       Added function getScreenPrompt().
02-jan-2003   Jan Kettenis
  1.37        - Put confirmChanges() as created by Steven Davelaar back in again (was not
                in version 1.35 and 1.36).
              - Documented various functions and made layout consistent
              - renamed isEmpty() -> isEmptyItem(), isEmptyDisplayField() -> isEmptyDisplayItem()
                userChanged() -> userHasChanged(), isEmptyLine() -> isEmptyDisplayLine(),
              - added parameter formName to checkRequiredRowItems(), isEmptyDisplayLine()
                createTableItemNamesArray() to allow for easier testing
              - Created getItemType() (to deal with radiogroups) and used it in hasChanged(),
                userHasChanged(), isEmptyDisplayItem(), checkRequiredRowItems(), isEmptyItem()
              - Added function isString() for proper 'exception handling' of stripSpaces(),
                isNumber() and replace().
              - Added 'exception handling' to hasChanged(), isEmptyDisplayItem, userHasChanged(),
                isEmptyItem(), stripSpaces(), isNumber() and replace()
              - Altered hasChanged() to deal with hidden items (otherwise hasChanged would almost
                exactly do what userHasChanged already did; if called for hidden items should be
                determined by calling function, not by hasChanged() itself). As a result of this,
                altered hasChanges() to call userHasChanged() instead of hasChanged().
              - Changed isEmptyDisplayItem() to call isEmptyItem() (to remove duplicate code).
              - Altered userHasChanged() to take an item as parameter and to call hasChanged() only
                for displayed items.
              - Changed checkRequiredRowItems() to call is emptyDisplayItem() and to deal with focus()
                for radiogroup.
              - Changed isEmptyItem() to ignore items with type (stand-alone) radio and checkbox,
                to handle radiogroups and to handle items with no value.
              - Changed isNumber() to return false for empty strings (like it already returned false
                for strings with only spaces)
              - Changed isEmptyDisplayLine() to call isEmptyItem() instead of isEmptyDisplayItem(),
23-dec-2002   Peter Ebell
  1.36        Fixed bug in hasChanged where radio groups and checkboxes were not
              detected as changed.
02-dec-2002   Peter Ebell
  1.35        - Rewrote checkForChanges to enable the end user to clear a row after trying
                to save it but running into an error (Jan Kettenis).
              - As a result of this, created funcions createTableItemNamesArray(),
                checkRequiredRowItems(),deleteRowChecked()  (Jan Kettenis).
              - Changed functions to use booleans instead of strings 'true' and 'false'
              - Prevent required items from being checked when the user already has
                indicated the row as to be deleted.
              - Changed debug method so that you can stop debugging by pressing 'Cancel'.
              - Changed isEmpty to not fail on undefined items (Jan Kettenis).
              - Fixed bug in checkSelectedRow that caused it not to reckognise the selected
                row when there is only one row.
17-jul-2002   Peter Ebell
  1.34        Removed string escaping on editorPopup; no longer necessary
              since we do not pass the string through a URL any more.
09-jul-2002   Peter Ebell
  1.33        - Added function go5 (based on go4) to navigate to a different URL
                (service). Difference with go4 is that you do not need to specify
                the name of the (router) form.
              - Fixed bug where value of editor field was no longer passed to the
                editor jsp. Also enlarged the editor window.
              - Added maxLength property to editor popup, so that you can no longer
                enter more characters in the editor than are allowed in the field
                itself.
08-jul-2002   Peter Ebell
  1.32        Fixed check for required fields in checkForChanges, because
              "LastName" and "ManagerLastName" would both be required if
              "LastName" was a required field.
12-jun-2002   Steven Davelaar
  1.31        Added function lovpagePopup
14-may-2002   Peter Ebell
  1.30        Moved all hardcoded messages to messages.js
14-may-2002   Peter Ebell
  1.29        Removed redundant debug messages
28-mrt-2002   Peter Ebell
  1.28        Adapted to JHeadstart 9i
28-sep-2001   Steven Davelaar
  1.27
27-sep-2001   Steven Davelaar
  1.26        Added parameter formName to treeInfo method
14-aug-2001   Sandra Muller
  1.25        fixed typo of previous change
13-aug-2001   Sandra Muller
  1.24        allow call to checkSelectedRow without checkFieldName
27-jul-2001   Peter Ebell
  1.23        Added function resetFormValues
26-jul-2001   Sandra Muller
  1.22        removed functions for converting display type, except changeDisplayPoplist
25-jul-2001   Sandra Muller
  1.21        added function isNumber (mainly written by Peter)
24-jul-2001   Peter Ebell
  1.20        - Changed function changeDisplayPoplist to take into account the global variable "ignoreChanges"
              - Added function checkSelectedRow
19-jul-2001   Peter Ebell
  1.19        Made a number of changes for code that did not perform as expected on MSIE5.5.
              - isEmptyItem now returns false for hidden fields because
                the end user cannot influence them anyway and on
                Netscape an empty hidden field that is empty
                returns two spaces to javascript.
              - element.focus errors on hidden fields in MSIE
              - Changed algorithm for determining the default
                selected entry in a poplist because if there
                is no "selected" attribute on any option
                Netscape makes the first option "defaultSelected",
                but MSIE does not.
12-jul-2001   Sandra Muller
  1.18        convertToDisplayType: round days to zero decimals and weeks/months/years to 1 decimal
11-jul-2001   Sandra Muller
  1.17        (together with Peter Ebell) added function changeDisplayPoplist
29-jun-2001   Peter Ebell
  1.16        Added functionality to skip required field checks for fields starting with '_'
27-jun-2001   Maurits Blok
  1.15        Added the method isEmptyItem(formElement). This methods checks if
              a form element has a value. Fix for issue 138.
21-jun-2001   Maurits Blok
  1.14        Added documentation.
07-jun-2001   Peter Ebell
  1.13        Maurits Blok added global variabele requiredFields and changed function
              checkForChanges to implement required field validation on multirecord forms.
28-may-2001   Peter Ebell
  1.12        Pass the Display Value to the lov JSP, so that it can use it
              as the default search criterium.
28-may-2001   Peter Ebell
  1.11
22-may-2001   Peter Ebell
  1.10        Removed debug alert statements accidentally left over from previous modifications.
22-may-2001   Peter Ebell
  1.9         Created global variable for message text for unsaved changes.
22-may-2001   Steven Davelaar
  1.8         Changed function HasChanges to include a check for hidden field HasErrors. This
              field is filled if there are error messages in the stack. If this is the case,
              the page is also thought to have outstanding changes, although the generic
              HadChanged code will not pick up on this.
01-may-2001   Steven Davelaar
  1.7         Enlarged editor popup window
26-apr-2001   Maurits Blok
  1.6         The text of the lov field is not added to the
              destination anymore.
              The lov field now returns two values:
              the display value and the value value.
25-apr-2001   Maurits Blok
  1.5         Added ignoreChangedFields. This is an array that contains form
              fields that are ignored while checking a page for changes.
              Added the method ignoreField.
20-apr-2001   Maurits Blok
  1.4         Added support for durations that should be displayed on the client
              in several ways, such as weeks, days, years, and months.
06-apr-2001   Maurits Blok
  1.3         Removed the link parameter from the functions
              popup, lovPopup and editorPopup.
23-mar-2001   Maurits Blok
  1.2         Changes launchWindow to popup.
              Added lovPopup and editorPopup.
15-mar-2001   Maurits Blok
  1.1         Added functions changeListSelection, back, next, go.
              Added javadoc.
10-feb-2001   Maurits Blok
  1.0         Initial creation
******************************************************************************/
/*
$add_to_revision_history
$
*/


/* Debug procedure */
var debugMode = false;

/* View type. The default is UIX, in generated JSP pages it will be set to JSP */
var viewType="UIX";

/**
 * debug gives debug messages based on the setting of debugMode
 * the debugMode can be switched of during debugging
 */
function debug(text)
{
  if (debugMode)
  {
    debugMode = confirm(text);
  }
} // debug


// Global variables.
/**
 * form contains the name of the form that should be submitted.
 */
var form;

/**
 * field contains the name of the field that should get some value.
 */
var field;

/**
 * reset contains an array of fields that should be reset before a form
 * submission.
 */
var reset;

/**
 * serviceRoot is a constant that contains the package name of a controller.
 */
var serviceRoot="oracle.ipet.process.service.";

/**
 * if ignoreChanges has a value then the appropriate functions don't check
 * for changes.
 */
var ignoreChanges;

/**
 * ignoreChangedFields is an array that contains form
 * fields that are ignored while checking a page for changes.
 */
var ignoreChangedFields = new Array();

/**
 * requiredFields contains the names of the items that are required in a marlin table.
 * Required fields cannot be enforced by marlin validation if empty rows are used.
 * The fields are contained, without the table name, for example:
 *   ['fields', 'fields']
 * This array has been depricated and is replaced by the array requiredTableItems
 */
var requiredFields = new Array();

/**
 * requiredRowItems contains the names of the items that are required in a marlin table.
 * Required table items cannot be enforced by marlin validation if empty rows are used.
 * The items are contained, including the table name, for example:
 *   ['table0:item0','table0:item1','table1:item0']
 */
var requiredRowItems = new Array();

/**
 * itemScreenPrompts contains the prompts of all form items.
 */
var itemScreenPrompts = new Array();

/**
 * These are public variables that are used by CheckForChanges, and its helper functions
 */
var dataForm;


/**
 * JSP specific variables
 */
var lovWindow;
var calendarWindow;
var editorWindow;
// pointer to the key item in the calling form, used for LoV's
var _baseKeyItem;
// array with pointers to the descriptor items in the calling form, used for LoV's
var _baseDescriptorItems = new Array();
var editorValue;
var editorField;
var editorMaxlength;


/**
 * getItemName gets the 'technical' name of an item. In case of UIX table item names have
 * the format [tableName]:[itemName]:[rownum], in which case the rownum needs to be stripped
 * note: UIX & JSP
 * @param item: item to get the name for
 * @return: the name of the item
 */
function getItemName(item)
{
  var type;
  var itemName;
  var colon1;
  var colon2;

  debug("in getItemName");
  if (!item)
  {
    alert("undefined item >"+item+"< in getItemName");
    return null;
  }
  else
  {
    type = getItemType(item);
    if (type == "radiogroup")
    {
      itemName = item[0].name;
    }
    else
    {
      itemName = item.name;
    }

    // only in case of UIX does the item name contain the rownum
    if (viewType == "UIX")
    {
      itemName = stripRownumFromName(itemName);
    }
  }
  return itemName;
} // getItemName


/**
 * constructTableItemName construct the HTML item name of an item in a table. In case of UIX
 * table item names have the format [tableName]:[itemName]:[rownum]. In case of JSP,
 * table item names have the format [itemName]:[[rownum]]
 * note: UIX & JSP
 * @param itemName: the item name including the UIX table name, when applicable
 * @param rownum: the index of the item in the table
 * @return: the name of the item as it will be rendered in the HTML table
 */
function constructTableItemName(itemName, rownum)
{
  var tableItemName;

  if (!itemName)
  {
    alert("undefined itemName in constructTableItemName");
    return null;
  }
  if (!rownum)
  {
    alert("undefined rownum in constructTableItemName");
    return null;
  }

  if (viewType == "UIX")
  {
    tableItemName = itemName+":"+rownum;
  }
  else
  {
    tableItemName = itemName+"["+rownum+"]";
  }
  return tableItemName;
} // constructTableItemName


/**
 * stripTableItemName strips the rowNum from items that have the format
 * [tableName]:[itemName]:[rownum]
 * note: UIX
 * @param itemName: the itemName before stripping the rownum
 * @return: the 'short' name of the item
 */
function stripRownumFromName(itemName)
{
  colon1 = itemName.indexOf(":");
  colon2 = itemName.lastIndexOf(":");
  if (colon1 < colon2)
  { // strip the rownum from the itemName
    itemName = itemName.substring(0,colon2);
  }

  return itemName;
} // stripRownumFromTableItemName


/**
 * hasChanged checks if the value of item has changed. Thereby also hidden items are checked.
 * note: UIX & JSP
 * @param item: the item to check
 * @return: true when the item has changed, false otherwise
 */
function hasChanged(item)
{
  // debug("In hasChanged for item: "+item);
  var changed = false;

  if (!item)
  {
    alert("undefined item >"+item+"< in hasChanged");
    changed = null;
  }
  else
  {
    // getItemType will discover when item is an invalid parameter
    var type = getItemType(item);
    var itemName = getItemName(item);

    if (!ignoreField(itemName))
    {
      if (  (  (type == "password" || type == "text" || type == "textarea")
            && item.defaultValue != item.value
            )
         || (  (type == "checkbox" || type == "radio")
            && item.defaultChecked != item.checked
            )
         )
      {
        changed = true;
      }
      else if (type == "radiogroup")
      {
        for (var i = 0; i < item.length && !changed; i++)
        {
          changed = (item[i].defaultChecked != item[i].checked);
        }
      }
      else if (type == "select-one" || type == "select-multiple")
      {
        var options         = item.options;
        var defaultSelected = 0;
        for (var j = 0; j < options.length; j++)
        {
          if (options[j].defaultSelected)
          {
            defaultSelected = j;
          }
        }

        if (  (defaultSelected != options.selectedIndex)
           && (options.selectedIndex != -1)
           )
        {
          changed = true;
        }
      }
      else if (type == "hidden")
      {
        changed = (item.value != item.defaultValue);
      }
    }
  }

  return changed;
} // hasChanged


/**
 * isEmptyDisplayItem checks if the item is "empty" from the end user perspective.
 * note: UIX & JSP
 * @param item: the item to check
 * @return: true when it is an empty display item, false otherwise
 */
function isEmptyDisplayItem(item)
{
  var empty = true;
  var type;

  if (!item)
  {
    alert("undefined item >"+item+"< isEmptyDisplayItem");
    return null;
  }
  else
  {
    // getItemType will discover when item is an invalid parameter
    var type  = getItemType(item);
    if (type != "hidden")
    {
      empty = isEmptyItem(item);
    }
    else // hidden items are never an empty display item
    {
      empty = false;
    }
    debug("item "+item.name+": empty is " + empty);
    return empty;
  }
} // isEmptyDisplayItem


/**
 * Get the value of the "hasChanges" item of a specific row in the table
 * Helper function for checkForChanges.
 * note: UIX
 * @param tableName: the name of the table
 * @param rownum: the number of the row
 * @return: the value of hasChanges
 */
function getChangedStatus(tableName, rownum)
{
  return dataForm[tableName+":hasChanges:"+rownum].value;
} // getChangedStatus


/**
 * setChangedStatus sets the value of the "hasChanges" item of a specific row in the table
 * Helper function for checkForChanges.
 * note: UIX
 * @param tableName: the name of the table
 * @param rownum: the number of the row
 * @param value: the value to set
 */
function setChangedStatus(tableName, rownum, value)
{
  dataForm[tableName+":hasChanges:"+rownum].value = value;
} // setChangedStatus


/**
 * userHasChanged checks if an item has been changed by the user. Therefore only non-hidden
 * items are checked.
 * note: UIX & JSP
 * @param item: the item to check
 * @return: true when the value has changed, false otherwise
 */
function userHasChanged(item)
{
  if (!item)
  {
    alert("undefined item >"+item+"< in userHasChanged");
    return null;
  }
  else
  {
    var type = getItemType(item);
    // only displayed items can return true
    if (type != "hidden")
    {
      return hasChanged(item);
    }
    else
    {
      return false;
    }
  }
} // userHasChanged


/**
 * hasInsertStatus gets the value of the "inserting" field of a specific row in the table
 * Helper function for checkForChanges.
 * note: UIX
 * @param tableName: the name of the table
 * @param rownum: the number of the row
 * @return: true when inserting equals Y, false otherwise
 */
function hasInsertStatus(tableName, rownum)
{
  if (dataForm[tableName+":inserting:"+rownum])
  {
    debug("Row "+rownum+" inserting = "+dataForm[tableName+":inserting:"+rownum].value);
    return (dataForm[tableName+":inserting:"+rownum].value == "Y");
  }
  else
  {
    return false;
  }
} // hasInsertStatus


/**
 * isEmptyDisplayLine checks if all the visible items in a row are empty
 * Helper function for checkForChanges
 * note: UIX & JSP
 * @param formName: the name of the form
 * @param tableItems: array of table item names
 * @param rownum: the number of the row
 * @return: true when all visible items are empty, false otherwise
 */
function isEmptyDisplayLine(formName, tableItems, rownum)
{
  debug("in isEmptyDisplayLine");

  var form      = document.forms[formName];
  var emptyline = true;
  var type;
  var item;
  var tableItemName;

  for (var k = 0; (k < tableItems.length) && emptyline; k++)
  {
    // pass rownum as a string
    tableItemName = constructTableItemName(tableItems[k], rownum+"");
    item = form[tableItemName];
    // getItemType deals with undefined items
    type = getItemType(item);
    // ignore hidden items, checkboxes and radio buttons
    // checking for hidden & using isEmptyItem, because isEmptyDisplayItem
    // will return false for hidden items.
    if (  type != "hidden"
       && type != "checkbox"
       && type != "radio"
       && !isEmptyItem(item)
       )
    {
      emptyline = false;
    }
  }

  debug("Row "+ rownum+": empty line is "+ emptyline);
  return emptyline;
} // isEmptyDisplayLine


/**
 * createTableItemNamesArray creates an array of table item names
 * note: UIX
 * @param formName: the name of the form
 * @param tableName: the name of the table for which the array should be created
 * @return: an array of table item names ("table name:item name")
 */
function createTableItemNamesArray(formName, tableName)
{
  var form               = document.forms[formName];
  var formElementsCount  = form.elements.length / 1;
  var tempTableItemNames = new Array(formElementsCount);
  var itemFound          = new Array();
  var itemCount          = 0;

  var item;
  var itemName;
  var colon1;
  var colon2;

  // loop through all items in the form and add those items to tempTableItems
  // that belong to tableName
  for (var i = 0 ; i < formElementsCount ; i++)
  {
    item     = form.elements[i];
    itemName = new String(item.name);
    colon1   = itemName.indexOf(":");
    colon2   = itemName.lastIndexOf(":");
    // add item to the array when it is in table with tableName
    if (  colon1 > -1 && colon2 > -1 && colon1 != colon2
       && tableName == itemName.substring(0, colon1)
       )
    {
      // itemFound indicates if a specific item name already has been found
      if (!itemFound[itemName.substring(colon1+1,colon2)])
      {
        itemFound[itemName.substring(colon1+1,colon2)] = true;
        tempTableItemNames[itemCount] = itemName.substring(0,colon2);
        itemCount++;
      }
    }
  }

  // As there may be items in the form than do not belong to the table, the length
  // of tempTableItems may be longer than necessary. Therefore an array is created
  // with a length equal to the number of table items, to be returned.
  var tableItemNames = new Array(itemCount);
  for (var i = 0; i < itemCount; i++)
  {
    tableItemNames[i] = tempTableItemNames[i];
  }

  //  debugString = "Found the following items for table "+tableName+": \n\n";
  //  for (var i = 0 ; i < itemCount; i++) debugString +="- "+tableItemNames[i]+"\n";
  //  debug(debugString);

  return tableItemNames;

} // createTableItemNamesArray


/**
 * getScreenPrompt translates an item name to a prompt as displayed on the screen.
 * It is used for form layout and table layout, and also in samepage master-detail
 * pages so for items that are in a table, the table name needs to be added, e.g.:
 *   getScreenPrompt("Ename");
 *   getScreenPrompt("EmpDepSet:Ename");
 * note: UIX & JSP
 * @param itemName: the name of the item for which to get the prompt
 * @return: the prompt of the item if it could be found in itemScreenPrompts, the
 * name of the item otherwise
 */
function getScreenPrompt(itemName)
{
  var prompt;

  debug("in getScreenPrompt");
  if (!itemName)
  {
    alert("undefined itemName in getScreenPrompt");
    return null;
  }
  else
  {
    // For backward compatibility, check if itemScreenPrompts array exists.
    if (itemScreenPrompts)
    {
      for (var i = 0; i < itemScreenPrompts.length; i++)
      {
        if (itemScreenPrompts[i][0] == itemName)
        {
          debug("prompt for item "+itemName+" = "+itemScreenPrompts[i][1]);
          prompt = itemScreenPrompts[i][1];
        }
      }
    }
    if (!prompt) // itemScreenPrompts does not exists, or does not include itemName
    {
      prompt = itemName;
      // if itemName includes the table name remove the table name
      // in case of JSP we cannot strip the FormBean name
      var j  = prompt.lastIndexOf(':');
      if (j > -1)
      {
        prompt = itemName.substring(j+1);
      }
      debug("prompt for item is "+prompt+", because no prompt could be found");
    }
  }
  return prompt;
} // getScreenPrompt


/**
 * DEPRICATED FUNCTION, has been replaced by checkForEmptyRequiredRowItems as of
 * JHeadstart for JDeveloper 9.0.4
 * checkForEmptyRequiredRowFields checks for all required items in a row if they have
 * a value. An alert is given when a required item has no value.
 * @param formName: the name of the form
 * @param tableName: the table name
 * @param rownum: the index of the row to be checked
 * @return: true when all required items have a value, false otherwise
 */
function checkForEmptyRequiredRowFields(formName, rownum, tableName)
{
  var form = document.forms[formName];
  var formItem;
  var type;
  var formItemName;

  debug("In checkForEmptyRequiredRowFields");

  // only check when requiredFields have been specified
  if (requiredFields)
  {
    // loop through all the required items
    for (var i = 0; i < requiredFields.length; i++)
    {
      formItemName = requiredFields[i];
      formItem = form[tableName+":"+formItemName+":"+rownum];

      // non-updateble items are rendered as text, so it should be checked if the item
      // exists before checking the requiredness
      if (formItem)
      {
        debug("Checking for requiredness of item "+i+": "+tableName+":"+formItemName+":"+rownum+", formitem="+formItem);
        if (isEmptyDisplayItem(formItem))
        {
          // Show the message to fill in this field
          alert(getMessage("requiredFields")+formItemName);
          type = getItemType(formItem);
          // Give the offending item the focus
          if (type == "radiogroup")
          {
            formItem[0].focus();
          }
          else
          {
            formItem.focus();
          }
          // return false as soon as an empty required item has been found
          debug("returning false in checkForEmptyRequiredRowFields because of item "+tableName+":"+requiredFields[i]+":"+rownum);
          return false;
        }
      }
    }
    // return true when no empty required items could be found
    return true;
  }
  else
  {
    // return true when there are no requiredFields
    return true;
  }
} // checkForEmptyRequiredRowFields


/**
 * checkForEmptyRequiredRowItems checks for all required items in a row if they have
 * a value. An alert is given when a required item has no value.
 * note: UIX & JSP
 * @param formName: the name of the form
 * @param tableName: the table name
 * @param rownum: the index of the row to be checked
 * @return: true when all required items have a value, false otherwise
 */
function checkForEmptyRequiredRowItems(formName, rownum, tableName)
{
  // TO DO: should refactor this function as to interpret tableName as
  // FormBean name in case of JSP
  var form = document.forms[formName];
  var formItem;
  var type;
  var formItemName;
  var formTableItemName;

  debug("In checkForEmptyRequiredRowItems");

  // only check when requiredRowItems have been specified
  if (requiredRowItems)
  {
    // loop through all the required items
    for (var i = 0; i < requiredRowItems.length; i++)
    {
      debug("checking item "+i);
      // only check the items that have a name that includes the tableName
      // when no tableName has been passed it does not concern a UIX page
      if (  (  tableName
            && requiredRowItems[i].indexOf(tableName) > -1
            )
         || !tableName
         )
      {
        formItemName = requiredRowItems[i];
        // convert rownum to string
        formTableItemName = constructTableItemName(formItemName, rownum+"");
        formItem = form[formTableItemName];

        // non-updateble items are rendered as text, so it should be checked if the item
        // exists before checking the requiredness
        if (formItem)
        {
          debug("Checking for requiredness of item "+i+": "+formTableItemName+", formitem="+formItem);
          if (isEmptyDisplayItem(formItem))
          {
            // Show the message to fill in this field
            alert(getMessage("requiredFields")+getScreenPrompt(formItemName));
            type = getItemType(formItem);
            // Give the offending item the focus
            if (type == "radiogroup")
            {
              formItem[0].focus();
            }
            else
            {
              formItem.focus();
            }
            // return false as soon as an empty required item has been found
            debug("returning false in checkForEmptyRequiredRowItems because of item "+formItemName+":"+rownum);
            return false;
          }
        }
      }
    }
    // return true when no empty required items could be found
    return true;
  }
  else
  {
    // return true when there are no requiredRowItems
    return true;
  }
} // checkForEmptyRequiredRowItems


/**
 * checkRequiredRowItems is a function router that either calls
 * checkForEmptyRequiredRowFields() or checkForEmptyRequiredRowItems() for backward
 * compatability. JHeadstart for JDeveloper 9.0.2 and 9.0.3 included the array
 * requiredFields that contains item names without the table prefix. From 9.0.4 the
 * new array requiredRowItems do include the table name.
 * note: UIX & JSP
 * @param formName: the name of the form
 * @param tableName: the table name
 * @param rownum: the index of the row to be checked
 * @return: true when all required items have a value, false otherwise
 */
function checkRequiredRowItems(formName, tableName, rownum)
{
  // TO DO: should refactor this as to interpret tableName as FormBean name in case of JSP
  var returnValue = true;

  if (  requiredFields
     && requiredFields.length > 0
     )
  { // JHeadstart 9.0.2 & 9.0.3
    returnValue = checkForEmptyRequiredRowFields(formName, rownum, tableName);
  }
  else if (  requiredRowItems
          && requiredRowItems.length > 0
          )
  { // JHeadstart 9.0.4
    returnValue = checkForEmptyRequiredRowItems(formName, rownum, tableName);
  }

  debug("returning "+returnValue+" in checkRequiredRowItems");
  return returnValue;
} // checkRequiredRowItems


/**
 * deleteRowChecked checks if the delete checkbox has been checked for a row
 * Helper function for checkForChanges.
 * note: UIX
 * @param tableName: the name of the table
 * @param rownum: the number of the row in the table
 * @return: true when deleteRow has been checked, false otherwise
 */
function deleteRowChecked(tableName, rownum)
{
  debug("In deleteRowChecked, tableName="+tableName+", rownum="+rownum);

  var deleteRowItem = dataForm[tableName+":deleteRow:"+rownum];

  if (  deleteRowItem
     && deleteRowItem.checked
     )
  {
    return true;
  }
  else
  {
    // return false for when there is no item deleteRowItem or when it is not checked
    return false;
  }
} // deleteRowChecked


/**
 * checkForChanges is used when a table is submitted. The table should contain
 * a field 'hasChanges' for each row. This function populates those fields. If
 * a row has not changed then the value is 'N', if it has changes the value is
 * 'Y'. It also calls checkRequiredRowItems() to validate if there are required
 * items without a value.
 * note: UIX
 * @param formName: the name of the form
 * @param tableName: the name of the table to check for changes
 * @return: false when an error occured regarding a required item, true otherwise
 */
function checkForChanges(formName, tableName)
{
  debug("in checkForChanges for table "+tableName);
  // Set dataForm variable so that we don't need to supply the form name
  // to all helper functions of this method.
  dataForm       = document.forms[formName];
  // divide by one causes length to be a number
  var rowcount   = dataForm[tableName+":length"].value / 1;
  // Create array of all table items.
  var tableItems = createTableItemNamesArray(formName, tableName);
  var changeMade = false;
  var initialStatus;
  var item;

  // Loop through all the rows, and determine the record status
  for (var i = 0 ; i < rowcount; i++)
  {
    debug("determining record status for row "+i);
    initialStatus = getChangedStatus(tableName,i);
    changeMade    = false;

    if (initialStatus == "Y")
    {
      // There were errors when trying to save this page.
      debug("There were errors for row "+i);

      if (  hasInsertStatus(tableName,i)
         && (  isEmptyDisplayLine(formName, tableItems, i)
            || deleteRowChecked(tableName, i)
            )
         )
      {
        // This record should be deleted (i.e. not submitted, i.e. no changes)
        changeMade = false;
      }
      else
      {
        // This record needs to be saved, even if the user does not manually
        // change any fields
        changeMade = true;
      }
    }
    else
    {
      // Loop through all items in the row to see if one was changed
      for (var j = 0 ; j < tableItems.length && !changeMade; j++)
      {
        // set hasChanges to 'Y' for row as soon as a changed item has been found
        item = dataForm[tableItems[j]+":"+i];
        // non-updateble items are rendered as text, so it should be checked if the item
        // exists before calling userHasChanged
        if (  item
           && userHasChanged(item)
           )
        {
          debug("Field "+tableItems[j]+" on row "+i+" was changed");
          changeMade = true;
        }
      }
    }

    if (changeMade)
    {
      debug ("### Setting hasChanged on row "+i+" to Y");

      // Write the status to the row
      setChangedStatus(tableName,i,"Y");
    }
    else
    {
      debug ("### Setting hasChanged on row "+i+" to N");

      setChangedStatus(tableName,i,"N");
    }

    // If a change was made to this row (and it does not have the delete checkbox checked)
    // check if the required fields for this row are filled in. If not, abort the loop
    // and return false;
    if (  changeMade
       && !deleteRowChecked(tableName, i)
       && !checkRequiredRowItems(formName, tableName, i)
       )
    {
      debug("checkForChanges returns false for table "+tableName);
      return false;
    }
  }

  debug("checkForChanges returns true for table "+tableName);
  return true;
} // checkForChanges


/**
 * getItemType returns the type of an item. If it is a radio group with more than 1 option,
 * the item is itself an array of radio group items.
 * note: UIX & JSP
 * @param item: the item for which to return the type
 * @return: the type of the item
 */
function getItemType(item)
{
  var type = null;

  if (!item)
  {
    alert("undefined item >"+item+"< in getItemType");
  }
  else
  {
    if (  !item.type
       && !isString(item)
       && item[0].type == "radio"
       ) // the item is an array of radio items
    {
      type = "radiogroup";
    }
    else if (!item.type)
    {
      alert("unrecognized item >"+item+"< in getItemType");
    }
    else
    {
      type = item.type;
    }
  }

  return type;
} // getItemType


/**
 * isEmptyItem checks if an item is null or has an empty value
 * note: UIX & JSP
 * @param item: the item to check
 * @return: true when the item is null or has an empty value
 * (leading and trailing spaces are stripped before checking)
 */
function isEmptyItem(item)
{
  var type;

  if (!item)
  {
    alert("undefined item >"+item+"< in isEmptyItem");
    return null;
  }
  else
  {
    // getItemType will discover when item is an invalid parameter
    type = getItemType(item);

    // an isolated radio button should be ignored, i.e. return false
    if (type == "radio")
    {
      return false;
    }
    // a checkbox should be ignored, i.e. return false, as checked/unchecked are equivalent to
    // something like Y/N
    else if (type == "checkbox")
    {
      return false;
    }
    // the value of a radiogroup depends on one of the items having the property checked
    else if (type == "radiogroup")
    {
      var isEmpty = true;
      for (var i=0; i < item.length && isEmpty; i++)
      {
        isEmpty = !item[i].checked;
      }
      return isEmpty;
    }
    // the value of poplists need to be retrieved in a different way
    else if (type == "select-one" || type == "select-multiple")
    // the value of poplists is depends on the selectedIndex
    {
      // strip all leading and trailing spaces (in Netscape hidden items contain spaces)
      return (stripSpaces(item.options[item.selectedIndex].value).length == 0);
    }
    // in other cases (password, text, textarea) the value depends on the property value
    else if (item.value)
    {
      // strip all leading and trailing spaces (in Netscape hidden items contain spaces)
      return (stripSpaces(item.value).length == 0);
    }
    else // when item has no value, it is considered to be empty
    {
      return true;
    }
  }
} // isEmptyItem


/**
 * back is used for the interview wizard if the user clicks on the back button.
 * note: UIX & JSP (should move to utility library)
 * @param formName: the name of the form
 * @param selectName: TO BE PROVIDED
 */
function back(formName, selectName)
{
  changeListSelection(formName, selectName, "back");
} // back


/**
 * next is used for the interview wizard if the user clicks on the next button.
 * note: UIX & JSP (should move to utility library)
 * @param formName: the name of the form
 * @param selectName: TO BE PROVIDED
 */
function next(formName, selectName)
{
  changeListSelection(formName, selectName, "next");
} // next


/**
 * changeListSelection select the next or previous option from the list of
 * options
 * note: UIX & JSP (should move to utility library)
 * @param formName: the name of the form
 * @param selectName: the list with options
 * @param direction: indicates if the next or previous options should be
 * selected from the list
 */
function changeListSelection(formName, selectName, direction)
{
  var form = document.forms[formName];
  var dif  = 0;
  if (direction == "back")
  {
    dif = -1;
  }
  else if (direction == "next")
  {
    dif = 1;
  }

  var nrOfOptions = form[selectName].options.length;
  var newSelectedIndex = 1;
  for (var i = 0 ; i < nrOfOptions ; i++)
  {
    if (form[selectName].options[i].defaultSelected)
    {
      newSelectedIndex = i + dif;
    }
  }
  if (nrOfOptions > 1)
  {
    form[selectName].selectedIndex = newSelectedIndex;
  }
} // changeListSelection


/**
 * resetValues loops through the global array 'reset' and resets all the fields
 * that correspond to the array items.
 * note: UIX & JSP
 * @param formName: the name of the form
 */
function resetValues(formName)
{
  var form = document.forms[formName];
  var item;
  var itemType;
  if (reset)
  {
    for (var i = 0 ; i < reset.length ; i++)
    {
      item = form[reset[i]];
      if (item)
      {
        itemType = getItemType(item);
        if (  itemType != "radio"
           && itemType != "radiogroup"
           && itemType != "checkbox"
           && itemType != "select-one"
           )
        {
          item.value = "";
        }
      }
    }
  }
} // resetValues


/**
 * resetFormValues loops through all forms, then through all form items of type 'hidden',
 * and if the hidden item did not have a default value, it is reset. The effect of this
 * is therefore similar to reloading a page, but only for hidden fields. This procedure
 * is called by default from the onLoad event of each UIX page, to make sure that (navigation)
 * hidden fields are cleared when the user performs a navigation, uses the backbutton, and
 * performs another navigation. On iExploder the hidden fields are not cleared automatically.
 * on Netscape they are.
 * note: UIX & JSP
 *       this function does not work properly for Netscape 7.0.2 as item.defaultValue for
 *       hidden items will be the same as the changed value
 */
function resetFormValues()
{
  var form;
  var item;
  for (var f = 0 ; f < document.forms.length ; f++)
  {
    form = document.forms[f];
    for (var i = 0 ; i < form.elements.length ; i++)
    {
      item = form.elements[i];
      if (  item.type == "hidden"
         && item.defaultValue == ""
         )
      {
        item.value = "";
      }
    }
  }
} // resetFormValues


/**
 * go should be used if you want to add the global 'field' variable to the keys
 * and values arrays. Go calls go2 to submit the form. This function is useful
 * if you have many links since you can omit the field name and value and the
 * form name.
 * note: UIX & JSP
 * @param keys: TO BE PROVIDED
 * @param values: TO BE PROVIDED
 */
function go(keys, values)
{
  if (field)
  {
    keys[keys.length] = field;
    values[values.length] = field;
  }
  go2(keys,values);
} // go


/**
 * go1 can be used to go to another page, but not without checking for any
 * unsaved changes on the current page.
 * note: UIX & JSP
 * @param link: the link to go to
 */
function go1(link)
{
  var changes = false;
  if (!ignoreChanges)
  {
    changes = hasChanges();
  }

  var confirmed;
  if (  !changes
     || confirm(getMessage("hasChanged"))
     )
  {
    window.location.href = link;
  }
} // go1


/**
 * go2 calls go3 with the global 'form' variable to submit the form. This
 * function is useful if you have many links since you can omit the form name.
 * note: UIX & JSP
 * @param keys: TO BE PROVIDED
 * @param values: TO BE PROVIDED
 */
function go2(keys, values)
{
  go3(form,keys,values);
} // go2


/**
 * go3 is used to go to another page. Instead of using a link with parameters,
 * a form is submitted so that a single point of access can be garanteed.
 * The function first checks for changes on the current page. If there are
 * changes then an alert will show up, and the user can confirm that he wants
 * to continue. Then all the values of the 'values' array are assigned to the
 * keys of the 'keys' array. Finally the form is submitted.
 * note: UIX & JSP
 * @param formName: the name of the form
 * @param keys: TO BE PROVIDED
 * @param values: TO BE PROVIDED
 */
function go3(formName, keys, values)
{
  // reset hidden fields for IE
  resetValues(formName);

  var form    = document.forms[formName];
  var changed = false;
  if (!ignoreChanges)
  {
    changed = hasChanges();
  }

  var confirmed;
  if (changed)
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (!changed || confirmed)
  {
    for (var i = 0 ; i < keys.length ; i++)
    {
      form[keys[i]].value = values[i];
    }

    form.submit()
  }
} // go3


/**
 * go4 is used to go to another page specified by the formAction. It calls go3
 * to do the form submission.
 * note: UIX & JSP
 * @param formName: the name of the form
 * @param formAction: TO BE PROVIDED
 * @param keys: TO BE PROVIDED
 * @param values: TO BE PROVIDED
 */
function go4(formName, formAction, keys, values)
{
  document.forms[formName].action = serviceRoot+formAction;
  go3(formName,keys,values);
} // go4


/**
 * go5 is used to go to another page specified by the formAction. It calls go3
 * to do the form submission but depends on the 'form' variable to hold the
 * name of the form to submit.
 * note: UIX & JSP
 * @param formAction: TO BE PROVIDED
 * @param keys: TO BE PROVIDED
 * @param values: TO BE PROVIDED
 */
function go5(formAction, keys, values)
{
  document.forms[form].action = serviceRoot+formAction;
  go3(form,keys,values);
} // go5


/**
 * treeInfo is a convenience function for trees. This function is called when
 * the user clicks on the '+' or '-'.
 * note: UIX & JSP (should move to utility library)
 * @param formName: the name of the form
 * @param action: TO BE PROVIDED
 * @param id: TO BE PROVIDED
 * @param type: TO BE PROVIDED
 */
function treeInfo(formName, action, id, type)
{
  go3(formName,["treeAction","Id","treeNodeType"],[action,id,type]);
} // treeInfo


/**
 * isString checks if the argument is a string
 * note: UIX & JSP
 * @param 'argument': the argument to check
 * @return: true if the 1st argument is a string, false otherwise
 */
function isString()
{
  if (arguments[0] == null)
  {
    return false;
  }
  else if (typeof arguments[0] == 'string')
  {
    return true;
  }
  else if (  (typeof arguments[0] == 'object')
          && (arguments[0].constructor)
          )
  {
    var criterion = arguments[0].constructor.toString().match(/string/i);
    return (criterion != null);
  }
  else
  {
    return false;
  }
}


/**
 * stripSpaces removes leading and trailing spaces from a string
 * note: UIX & JSP
 * @param aString: the string to strip
 * @return: the stripped string
 */
function stripSpaces(aString)
{
  if (!isString(aString))
  {
    alert("undefined string >"+aString+"< in stripSpaces");
    return null;
  }
  else
  {
    // strip leading spaces
    while (aString.substring(0,1) == " ")
    {
      aString = aString.substring(1);
    }
    // strip trailing spaces
    while (aString.substring(aString.length-1,aString.length) == " ")
    {
      aString = aString.substring(0,aString.length-1);
    }

    return aString
  }
} // stripSpaces


/**
 * isNumber checks if a string (after stripping leading and trailing spaces) represents
 * a numeric value. Empty strings are not considered to be a number.
 * This validator is more strict than the Marlin decimal validator, because it does not
 * accept values like "1a2".
 * Remark: values like "1e3" are accepted, because its interpreted as 1000 (one raised
 * to the power of three)
 * note: UIX & JSP
 * @param aString: the string to check
 * @return: true when aString represents a number, false otherwise
 */
function isNumber(aString)
{
  if (!isString(aString))
  {
    alert("undefined string >"+aString+"< in isNumber");
    return null;
  }
  else
  {
    // empty strings are not numbers
    if (stripSpaces(aString).length == 0)
    {
      return false;
    }

    // if aString evaluates to number zero, return true
    if ((aString*aString) == 0)
    {
      return true;
    }

    // in all other cases, return true if item divided by itself it returns 1
    return (aString/aString == 1);
  }
} // isNumber


/**
 * ignoreField checks if an item should be ignored when checking for changed items.
 * Items to be ignored are expected to start with an underscore (_) or be in the array
 * ignoreChangedFields
 * note: UIX & JSP
 * @param itemName: the item to be checked
 * @return: true when the item should be ignored (when the name start with an
 * underscore or when it is in the array ignoreChangedFields), false otherwise
 */
function ignoreField(itemName)
{
  debug("In ignoreField with itemName: "+itemName);

  itemName = stripRownumFromName(itemName);

  if (itemName.substring(0,1) == "_")
  {
    return true;
  }

  if (ignoreChangedFields)
  {
    for (var j = 0 ; j < ignoreChangedFields.length ; j++)
    {
      if (ignoreChangedFields[j] == itemName)
      {
        return true;
      }
    }
  }

  // return false in all other cases
  return false;

} // ignoreField


/**
 * hasChanges loops over all forms in the document and checks if one or more of the form items
 * have been changed by the user. If there are (unsumitted) changes, the user is asked
 * whether he wants to leave the page, or return to the page and save the changes. By calling
 * ignoreField, some of the items are ignored by hasChanges().
 * note: UIX & JSP
 * @return: true when there are unsubmitted changes (that might be because of an error on
 * the page), false otherwise
 */
function hasChanges()
{
  for (var i = 0 ; i < document.forms.length ; i++)
  {
    var form = document.forms[i];

    debug("checking form >"+form.name+"< for changes");

    // return true when there are errors on the page
    if (form.elements["HasErrors"])
    {
      if (stripSpaces(form.elements["HasErrors"].value) != "" )
      {
        debug("hasChanges returns true because HasErrors has a value");
        return true;
      }
      // else it concerns a select page
    }

    // return true when one of the items in any form have been changed by the user
    for (var k = 0 ; k < form.elements.length ; k++)
    {
      var item = form.elements[k];

      if (  !ignoreField(item.name)
         && userHasChanged(item)
         )
      {
        debug("hasChanges returns true because item >"+item.name+"< has changed");
        return true;
      }
    }
  }

  // in any other case, return false
  debug("hasChanges returns false");
  return false;
} // hasChanges


/**
 * confirmChanges checks if one of the form items has changed by calling hasChanges
 * If so, it asks the user to confirm if he wants to leave the page without saving,
 * or return to the same page. After the user has the opportunity to save the changes.
 * note: UIX & JSP
 * @return; true when there are no changes or when the user choose to leave the page
 * whithout saving them, false otherwise.
 */
function confirmChanges()
{
  var anyFormChanged = hasChanges();

  var confirmed;

  if (anyFormChanged)
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (!anyFormChanged || confirmed)
  {
    return true;
  }
  else
  {
    return false;
  }
} // confirmChanges


/**
 * * valueItemName is optional. If valueItemName is passed to this function
 * the value of the selected item in the lov screen is copied to this field.
 * * displayItemName can get two values:
 * 1. if the valueItemName is used then the display text of the selected item
 *    in the lov screen is copied to this field.
 * 2. if the valueItemName is NOT used then the value of the selected item in
 *    the lov screen is copied to this field.
 * * destination looks like this:
 * '/jheadstart/lovFrame.jsp?title=Find+Employee&#38;class=oracle.empdepdemo.resource.Employee&#38;displayAttr=Ename&#38;valueAttr=Empno'
 * if the valueAttr is not specified then valueAttr = displayAttr.
 * displayAttr should be a columnName which is used to restrict the number of
 * rows.
 * note: UIX
 */
function lovPopup(destination, formName, displayItemName, valueItemName)
{
  destination += "&text="+escape(document.forms[formName][displayItemName].value);
  popup(destination, formName, displayItemName, 440, 500, valueItemName);
} // lovPopup


/**
 * DESCRIPTION TO BE PROVIDED
 * note: UIX
 */
function editorPopup(destination, formName, itemName, maxLength)
{
  destination += (destination.indexOf("?") == -1)
                 ? "?"
                 : "&";
  if (maxLength != null)
  {
    destination += "maxLength=" + maxLength;
  }
  top.editorFieldValue=document.forms[formName][itemName].value;
  popup(destination,formName,itemName,670,320);
} // editorPopup


/**
 * DESCRIPTION TO BE PROVIDED
 * note: UIX
 */
function popup(destination, formName, displayItemName, windowWidth, windowHeight, valueItemName)
{
  var displayField = document.forms[formName][displayItemName];

  var calWindow = openWindow( self
                            , destination
                            , "ModalWindow"
                            , {width:windowWidth, height:windowHeight}
                            , true
                            , "dialog"
                            , returnValue
                            );
  calWindow.displayField = displayField;
  if (valueItemName)
  {
    calWindow.valueField = document.forms[formName][valueItemName];
  }
} // popup


/**
 * returnValue copies the values set by de modal window and assigns them to
 * the form fields.
 * note: UIX
 * @param closingWindow: the modal window
 * @param event: not used
 */
function returnValue(closingWindow, event)
{
  var newDisplayValue = closingWindow.returnDisplayValue;
  var newValueValue   = closingWindow.returnValueValue;
  if (  !closingWindow.valueField
     && newDisplayValue != (void 0)
     )
  {
    // update the contents of the text field
    closingWindow.displayField.value = newValueValue;
  }
  else if (newDisplayValue != (void 0))
  {
    closingWindow.displayField.value = newDisplayValue;
  }

  if (  closingWindow.valueField
     && newValueValue != (void 0)
     )
  {
    // update the contents of the text field
    closingWindow.valueField.value = newValueValue;
  }
} // returnValue


/**
 * This function should be called from a button on a singleSelection
 * table. It check whether there is a selected row and, if so,
 * whether that row is not a new (and therefore not yet inserted) row.
 * To do this, the parameter checkItemName is passed. This should
 * be a field that is always null for new rows (no matter which fields
 * the end user already filled in, and that is never null for queried
 * rows. A perfect candidate would be the readOnly<PrimaryKey> formvalue.
 * To omit this check for new rows (for example if the table does not
 * have any new rows), call checkSelectedRow without the 3rd argument.
 * note: UIX
 */
function checkSelectedRow(formName, tableName, checkItemName)
{
  var form  = document.forms[formName];
  var field = form[tableName+":selected"];
  var j     = -1;

  if (field.length)
  {
    // than more than one radio group item is selectable
    for (var i = 0; i < field.length; i++)
    {
      debug("Checking if row "+i+" is checked");
      if (field[i].checked)
      {
        j = i;
      }
    }

    if (j == -1)
    {
      alert(getMessage("noRowsSelected"));
      return false;
    }
  }
  else
  {
    if (!field.checked)
    {
      alert(getMessage("noRowsSelected"));
      return false;
    }
  }

  if (checkItemName)
  {
    var checkItem = form[tableName+":"+checkItemName+":"+j];
    if (stripSpaces(checkItem.value) == "")
    {
      alert(getMessage("newRowSelected"));
      return false;
    }
  }

  var anyFormChanged = false;
  if (!ignoreChanges)
  {
    anyFormChanged = hasChanges();
  }

  var confirmed;
  if (  !anyFormChanged
     || confirm(getMessage("hasChanged"))
     )
  {
    return true;
  }
  else
  {
    return false;
  }
} // checkSelectedRow


/**
 * changeDisplayPoplist give a warning if displayPoplist has changed, while there are
 * unsaved changes in form. When the user confirms the contextChangedItem is to 'ok'
 * and the form is submitted. Otherwise the the displayPoplist is reset to it's orginal
 * value.
 * Explanation:
 * A display poplist contains values which influence the display of certain fields in the form,
 * for example Hours or Days. If the users gets the warning and chooses Cancel,
 * the display poplist is restored to the value it had when the screen was originally painted
 * (consistent with refreshing the screen whenever the display poplist is changed).
 * note: UIX & JSP (should move to utility library)
 * @param form: name of form
 * @param contextChangedItem: form field that must be set to 'ok' if context has changed,
 * so that jHeadstart knows which process to call next (typically a refresh of the data,
 * using the right display unit)
 * @param displayPoplist: field name of messageChoice field that contains allowable display
 * values (for example hours and days)
 * @return: false, when the user does not confirm the change
 */
function changeDisplayPoplist(form, contextChangedItem, displayPoplist)
{
  var changed   = hasChanges();
  var confirmed = true;
  if (  hasChanges()
     && !ignoreChanges
     )
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (confirmed)
  {
    contextChangedItem.value = "ok";
    form.submit();
  }
  else
  {
    /* go back to original value of displayPoplist */
    for (var i = 0; i < displayPoplist.options.length; i++)
    {
      if (displayPoplist.options[i].defaultSelected)
      {
        displayPoplist.selectedIndex = i;
      }
    }
    return false;
  }
} // changeDisplayPoplist


/**
 * DESCRIPTION TO BE PROVIDED
 * note: UIX
 */
function lovpagePopup(target, formName, displayItemName, valueItemName)
{
  destination = target + "&destination="+document.location;
  popup(destination,formName,displayItemName,800,500,valueItemName);
} // lovpagePopup


/**
 * replace replaces all occurences of a specific text in a string
 * note: UIX & JSP (should move to utility library)
 * @param string: the string for which text needs to be replaced
 * @param text: the text that needs to be replaced
 * @param by: the replacement text
 */
function replace(aString, text, by)
{
  if (  !isString(aString)
     || !isString(text)
     || !isString(by)
     )
  {
    alert("undefined aString >"+aString+", "+text+", "+by+"< in replace");
    return null;
  }
  else
  {
    var strLength = aString.length;
    var txtLength = text.length;

    // return the original string when either the string or the text to be
    // replaced have length 0
    if (  (strLength == 0)
       || (txtLength == 0)
       )
    {
      return aString;
    }

    var i = aString.indexOf(text);
    if (  (!i)
       && (text != aString.substring(0,txtLength))
       )
    {
      return aString;
    }

    if (i == -1)
    {
      return aString;
    }

    var newstr = aString.substring(0,i) + by;
    if ((i + txtLength) < strLength)
    {
      newstr += replace(aString.substring(i+txtLength,strLength),text,by);
    }

    return newstr;
  }
} // replace


/**
 * Translates a string into a proper url
 *
 * note: UIX & JSP (should move to utility library)
 */
function toURLString(text)
{
  text = replace(text,'"',unescape('%22'));
  text = replace(text,'&',unescape('%26'));
  text = replace(text,'<',unescape('%3C'));
  text = replace(text,'>',unescape('%3E'));
  text = replace(text,' ',unescape('%A0'));
  text = replace(text,'',unescape('%A1'));
  text = replace(text,'',unescape('%A2'));
  text = replace(text,'',unescape('%A3'));
  text = replace(text,'',unescape('%A5'));
  text = replace(text,'',unescape('%A6'));
  text = replace(text,'',unescape('%A7'));
  text = replace(text,'',unescape('%A8'));
  text = replace(text,'',unescape('%A9'));
  text = replace(text,'',unescape('%AA'));
  text = replace(text,'',unescape('%AB'));
  text = replace(text,'',unescape('%AC'));
  text = replace(text,'',unescape('%AD'));
  text = replace(text,'',unescape('%AE'));
  text = replace(text,'',unescape('%AF'));
  text = replace(text,'',unescape('%B0'));
  text = replace(text,'',unescape('%B1'));
  text = replace(text,'',unescape('%B2'));
  text = replace(text,'',unescape('%B3'));
  text = replace(text,'',unescape('%B4'));
  text = replace(text,'',unescape('%B5'));
  text = replace(text,'',unescape('%B6'));
  text = replace(text,'',unescape('%B7'));
  text = replace(text,'',unescape('%B8'));
  text = replace(text,'',unescape('%B9'));
  text = replace(text,'',unescape('%BA'));
  text = replace(text,'',unescape('%BB'));
  text = replace(text,'',unescape('%BC'));
  text = replace(text,'',unescape('%BD'));
  text = replace(text,'',unescape('%BE'));
  text = replace(text,'',unescape('%BF'));
  text = replace(text,'',unescape('%D1'));
  text = replace(text,'',unescape('%D2'));

  return text;
} // toURLString


/**
 * submitJspForm submits an html form. Before submitting the form, submitJspForm
 * may check whether there are changes. Whether the function checks for changes,
 * depends in the first place on the parameter isCheckForChangesButton. This parameter
 * is 'true' when the form should be checked for changes.
 * All the buttons will have isCheckForChangesButton is 'true' except the buttons
 * that have as function to update the database using the information in the form, e.g. save
 * and delete buttons.
 * Second of all, whether the function checks for changes, depends on the global variable
 * ignoreChanges. This variable will have been set to 'true', if the form never has to be checked,
 * e.g. a find page form.
 * If the function will check for changes and there are changes, the user will be asked if he
 * really wants the form to be submitted. This is a good thing, because submitting the form will
 * meaning losing your changes.
 * Information about the button pressed should be made part of the form before the form can
 * can be submitted, such that Struts or MVC knows what to do. The name and value parameters
 * contain the name and value of the button pressed. submitJspForm will fill the hidden fields
 * targetCondition (name) and targetConditionValue (value). These hidden fields are, and have to be, part
 * of every form that can be submitted.
 * Before submitting the form, submitJspForm will close all opened windows, such as calendar, lov or
 * editor windows.
 *
 * note: JSP
 * @param formName: The name of the form to be submitted
 * @param name: The name of the button pressed
 * @param value: The value of the button pressed
 * @param isCheckForChangesButton: 'true' if the form should be checked for changes before submitting
 *                                  the form, else 'false'
 */
function submitJspForm(formName, name, value, isCheckForChangesButton)
{
  var form    = document.forms[formName];

  var changed = false;
  if (!ignoreChanges && isCheckForChangesButton)
  {
    changed = hasChanges();
  }

  var confirmed;
  if (changed)
  {
    confirmed = confirm(getMessage("hasChanged"));
  }

  if (!changed || confirmed)
  {
    closeWindows();
    form.targetCondition.value = name;
    form.targetConditionValue.value = value;
    form.submit();
  }
} // submitJspForm


/**
 * closeWindows closes the lovWindow, calendarWindow and editorWindow if they are still open.
 * note: JSP
 */
function closeWindows()
{
  if (lovWindow) lovWindow.close();
  if (calendarWindow) calendarWindow.close();
  if (editorWindow) editorWindow.close();
} // closeWindows

/**
 * Opens an editor window.
 *
 * note: JSP
 * @param destination The url pointing to the editor
 * @param itemName The field in which the result of the editor will be stored
 * @param maxLength The maximum amount of characters allowed in the editor.
 */
function jspEditorPopup(destination, itemName, maxLength)
{
  editorMaxlength = maxLength;

  editorField = getFormElement(itemName);
  editorValue = editorField.value;

  editorWindow = window.open
  (
    destination
  , "editor"
  , "toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=no," +
    "resizable=no,dependent=yes, width=350, height=190, screenX=200, screenY=300"
  );

} // editorPopup


/**
 * Opens an LoV popup. Furthermore this function initializes the variables
 * _baseKeyItem and _baseDescriptorItem as pointer to the items in the calling form
 * so that they can be referred to in the LoV.
 *
 * @param lovName The name of the lov
 * @param baseKeyItemName The name of the key item in the calling form
 * @param baseDescriptorItemNames Comma-delimited list with names of descriptor items in
 *        the calling form
 * @param lovKeyItemAttributeName The name of the attribute to use as key item in the LoV
 * @param lovDescriptorAttributeNames The names of the attributes to use as descriptor items
 *        in the LoV
 * @param searchArguments Array with items containing the search arguments
 *
 * note: JSP
 *
 * At this moment the parameters lovKeyItemName, lovDescriptorItemName, searchItems are
 * not used yet.
 */
function jspLovPopup(lovName, baseKeyItemName, baseDescriptorItemNames, lovKeyAttributeName, lovDescriptorAttributeNames, searchItems)
{
  // String array with the names of the base descriptor items
  var baseDescriptorItemNamesArray = baseDescriptorItemNames.split(",");
  // located base descriptor item
  var baseDescriptorItem = null;

  // find the base key item in the form
  _baseKeyItem = getFormElement(baseKeyItemName);

  // reset the array base descriptor items
  _baseDescriptorItems = new Array();
  for (var i = 0; i < baseDescriptorItemNamesArray.length; i++)
  {
    baseDescriptorItem = getFormElement(baseDescriptorItemNamesArray[i]);
    if (!baseDescriptorItem)
    {
      alert("Could not find descriptor item with name "+baseDescriptorItemNamesArray[i]);
    }
    _baseDescriptorItems[i] = baseDescriptorItem;
  }

  // String array that contains the URI of the LoV
  var uri;
  // pointer to a item containing a search argument
  var searchItem;

  uri = "?targetGroup=Lov" + lovName + "&pageTimeStamp=ignore";

  for (var i = 0; i < searchItems.length; i++)
  {
    searchItem = searchItems[i];
    uri = uri + "&searchItem" + i + "=" + searchItems[i].value;
  }

  lovWindow = window.open
  ( uri
  , "lov"
  , "toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=no," +
    "resizable=yes,dependent=yes, width=450, height=500"
  );
} // lovPopup


/**
 * Updates a page when an option in a lov popup has been selected. This function
 * requires that the variable _baseKeyItem has been initialized with the field to store
 * the display value (e.g. "King") in. If the variable _baseDescriptorItem has been
 * initialized as well, then the base value of the option, e.g. the foreign
 * key value 10, will be stored in this field.
 *
 * note: JSP
 * @param descriptorValue The display value of the selected option
 * @param keyValue The base value of the selected option
 */
function chooseLovValue(descriptorValue, keyValue)
{
//if (_baseKeyItem) alert("_baseKeyItem.name: "+_baseKeyItem.name);
//if (_baseKeyItem) alert("keyValue: "+keyValue);
//alert("_baseDescriptorItems[0].name: "+_baseDescriptorItems[0].name);
//alert("descriptorValue: "+descriptorValue);

  _baseDescriptorItems[0].value = descriptorValue;
  // in case of a find form, _baseKeyItem will be null
  if (_baseKeyItem)
  {
    _baseKeyItem.value = keyValue;
  }
} // chooseLovValue


/**
 * Opens the calendar window. Before opening the window it will call the functions
 * setDateField and setDateFormat in calendar.js such that the calendar knows
 * where to put the result of the calendar window (setDateField) and which date format
 * to use (setDateFormat).
 *
 * note: JSP
 * @param calendarPath The relative URI to locate the calendar html page
 * @param itemName The element where the result of the calendarWindow
 *                 should be put
 * @param format   The date format
 */
function calendarPopup(calendarPath, itemName, format)
{
  setDateField(getFormElement(itemName));
  setDateFormat(format);

  calendarWindow = window.open
  (
    calendarPath
  , "calendar"
  , "toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=no,resizable=no,dependent=yes, width=210, height=230, screenX=200, screenY=300"
  );
} // lovPopup

/**
 * Loops through all the forms and look for the first occurence of an item with name
 * itemName.
 *
 * note: JSP
 * @param itemName The name of the item to look for
 * @result The first occurence of the item with name itemName. null, if no
 *         item with name itemName could be found
 */
function getFormElement(itemName)
{
  // the found item
  var result = null;
  // indicator that the item has been found
  var found = false;
  // form that is searched through
  var searchForm;

  for (var i = 0 ; (i < document.forms.length) && !found; i++)
  {
    searchForm = document.forms[i];
    if (searchForm.elements[itemName])
    {
//alert("searching in form "+searchForm.name);
      result = searchForm.elements[itemName];
      found = true;
//alert("found!");
    }
  }

  return result;
}

/**
 * Loops through all the forms and looks for the first occurence of an element with name
 * elementName. The function returns the value of this element.
 *
 * note: JSP
 * @param elementName The name of the element to look for
 * @result The value of the first occurence of the element with name elementName. null, if no
 *         element with name elementName could be found
 */
function getFormElementValue(elementName)
{
  var result = null;
  var formElement = getFormElement(elementName);
  if (formElement)
  {
    result = formElement.value;
  }
  return result;
}


 // 1.54AMIS1.0 this function will open a new window and navigate it to the destination specified
function popupAndGo(destination)
{
  // do not open the window if no destination is specified
  if ((destination ==null)|| (destination.length==0))
    return;
  // if http:// is not included in the destination, then add it
  if (destination.indexOf("http://") == -1)
  {
    theDestination = "http://"+destination;
  }
  else
    theDestination = destination;
  // open the new window, allow the user to resize it, scroll it and navigate
  // away from it (who are we to restrict the user in this
  var x=window.open(theDestination, "HyperWindow"
           , "width=600,height=300,"
           + "menubar=yes,scrollbars=yes,toolbar=yes,resizable=yes"
           );
} // popupAndGo

